@@ -23,7 +23,6 @@ from neutron._i18n import _, _LE, _LW
 from neutron.agent.l3 import fip_rule_priority_allocator as frpa
 from neutron.agent.l3 import link_local_allocator as lla
 from neutron.agent.l3 import namespaces
-from neutron.agent.l3 import router_info
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import iptables_manager
 from neutron.common import constants
@@ -68,7 +67,6 @@ class FipNamespace(namespaces.Namespace):
         self.local_subnets = lla.LinkLocalAllocator(
             path, constants.DVR_FIP_LL_CIDR)
         self.destroyed = False
-        self._stale_fips_checked = False
 
     @classmethod
     def _get_ns_name(cls, ext_net_id):
@@ -186,10 +184,9 @@ class FipNamespace(namespaces.Namespace):
         # Somewhere in the 3.19 kernel timeframe ip_nonlocal_bind was
         # changed to be a per-namespace attribute.  To be backwards
         # compatible we need to try both if at first we fail.
-        try:
-            ip_lib.set_ip_nonlocal_bind(
+        failed = ip_lib.set_ip_nonlocal_bind(
                 value=1, namespace=self.name, log_fail_as_error=False)
-        except RuntimeError:
+        if failed:
             LOG.debug('DVR: fip namespace (%s) does not support setting '
                       'net.ipv4.ip_nonlocal_bind, trying in root namespace',
                       self.name)
@@ -208,8 +205,7 @@ class FipNamespace(namespaces.Namespace):
     @namespaces.check_ns_existence
     def _delete(self):
         ip_wrapper = ip_lib.IPWrapper(namespace=self.name)
-        for d in ip_wrapper.get_devices(exclude_loopback=True,
-                                        exclude_gre_devices=True):
+        for d in ip_wrapper.get_devices(exclude_loopback=True):
             if d.name.startswith(FIP_2_ROUTER_DEV_PREFIX):
                 # internal link between IRs and FIP NS
                 ip_wrapper.del_veth(d.name)
@@ -295,11 +291,10 @@ class FipNamespace(namespaces.Namespace):
 
         # Now add the filter match rule for the table.
         ip_rule = ip_lib.IPRule(namespace=self.get_name())
-        ip_rule.rule.add(**{'ip': str(fip_2_rtr.ip),
-                            'iif': fip_2_rtr_name,
-                            'table': rt_tbl_index,
-                            'priority': rt_tbl_index,
-                            'from': '0.0.0.0/0'})
+        ip_rule.rule.add(ip=str(fip_2_rtr.ip),
+                         iif=fip_2_rtr_name,
+                         table=rt_tbl_index,
+                         priority=rt_tbl_index)
 
     def _update_gateway_port(self, agent_gateway_port, interface_name):
         if (self.agent_gateway_port and
@@ -335,7 +330,7 @@ class FipNamespace(namespaces.Namespace):
             ip_lib.send_ip_addr_adv_notif(ns_name,
                                           interface_name,
                                           fixed_ip['ip_address'],
-                                          self.agent_conf)
+                                          self.agent_conf.send_arp_for_ha)
 
         for subnet in agent_gateway_port['subnets']:
             gw_ip = subnet.get('gateway_ip')
@@ -399,17 +394,3 @@ class FipNamespace(namespaces.Namespace):
         device = ip_lib.IPDevice(rtr_2_fip_interface, namespace=ri.ns_name)
         if device.exists():
             ri.dist_fip_count = len(ri.get_router_cidrs(device))
-            # On upgrade, there could be stale IP addresses configured, check
-            # and remove them once.
-            # TODO(haleyb): this can go away after a cycle or two
-            if not self._stale_fips_checked:
-                stale_cidrs = (
-                    ip for ip in router_info.RouterInfo.get_router_cidrs(
-                        ri, device)
-                    if common_utils.is_cidr_host(ip))
-                for ip_cidr in stale_cidrs:
-                    LOG.debug("Removing stale floating ip %s from interface "
-                              "%s in namespace %s",
-                              ip_cidr, rtr_2_fip_interface, ri.ns_name)
-                    device.delete_addr_and_conntrack_state(ip_cidr)
-                self._stale_fips_checked = True
