@@ -16,12 +16,11 @@ import os
 import shutil
 import signal
 
-import eventlet
 import netaddr
 from neutron_lib import constants as n_consts
 from oslo_log import log as logging
 
-from neutron._i18n import _, _LE
+from neutron._i18n import _LE
 from neutron.agent.l3 import namespaces
 from neutron.agent.l3 import router_info as router
 from neutron.agent.linux import external_process
@@ -35,11 +34,6 @@ HA_DEV_PREFIX = 'ha-'
 IP_MONITOR_PROCESS_SERVICE = 'ip_monitor'
 SIGTERM_TIMEOUT = 10
 
-# The multiplier is used to compensate execution time of function sending
-# SIGHUP to keepalived process. The constant multiplies ha_vrrp_advert_int
-# config option and the result is the throttle delay.
-THROTTLER_MULTIPLIER = 1.5
-
 
 class HaRouterNamespace(namespaces.RouterNamespace):
     """Namespace for HA router.
@@ -102,13 +96,12 @@ class HaRouter(router.RouterInfo):
         return self.ns_name
 
     def initialize(self, process_monitor):
+        super(HaRouter, self).initialize(process_monitor)
         ha_port = self.router.get(n_consts.HA_INTERFACE_KEY)
         if not ha_port:
-            msg = _("Unable to process HA router %s without "
-                    "HA port") % self.router_id
-            LOG.exception(msg)
-            raise Exception(msg)
-        super(HaRouter, self).initialize(process_monitor)
+            LOG.error(_LE('Unable to process HA router %s without HA port'),
+                      self.router_id)
+            return
 
         self.ha_port = ha_port
         self._init_keepalived_manager(process_monitor)
@@ -122,9 +115,7 @@ class HaRouter(router.RouterInfo):
             keepalived.KeepalivedConf(),
             process_monitor,
             conf_path=self.agent_conf.ha_confs_path,
-            namespace=self.ha_namespace,
-            throttle_restart_value=(
-                self.agent_conf.ha_vrrp_advert_int * THROTTLER_MULTIPLIER))
+            namespace=self.ha_namespace)
 
         config = self.keepalived_manager.config
 
@@ -360,7 +351,7 @@ class HaRouter(router.RouterInfo):
         try:
             common_utils.wait_until_true(lambda: not pm.active,
                                          timeout=SIGTERM_TIMEOUT)
-        except eventlet.timeout.Timeout:
+        except common_utils.WaitTimeout:
             pm.disable(sig=str(int(signal.SIGKILL)))
 
     def update_initial_state(self, callback):
@@ -388,13 +379,8 @@ class HaRouter(router.RouterInfo):
         self._plug_external_gateway(ex_gw_port, interface_name, self.ns_name)
         self._add_gateway_vip(ex_gw_port, interface_name)
         self._disable_ipv6_addressing_on_interface(interface_name)
-
-        # Enable RA and IPv6 forwarding only for master instances. This will
-        # prevent backup routers from sending packets to the upstream switch
-        # and disrupt connections.
-        enable = self.ha_state == 'master'
-        self._configure_ipv6_params_on_gw(ex_gw_port, self.ns_name,
-                                          interface_name, enable)
+        if self.ha_state == 'master':
+            self._enable_ra_on_gw(ex_gw_port, self.ns_name, interface_name)
 
     def external_gateway_updated(self, ex_gw_port, interface_name):
         self._plug_external_gateway(
@@ -417,14 +403,14 @@ class HaRouter(router.RouterInfo):
                                namespace=self.ns_name,
                                prefix=router.EXTERNAL_DEV_PREFIX)
 
-    def delete(self, agent):
+    def delete(self):
         self.destroy_state_change_monitor(self.process_monitor)
         self.disable_keepalived()
         self.ha_network_removed()
-        super(HaRouter, self).delete(agent)
+        super(HaRouter, self).delete()
 
-    def process(self, agent):
-        super(HaRouter, self).process(agent)
+    def process(self):
+        super(HaRouter, self).process()
 
         self.ha_port = self.router.get(n_consts.HA_INTERFACE_KEY)
         if (self.ha_port and
