@@ -28,5 +28,5 @@ class LegacyRouter(router.RouterInfo):
         ip_lib.send_ip_addr_adv_notif(self.ns_name,
                                       interface_name,
                                       fip['floating_ip_address'],
-                                      self.agent_conf)
+                                      self.agent_conf.send_arp_for_ha)
         return lib_constants.FLOATINGIP_STATUS_ACTIVE
