@@ -161,6 +161,15 @@ class PrefixDelegation(object):
             router['gw_interface'] = None
             self.delete_router_pd(router)
 
+    @utils.synchronized("l3-agent-pd")
+    def get_preserve_ips(self, router_id):
+        preserve_ips = []
+        router = self.routers.get(router_id)
+        if router is not None:
+            for subnet_id, pd_info in router['subnets'].items():
+                preserve_ips.append(pd_info.get_bind_lla_with_mask())
+        return preserve_ips
+
     @utils.synchronized("l3-agent-pd")
     def sync_router(self, router_id):
         router = self.routers.get(router_id)
@@ -205,10 +214,10 @@ class PrefixDelegation(object):
                                    lla_with_mask)
 
     def _spawn_lla_thread(self, gw_ifname, ns_name, lla_with_mask):
-            eventlet.spawn_n(self._ensure_lla_task,
-                             gw_ifname,
-                             ns_name,
-                             lla_with_mask)
+        eventlet.spawn_n(self._ensure_lla_task,
+                         gw_ifname,
+                         ns_name,
+                         lla_with_mask)
 
     def _delete_lla(self, router, lla_with_mask):
         if lla_with_mask and router['gw_interface']:
