@@ -28,15 +28,10 @@ def _connection_to_manager_uri(conn_uri):
         return 'p%s:%s' % (proto, addr)
 
 
-def enable_connection_uri(conn_uri, set_timeout=False):
+def enable_connection_uri(conn_uri):
     class OvsdbVsctlContext(object):
         vsctl_timeout = cfg.CONF.ovs_vsctl_timeout
 
     manager_uri = _connection_to_manager_uri(conn_uri)
     api = ovsdb.API.get(OvsdbVsctlContext, 'vsctl')
-    with api.transaction() as txn:
-        txn.add(api.add_manager(manager_uri))
-        if set_timeout:
-            timeout = cfg.CONF.ovs_vsctl_timeout * 1000
-            txn.add(api.db_set('Manager', manager_uri,
-                               ('inactivity_probe', timeout)))
+    api.add_manager(manager_uri).execute(check_error=False, log_errors=True)
