@@ -17,13 +17,18 @@ import os
 import time
 import uuid
 
+from neutron_lib import exceptions
+from oslo_utils import excutils
 from ovs.db import idl
 from ovs import jsonrpc
 from ovs import poller
 from ovs import stream
+import six
+import tenacity
 
 from neutron._i18n import _
-from neutron.common import exceptions
+from neutron.agent.ovsdb import api
+from neutron.agent.ovsdb.native import helpers
 
 
 RowLookup = collections.namedtuple('RowLookup',
@@ -36,6 +41,7 @@ _LOOKUP_TABLE = {
     'IPFIX': RowLookup('Bridge', 'name', 'ipfix'),
     'Mirror': RowLookup('Mirror', 'name', None),
     'NetFlow': RowLookup('Bridge', 'name', 'netflow'),
+    'Open_vSwitch': RowLookup('Open_vSwitch', None, None),
     'QoS': RowLookup('Port', 'name', 'qos'),
     'Queue': RowLookup(None, None, None),
     'sFlow': RowLookup('Bridge', 'name', 'sflow'),
@@ -74,11 +80,11 @@ def row_by_record(idl_, table, record):
         raise RowNotFound(table=table, col='uuid', match=record)
 
     rl = _LOOKUP_TABLE.get(table, RowLookup(table, get_index_column(t), None))
-    # no table means uuid only, no column is just SSL which we don't need
+    # no table means uuid only, no column means lookup table only has one row
     if rl.table is None:
         raise ValueError(_("Table %s can only be queried by UUID") % table)
     if rl.column is None:
-        raise NotImplementedError(_("'.' searches are not implemented"))
+        return t.rows.values()[0]
     row = row_by_value(idl_, rl.table, rl.column, record)
     if rl.uuid_column:
         rows = getattr(row, rl.uuid_column)
@@ -94,7 +100,7 @@ class ExceptionResult(object):
         self.tb = tb
 
 
-def get_schema_helper(connection, schema_name):
+def _get_schema_helper(connection, schema_name):
     err, strm = stream.Stream.open_block(
         stream.Stream.open(connection))
     if err:
@@ -112,6 +118,26 @@ def get_schema_helper(connection, schema_name):
     return idl.SchemaHelper(None, resp.result)
 
 
+def get_schema_helper(connection, schema_name, retry=True):
+    try:
+        return _get_schema_helper(connection, schema_name)
+    except Exception:
+        with excutils.save_and_reraise_exception(reraise=False) as ctx:
+            if not retry:
+                ctx.reraise = True
+            # We may have failed due to set-manager not being called
+            helpers.enable_connection_uri(connection)
+
+            # There is a small window for a race, so retry up to a second
+            @tenacity.retry(wait=tenacity.wait_exponential(multiplier=0.01),
+                            stop=tenacity.stop_after_delay(1),
+                            reraise=True)
+            def do_get_schema_helper():
+                return _get_schema_helper(connection, schema_name)
+
+            return do_get_schema_helper()
+
+
 def wait_for_change(_idl, timeout, seqno=None):
     if seqno is None:
         seqno = _idl.change_seqno
@@ -126,6 +152,11 @@ def wait_for_change(_idl, timeout, seqno=None):
 
 
 def get_column_value(row, col):
+    """Retrieve column value from the given row.
+
+    If column's type is optional, the value will be returned as a single
+    element instead of a list of length 1.
+    """
     if col == '_uuid':
         val = row.uuid
     else:
@@ -135,8 +166,9 @@ def get_column_value(row, col):
     if isinstance(val, list) and len(val):
         if isinstance(val[0], idl.Row):
             val = [v.uuid for v in val]
+        col_type = row._table.columns[col].type
         # ovs-vsctl treats lists of 1 as single results
-        if len(val) == 1:
+        if col_type.is_optional():
             val = val[0]
     return val
 
@@ -147,9 +179,28 @@ def condition_match(row, condition):
     :param row:       An OVSDB Row
     :param condition: A 3-tuple containing (column, operation, match)
     """
-
     col, op, match = condition
     val = get_column_value(row, col)
+
+    # both match and val are primitive types, so type can be used for type
+    # equality here.
+    if type(match) is not type(val):
+        # Types of 'val' and 'match' arguments MUST match in all cases with 2
+        # exceptions:
+        # - 'match' is an empty list and column's type is optional;
+        # - 'value' is an empty and  column's type is optional
+        if (not all([match, val]) and
+                row._table.columns[col].type.is_optional()):
+            # utilize the single elements comparison logic
+            if match == []:
+                match = None
+            elif val == []:
+                val = None
+        else:
+            # no need to process any further
+            raise ValueError(
+                _("Column type and condition operand do not match"))
+
     matched = True
 
     # TODO(twilson) Implement other operators and type comparisons
@@ -167,7 +218,25 @@ def condition_match(row, condition):
             else:
                 raise NotImplementedError()
     elif isinstance(match, list):
-        raise NotImplementedError()
+        # According to rfc7047, lists support '=' and '!='
+        # (both strict and relaxed). Will follow twilson's dict comparison
+        # and implement relaxed version (excludes/includes as per standard)
+        if op == "=":
+            if not all([val, match]):
+                return val == match
+            for elem in set(match):
+                if elem not in val:
+                    matched = False
+                    break
+        elif op == '!=':
+            if not all([val, match]):
+                return val != match
+            for elem in set(match):
+                if elem in val:
+                    matched = False
+                    break
+        else:
+            raise NotImplementedError()
     else:
         if op == '=':
             if val != match:
@@ -190,3 +259,28 @@ def get_index_column(table):
         idx = table.indexes[0]
         if len(idx) == 1:
             return idx[0].name
+
+
+def db_replace_record(obj):
+    """Replace any api.Command objects with their results
+
+    This method should leave obj untouched unless the object contains an
+    api.Command object.
+    """
+    if isinstance(obj, collections.Mapping):
+        for k, v in six.iteritems(obj):
+            if isinstance(v, api.Command):
+                obj[k] = v.result
+    elif (isinstance(obj, collections.Sequence)
+          and not isinstance(obj, six.string_types)):
+        for i, v in enumerate(obj):
+            if isinstance(v, api.Command):
+                try:
+                    obj[i] = v.result
+                except TypeError:
+                    # NOTE(twilson) If someone passes a tuple, then just return
+                    # a tuple with the Commands replaced with their results
+                    return type(obj)(getattr(v, "result", v) for v in obj)
+    elif isinstance(obj, api.Command):
+        obj = obj.result
+    return obj
