@@ -45,15 +45,41 @@ class IPWrapper(object):
 class IPDevice(object):
 
     def __init__(self, name):
-        self.device_name = name
+        self.name = name
+        self.link = IPLink(self)
+
+    def read_ifaddresses(self):
+        try:
+            device_addresses = netifaces.ifaddresses(self.name)
+        except ValueError:
+            LOG.error(_LE("The device does not exist on the system: %s."),
+                      self.name)
+            return
+        except OSError:
+            LOG.error(_LE("Failed to get interface addresses: %s."),
+                      self.name)
+            return
+        return device_addresses
 
     def device_has_ip(self, ip):
-        try:
-            addresses = [ip_addr['addr'] for ip_addr in
-                netifaces.ifaddresses(self.device_name).get(
-                    netifaces.AF_INET, [])]
-            return ip in addresses
-        except OSError:
-            LOG.error(_LE("Failed to get ip addresses for interface: %s."),
-                self.device_name)
+        device_addresses = self.read_ifaddresses()
+        if device_addresses is None:
             return False
+
+        addresses = [ip_addr['addr'] for ip_addr in
+                     device_addresses.get(netifaces.AF_INET, [])]
+        return ip in addresses
+
+
+class IPLink(object):
+
+    def __init__(self, parent):
+        self._parent = parent
+
+    @property
+    def address(self):
+        device_addresses = self._parent.read_ifaddresses()
+        if device_addresses is None:
+            return False
+        return [eth_addr['addr'] for eth_addr in
+                device_addresses.get(netifaces.AF_LINK, [])]
