@@ -1,5 +1,4 @@
 # Copyright 2011 OpenStack Foundation.
-# Copyright 2011 Justin Santa Barbara
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -19,17 +18,20 @@ import collections
 import imp
 import os
 
+from neutron_lib.api import extensions as api_extensions
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_middleware import base
 import routes
 import six
 import webob.dec
 import webob.exc
 
+from neutron._i18n import _, _LE, _LI, _LW
 from neutron.common import exceptions
-import neutron.extensions
-from neutron.i18n import _LE, _LI, _LW
-from neutron import manager
+from neutron import extensions as core_extensions
+from neutron.plugins.common import constants as const
 from neutron.services import provider_configuration
 from neutron import wsgi
 
@@ -37,6 +39,27 @@ from neutron import wsgi
 LOG = logging.getLogger(__name__)
 
 
+EXTENSION_SUPPORTED_CHECK_MAP = {}
+_PLUGIN_AGNOSTIC_EXTENSIONS = set()
+
+
+def register_custom_supported_check(alias, f, plugin_agnostic=False):
+    '''Register a custom function to determine if extension is supported.
+
+    Consequent calls for the same alias replace the registered function.
+
+    :param alias: API extension alias name
+    :param f: custom check function that returns True if extension is supported
+    :param plugin_agnostic: if False, don't require a plugin to claim support
+    with supported_extension_aliases. If True, a plugin must claim the
+    extension is supported.
+    '''
+
+    EXTENSION_SUPPORTED_CHECK_MAP[alias] = f
+    if plugin_agnostic:
+        _PLUGIN_AGNOSTIC_EXTENSIONS.add(alias)
+
+
 @six.add_metaclass(abc.ABCMeta)
 class PluginInterface(object):
 
@@ -62,109 +85,6 @@ class PluginInterface(object):
         return True
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ExtensionDescriptor(object):
-    """Base class that defines the contract for extensions."""
-
-    def get_name(self):
-        """The name of the extension.
-
-        e.g. 'Fox In Socks'
-        """
-        raise NotImplementedError()
-
-    def get_alias(self):
-        """The alias for the extension.
-
-        e.g. 'FOXNSOX'
-        """
-        raise NotImplementedError()
-
-    def get_description(self):
-        """Friendly description for the extension.
-
-        e.g. 'The Fox In Socks Extension'
-        """
-        raise NotImplementedError()
-
-    def get_updated(self):
-        """The timestamp when the extension was last updated.
-
-        e.g. '2011-01-22T13:25:27-06:00'
-        """
-        # NOTE(justinsb): Not sure of the purpose of this is, vs the XML NS
-        raise NotImplementedError()
-
-    def get_resources(self):
-        """List of extensions.ResourceExtension extension objects.
-
-        Resources define new nouns, and are accessible through URLs.
-        """
-        resources = []
-        return resources
-
-    def get_actions(self):
-        """List of extensions.ActionExtension extension objects.
-
-        Actions are verbs callable from the API.
-        """
-        actions = []
-        return actions
-
-    def get_request_extensions(self):
-        """List of extensions.RequestException extension objects.
-
-        Request extensions are used to handle custom request data.
-        """
-        request_exts = []
-        return request_exts
-
-    def get_extended_resources(self, version):
-        """Retrieve extended resources or attributes for core resources.
-
-        Extended attributes are implemented by a core plugin similarly
-        to the attributes defined in the core, and can appear in
-        request and response messages. Their names are scoped with the
-        extension's prefix. The core API version is passed to this
-        function, which must return a
-        map[<resource_name>][<attribute_name>][<attribute_property>]
-        specifying the extended resource attribute properties required
-        by that API version.
-
-        Extension can add resources and their attr definitions too.
-        The returned map can be integrated into RESOURCE_ATTRIBUTE_MAP.
-        """
-        return {}
-
-    def get_plugin_interface(self):
-        """Returns an abstract class which defines contract for the plugin.
-
-        The abstract class should inherit from extensions.PluginInterface,
-        Methods in this abstract class  should be decorated as abstractmethod
-        """
-        return None
-
-    def update_attributes_map(self, extended_attributes,
-                              extension_attrs_map=None):
-        """Update attributes map for this extension.
-
-        This is default method for extending an extension's attributes map.
-        An extension can use this method and supplying its own resource
-        attribute map in extension_attrs_map argument to extend all its
-        attributes that needs to be extended.
-
-        If an extension does not implement update_attributes_map, the method
-        does nothing and just return.
-        """
-        if not extension_attrs_map:
-            return
-
-        for resource, attrs in six.iteritems(extension_attrs_map):
-            extended_attrs = extended_attributes.get(resource)
-            if extended_attrs:
-                attrs.update(extended_attrs)
-
-
 class ActionExtensionController(wsgi.Controller):
 
     def __init__(self, application):
@@ -240,7 +160,7 @@ class ExtensionController(wsgi.Controller):
         raise webob.exc.HTTPNotFound(msg)
 
 
-class ExtensionMiddleware(wsgi.Middleware):
+class ExtensionMiddleware(base.ConfigurableMiddleware):
     """Extensions middleware for WSGI."""
 
     def __init__(self, application,
@@ -271,6 +191,17 @@ class ExtensionMiddleware(wsgi.Middleware):
                     submap.connect(path_prefix + path + "_format",
                                    "%s.:(format)" % path)
 
+            for action, method in resource.collection_methods.items():
+                conditions = dict(method=[method])
+                path = "/%s" % resource.collection
+                with mapper.submapper(controller=resource.controller,
+                                      action=action,
+                                      path_prefix=path_prefix,
+                                      conditions=conditions) as submap:
+                    submap.connect(path_prefix + path, path)
+                    submap.connect(path_prefix + path + "_format",
+                                   "%s.:(format)" % path)
+
             mapper.resource(resource.collection, resource.collection,
                             controller=resource.controller,
                             member=resource.member_actions,
@@ -390,88 +321,100 @@ class ExtensionManager(object):
         resources.append(ResourceExtension('extensions',
                                            ExtensionController(self)))
         for ext in self.extensions.values():
-            try:
-                resources.extend(ext.get_resources())
-            except AttributeError:
-                # NOTE(dprince): Extension aren't required to have resource
-                # extensions
-                pass
+            resources.extend(ext.get_resources())
+        return resources
+
+    def get_pecan_resources(self):
+        """Returns a list of PecanResourceExtension objects."""
+        resources = []
+        for ext in self.extensions.values():
+            resources.extend(ext.get_pecan_resources())
         return resources
 
     def get_actions(self):
         """Returns a list of ActionExtension objects."""
         actions = []
         for ext in self.extensions.values():
-            try:
-                actions.extend(ext.get_actions())
-            except AttributeError:
-                # NOTE(dprince): Extension aren't required to have action
-                # extensions
-                pass
+            actions.extend(ext.get_actions())
         return actions
 
     def get_request_extensions(self):
         """Returns a list of RequestExtension objects."""
         request_exts = []
         for ext in self.extensions.values():
-            try:
-                request_exts.extend(ext.get_request_extensions())
-            except AttributeError:
-                # NOTE(dprince): Extension aren't required to have request
-                # extensions
-                pass
+            request_exts.extend(ext.get_request_extensions())
         return request_exts
 
     def extend_resources(self, version, attr_map):
         """Extend resources with additional resources or attributes.
 
-        :param: attr_map, the existing mapping from resource name to
+        :param attr_map: the existing mapping from resource name to
         attrs definition.
 
         After this function, we will extend the attr_map if an extension
         wants to extend this map.
         """
-        update_exts = []
-        processed_exts = set()
+        processed_exts = {}
         exts_to_process = self.extensions.copy()
+        check_optionals = True
         # Iterate until there are unprocessed extensions or if no progress
         # is made in a whole iteration
         while exts_to_process:
             processed_ext_count = len(processed_exts)
             for ext_name, ext in list(exts_to_process.items()):
-                if not hasattr(ext, 'get_extended_resources'):
-                    del exts_to_process[ext_name]
+                # Process extension only if all required extensions
+                # have been processed already
+                required_exts_set = set(ext.get_required_extensions())
+                if required_exts_set - set(processed_exts):
                     continue
-                if hasattr(ext, 'update_attributes_map'):
-                    update_exts.append(ext)
-                if hasattr(ext, 'get_required_extensions'):
-                    # Process extension only if all required extensions
-                    # have been processed already
-                    required_exts_set = set(ext.get_required_extensions())
-                    if required_exts_set - processed_exts:
-                        continue
-                try:
-                    extended_attrs = ext.get_extended_resources(version)
-                    for res, resource_attrs in six.iteritems(extended_attrs):
-                        attr_map.setdefault(res, {}).update(resource_attrs)
-                except AttributeError:
-                    LOG.exception(_LE("Error fetching extended attributes for "
-                                      "extension '%s'"), ext.get_name())
-                processed_exts.add(ext_name)
+                optional_exts_set = set(ext.get_optional_extensions())
+                if check_optionals and optional_exts_set - set(processed_exts):
+                    continue
+                extended_attrs = ext.get_extended_resources(version)
+                for res, resource_attrs in six.iteritems(extended_attrs):
+                    attr_map.setdefault(res, {}).update(resource_attrs)
+                processed_exts[ext_name] = ext
                 del exts_to_process[ext_name]
             if len(processed_exts) == processed_ext_count:
+                # if we hit here, it means there are unsatisfied
+                # dependencies. try again without optionals since optionals
+                # are only necessary to set order if they are present.
+                if check_optionals:
+                    check_optionals = False
+                    continue
                 # Exit loop as no progress was made
                 break
         if exts_to_process:
-            # NOTE(salv-orlando): Consider whether this error should be fatal
-            LOG.error(_LE("It was impossible to process the following "
-                          "extensions: %s because of missing requirements."),
-                      ','.join(exts_to_process.keys()))
-
+            unloadable_extensions = set(exts_to_process.keys())
+            LOG.error(_LE("Unable to process extensions (%s) because "
+                          "the configured plugins do not satisfy "
+                          "their requirements. Some features will not "
+                          "work as expected."),
+                      ', '.join(unloadable_extensions))
+            self._check_faulty_extensions(unloadable_extensions)
         # Extending extensions' attributes map.
-        for ext in update_exts:
+        for ext in processed_exts.values():
             ext.update_attributes_map(attr_map)
 
+    def _check_faulty_extensions(self, faulty_extensions):
+        """Raise for non-default faulty extensions.
+
+        Gracefully fail for defective default extensions, which will be
+        removed from the list of loaded extensions.
+        """
+        default_extensions = set(const.DEFAULT_SERVICE_PLUGINS.values())
+        if not faulty_extensions <= default_extensions:
+            raise exceptions.ExtensionsNotFound(
+                extensions=list(faulty_extensions))
+        else:
+            # Remove the faulty extensions so that they do not show during
+            # ext-list
+            for ext in faulty_extensions:
+                try:
+                    del self.extensions[ext]
+                except KeyError:
+                    pass
+
     def _check_extension(self, extension):
         """Checks for required methods in extension objects."""
         try:
@@ -479,11 +422,10 @@ class ExtensionManager(object):
             LOG.debug('Ext alias: %s', extension.get_alias())
             LOG.debug('Ext description: %s', extension.get_description())
             LOG.debug('Ext updated: %s', extension.get_updated())
-        except AttributeError as ex:
-            LOG.exception(_LE("Exception loading extension: %s"),
-                          six.text_type(ex))
+        except AttributeError:
+            LOG.exception(_LE("Exception loading extension"))
             return False
-        return True
+        return isinstance(extension, api_extensions.ExtensionDescriptor)
 
     def _load_all_extensions(self):
         """Load extensions from the configured path.
@@ -516,17 +458,17 @@ class ExtensionManager(object):
                     ext_name = mod_name[0].upper() + mod_name[1:]
                     new_ext_class = getattr(mod, ext_name, None)
                     if not new_ext_class:
-                        LOG.warn(_LW('Did not find expected name '
-                                     '"%(ext_name)s" in %(file)s'),
-                                 {'ext_name': ext_name,
-                                  'file': ext_path})
+                        LOG.warning(_LW('Did not find expected name '
+                                        '"%(ext_name)s" in %(file)s'),
+                                    {'ext_name': ext_name,
+                                     'file': ext_path})
                         continue
                     new_ext = new_ext_class()
                     self.add_extension(new_ext)
             except Exception as exception:
-                LOG.warn(_LW("Extension file %(f)s wasn't loaded due to "
-                             "%(exception)s"),
-                         {'f': f, 'exception': exception})
+                LOG.warning(_LW("Extension file %(f)s wasn't loaded due to "
+                                "%(exception)s"),
+                            {'f': f, 'exception': exception})
 
     def add_extension(self, ext):
         # Do nothing if the extension doesn't check out
@@ -554,52 +496,69 @@ class PluginAwareExtensionManager(ExtensionManager):
         """Check if an extension is supported by any plugin."""
         extension_is_valid = super(PluginAwareExtensionManager,
                                    self)._check_extension(extension)
-        return (extension_is_valid and
-                self._plugins_support(extension) and
+        if not extension_is_valid:
+            return False
+
+        alias = extension.get_alias()
+        if alias in EXTENSION_SUPPORTED_CHECK_MAP:
+            return EXTENSION_SUPPORTED_CHECK_MAP[alias]()
+
+        return (self._plugins_support(extension) and
                 self._plugins_implement_interface(extension))
 
     def _plugins_support(self, extension):
         alias = extension.get_alias()
         supports_extension = alias in self.get_supported_extension_aliases()
         if not supports_extension:
-            LOG.warn(_LW("Extension %s not supported by any of loaded "
+            LOG.info(_LI("Extension %s not supported by any of loaded "
                          "plugins"),
-                     alias)
+                    alias)
         return supports_extension
 
     def _plugins_implement_interface(self, extension):
-        if(not hasattr(extension, "get_plugin_interface") or
-           extension.get_plugin_interface() is None):
+        if extension.get_plugin_interface() is None:
             return True
         for plugin in self.plugins.values():
             if isinstance(plugin, extension.get_plugin_interface()):
                 return True
-        LOG.warn(_LW("Loaded plugins do not implement extension %s interface"),
-                 extension.get_alias())
+        LOG.warning(_LW("Loaded plugins do not implement extension "
+                        "%s interface"),
+                    extension.get_alias())
         return False
 
     @classmethod
     def get_instance(cls):
         if cls._instance is None:
-            service_plugins = manager.NeutronManager.get_service_plugins()
+            service_plugins = directory.get_plugins()
             cls._instance = cls(get_extensions_path(service_plugins),
                                 service_plugins)
         return cls._instance
 
+    def get_plugin_supported_extension_aliases(self, plugin):
+        """Return extension aliases supported by a given plugin"""
+        aliases = set()
+        # we also check all classes that the plugins inherit to see if they
+        # directly provide support for an extension
+        for item in [plugin] + plugin.__class__.mro():
+            try:
+                aliases |= set(
+                    getattr(item, "supported_extension_aliases", []))
+            except TypeError:
+                # we land here if a class has a @property decorator for
+                # supported extension aliases. They only work on objects.
+                pass
+        return aliases
+
     def get_supported_extension_aliases(self):
         """Gets extension aliases supported by all plugins."""
         aliases = set()
         for plugin in self.plugins.values():
-            # we also check all classes that the plugins inherit to see if they
-            # directly provide support for an extension
-            for item in [plugin] + plugin.__class__.mro():
-                try:
-                    aliases |= set(
-                        getattr(item, "supported_extension_aliases", []))
-                except TypeError:
-                    # we land here if a class has an @property decorator for
-                    # supported extension aliases. They only work on objects.
-                    pass
+            aliases |= self.get_plugin_supported_extension_aliases(plugin)
+        aliases |= {
+            alias
+            for alias, func in EXTENSION_SUPPORTED_CHECK_MAP.items()
+            if func()
+        }
         return aliases
 
     @classmethod
@@ -610,6 +569,7 @@ class PluginAwareExtensionManager(ExtensionManager):
         """Check if an extension supported by a plugin has been loaded."""
         plugin_extensions = self.get_supported_extension_aliases()
         missing_aliases = plugin_extensions - set(self.extensions)
+        missing_aliases -= _PLUGIN_AGNOSTIC_EXTENSIONS
         if missing_aliases:
             raise exceptions.ExtensionsNotFound(
                 extensions=list(missing_aliases))
@@ -642,11 +602,17 @@ class ResourceExtension(object):
     """Add top level resources to the OpenStack API in Neutron."""
 
     def __init__(self, collection, controller, parent=None, path_prefix="",
-                 collection_actions={}, member_actions={}, attr_map={}):
+                 collection_actions=None, member_actions=None, attr_map=None,
+                 collection_methods=None):
+        collection_actions = collection_actions or {}
+        collection_methods = collection_methods or {}
+        member_actions = member_actions or {}
+        attr_map = attr_map or {}
         self.collection = collection
         self.controller = controller
         self.parent = parent
         self.collection_actions = collection_actions
+        self.collection_methods = collection_methods
         self.member_actions = member_actions
         self.path_prefix = path_prefix
         self.attr_map = attr_map
@@ -658,7 +624,7 @@ def get_extensions_path(service_plugins=None):
     paths = collections.OrderedDict()
 
     # Add Neutron core extensions
-    paths[neutron.extensions.__path__[0]] = 1
+    paths[core_extensions.__path__[0]] = 1
     if service_plugins:
         # Add Neutron *-aas extensions
         for plugin in service_plugins.values():
