@@ -15,25 +15,16 @@ import copy
 import pprint
 import time
 
-from neutron_lib import constants
+from neutron_lib.plugins import directory
 from oslo_log import log as logging
 from oslo_utils import importutils
 
 from neutron.api.rpc.callbacks import exceptions
-from neutron import manager
 
 LOG = logging.getLogger(__name__)
 
 VERSIONS_TTL = 60
 
-# This is the list of agents that started using this rpc push/pull mechanism
-# for versioned objects, but at that time stable/liberty, they were not
-# reporting versions, so we need to assume they need QosPolicy 1.0
-#TODO(mangelajo): Remove this logic in Newton, since those agents will be
-#                 already reporting From N to O
-NON_REPORTING_AGENT_TYPES = [constants.AGENT_TYPE_OVS,
-                             constants.AGENT_TYPE_NIC_SWITCH]
-
 
 # NOTE(mangelajo): if we import this globally we end up with a (very
 #                  long) circular dependency, this can be fixed if we
@@ -157,14 +148,6 @@ class ResourceConsumerTracker(object):
 
     def _handle_no_set_versions(self, consumer):
         """Handle consumers reporting no versions."""
-        if isinstance(consumer, AgentConsumer):
-            if consumer.agent_type in NON_REPORTING_AGENT_TYPES:
-                resources = _import_resources()
-                self._versions_by_consumer[consumer] = {
-                    resources.QOS_POLICY: '1.0'}
-                self._versions[resources.QOS_POLICY].add('1.0')
-                return
-
         if self._versions_by_consumer[consumer]:
             self._needs_recalculation = True
         self._versions_by_consumer[consumer] = {}
@@ -213,7 +196,7 @@ class CachedResourceConsumerTracker(object):
 
     def _update_consumer_versions(self):
         new_tracker = ResourceConsumerTracker()
-        neutron_plugin = manager.NeutronManager.get_plugin()
+        neutron_plugin = directory.get_plugin()
         agents_db = _import_agents_db()
         # If you use RPC callbacks, your plugin needs to implement
         # AgentsDbMixin so that we know which resource versions your
