@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.plugins import directory
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 import oslo_messaging
@@ -20,7 +21,6 @@ import oslo_messaging
 from neutron.common import constants
 from neutron.common import rpc as n_rpc
 from neutron.common import topics
-from neutron import manager
 
 LOG = logging.getLogger(__name__)
 
@@ -53,6 +53,8 @@ class DVRServerRpcApi(object):
 
     @log_helpers.log_method_call
     def get_ports_on_host_by_subnet(self, context, host, subnet):
+        """Get DVR serviced ports on given host and subnet."""
+
         cctxt = self.client.prepare()
         return cctxt.call(context, 'get_ports_on_host_by_subnet',
                           host=host, subnet=subnet)
@@ -83,7 +85,7 @@ class DVRServerRpcCallback(object):
     @property
     def plugin(self):
         if not getattr(self, '_plugin', None):
-            self._plugin = manager.NeutronManager.get_plugin()
+            self._plugin = directory.get_plugin()
         return self._plugin
 
     def get_dvr_mac_address_list(self, context):
@@ -95,6 +97,8 @@ class DVRServerRpcCallback(object):
         return self.plugin.get_dvr_mac_address_by_host(context, host)
 
     def get_ports_on_host_by_subnet(self, context, **kwargs):
+        """Get DVR serviced ports for given host and subnet."""
+
         host = kwargs.get('host')
         subnet = kwargs.get('subnet')
         LOG.debug("DVR Agent requests list of VM ports on host %s", host)
