@@ -17,7 +17,7 @@ import collections
 import copy
 
 import netaddr
-from oslo_config import cfg
+from neutron_lib import exceptions
 from oslo_log import log as logging
 from oslo_policy import policy as oslo_policy
 from oslo_utils import excutils
@@ -26,11 +26,12 @@ import webob.exc
 
 from neutron._i18n import _, _LE, _LI
 from neutron.api import api_common
-from neutron.api.rpc.agentnotifiers import dhcp_rpc_agent_api
 from neutron.api.v2 import attributes
 from neutron.api.v2 import resource as wsgi_resource
-from neutron.common import constants as const
-from neutron.common import exceptions
+from neutron.callbacks import events
+from neutron.callbacks import registry
+from neutron.common import constants as n_const
+from neutron.common import exceptions as n_exc
 from neutron.common import rpc as n_rpc
 from neutron.db import api as db_api
 from neutron import policy
@@ -58,6 +59,47 @@ class Controller(object):
     UPDATE = 'update'
     DELETE = 'delete'
 
+    @property
+    def plugin(self):
+        return self._plugin
+
+    @property
+    def resource(self):
+        return self._resource
+
+    @property
+    def attr_info(self):
+        return self._attr_info
+
+    @property
+    def member_actions(self):
+        return self._member_actions
+
+    @property
+    def allow_pagination(self):
+        return self._allow_pagination
+
+    @property
+    def allow_sorting(self):
+        return self._allow_sorting
+
+    def _init_policy_attrs(self):
+        """Create the list of attributes required by policy.
+
+        If the attribute map contains a tenant_id policy, then include
+        project_id to bring the resource into the brave new world.
+
+        :return: sorted list of attributes required by policy
+
+        """
+        policy_attrs = {name for (name, info) in self._attr_info.items()
+                        if info.get('required_by_policy')}
+        if 'tenant_id' in policy_attrs:
+            policy_attrs.add('project_id')
+
+        # Could use list(), but sorted() makes testing easier.
+        return sorted(policy_attrs)
+
     def __init__(self, plugin, collection, resource, attr_info,
                  allow_bulk=False, member_actions=None, parent=None,
                  allow_pagination=False, allow_sorting=False):
@@ -73,18 +115,8 @@ class Controller(object):
         self._native_bulk = self._is_native_bulk_supported()
         self._native_pagination = self._is_native_pagination_supported()
         self._native_sorting = self._is_native_sorting_supported()
-        self._policy_attrs = [name for (name, info) in self._attr_info.items()
-                              if info.get('required_by_policy')]
+        self._policy_attrs = self._init_policy_attrs()
         self._notifier = n_rpc.get_notifier('network')
-        # use plugin's dhcp notifier, if this is already instantiated
-        agent_notifiers = getattr(plugin, 'agent_notifiers', {})
-        self._dhcp_agent_notifier = (
-            agent_notifiers.get(const.AGENT_TYPE_DHCP) or
-            dhcp_rpc_agent_api.DhcpAgentNotifyAPI()
-        )
-        if cfg.CONF.notify_nova_on_port_data_changes:
-            from neutron.notifiers import nova
-            self._nova_notifier = nova.Notifier()
         self._member_actions = member_actions
         self._primary_key = self._get_primary_key()
         if self._allow_pagination and self._native_pagination:
@@ -97,7 +129,7 @@ class Controller(object):
                 LOG.info(_LI("Allow sorting is enabled because native "
                              "pagination requires native sorting"))
                 self._allow_sorting = True
-
+        self.parent = parent
         if parent:
             self._parent_id_name = '%s_id' % parent['member_name']
             parent_part = '_%s' % parent['member_name']
@@ -124,14 +156,10 @@ class Controller(object):
         return getattr(self._plugin, native_bulk_attr_name, False)
 
     def _is_native_pagination_supported(self):
-        native_pagination_attr_name = ("_%s__native_pagination_support"
-                                       % self._plugin.__class__.__name__)
-        return getattr(self._plugin, native_pagination_attr_name, False)
+        return api_common.is_native_pagination_supported(self._plugin)
 
     def _is_native_sorting_supported(self):
-        native_sorting_attr_name = ("_%s__native_sorting_support"
-                                    % self._plugin.__class__.__name__)
-        return getattr(self._plugin, native_sorting_attr_name, False)
+        return api_common.is_native_sorting_supported(self._plugin)
 
     def _exclude_attributes_by_policy(self, context, data):
         """Identifies attributes to exclude according to authZ policies.
@@ -142,6 +170,13 @@ class Controller(object):
         """
         attributes_to_exclude = []
         for attr_name in data.keys():
+            # TODO(amotoki): At now, all attribute maps have tenant_id and
+            # determine excluded attributes based on tenant_id.
+            # We need to migrate tenant_id to project_id later
+            # as attr_info is referred to in various places and we need
+            # to check all logis carefully.
+            if attr_name == 'project_id':
+                continue
             attr_data = self._attr_info.get(attr_name)
             if attr_data and attr_data['is_visible']:
                 if policy.check(
@@ -155,6 +190,12 @@ class Controller(object):
             # if the code reaches this point then either the policy check
             # failed or the attribute was not visible in the first place
             attributes_to_exclude.append(attr_name)
+            # TODO(amotoki): As mentioned in the above TODO,
+            # we treat project_id and tenant_id equivalently.
+            # This should be migrated to project_id in Ocata.
+            if attr_name == 'tenant_id':
+                attributes_to_exclude.append('project_id')
+
         return attributes_to_exclude
 
     def _view(self, context, data, fields_to_strip=None):
@@ -169,9 +210,9 @@ class Controller(object):
         """
         fields_to_strip = ((fields_to_strip or []) +
                            self._exclude_attributes_by_policy(context, data))
-        return self._filter_attributes(context, data, fields_to_strip)
+        return self._filter_attributes(data, fields_to_strip)
 
-    def _filter_attributes(self, context, data, fields_to_strip=None):
+    def _filter_attributes(self, data, fields_to_strip=None):
         if not fields_to_strip:
             return data
         return dict(item for item in six.iteritems(data)
@@ -204,7 +245,7 @@ class Controller(object):
                 except oslo_policy.PolicyNotAuthorized:
                     msg = _('The resource could not be found.')
                     raise webob.exc.HTTPNotFound(msg)
-                body = copy.deepcopy(kwargs.pop('body', None))
+                body = kwargs.pop('body', None)
                 # Explicit comparison with None to distinguish from {}
                 if body is not None:
                     arg_list.append(body)
@@ -288,8 +329,7 @@ class Controller(object):
             fields_to_strip += self._exclude_attributes_by_policy(
                 request.context, obj_list[0])
         collection = {self._collection:
-                      [self._filter_attributes(
-                          request.context, obj,
+                      [self._filter_attributes(obj,
                           fields_to_strip=fields_to_strip)
                        for obj in obj_list]}
         pagination_links = pagination_helper.get_links(obj_list)
@@ -319,19 +359,6 @@ class Controller(object):
                            pluralized=self._collection)
         return obj
 
-    def _send_dhcp_notification(self, context, data, methodname):
-        if cfg.CONF.dhcp_agent_notification:
-            if self._collection in data:
-                for body in data[self._collection]:
-                    item = {self._resource: body}
-                    self._dhcp_agent_notifier.notify(context, item, methodname)
-            else:
-                self._dhcp_agent_notifier.notify(context, data, methodname)
-
-    def _send_nova_notification(self, action, orig, returned):
-        if hasattr(self, '_nova_notifier'):
-            self._nova_notifier.send_network_change(action, orig, returned)
-
     @db_api.retry_db_errors
     def index(self, request, **kwargs):
         """Returns a list of the requested entity."""
@@ -376,7 +403,6 @@ class Controller(object):
                 fields_to_strip = self._exclude_attributes_by_policy(
                     request.context, item)
                 objs.append(self._filter_attributes(
-                    request.context,
                     obj_creator(request.context, **kwargs),
                     fields_to_strip=fields_to_strip))
             return objs
@@ -414,7 +440,7 @@ class Controller(object):
         """Creates a new instance of the requested entity."""
         parent_id = kwargs.get(self._parent_id_name)
         body = Controller.prepare_request_body(request.context,
-                                               copy.deepcopy(body), True,
+                                               body, True,
                                                self._resource, self._attr_info,
                                                allow_bulk=self._allow_bulk)
         action = self._plugin_handlers[self.CREATE]
@@ -453,9 +479,9 @@ class Controller(object):
                     {self._resource: delta},
                     self._plugin)
                 reservations.append(reservation)
-        except exceptions.QuotaResourceUnknown as e:
-                # We don't want to quota this resource
-                LOG.debug(e)
+        except n_exc.QuotaResourceUnknown as e:
+            # We don't want to quota this resource
+            LOG.debug(e)
 
         def notify(create_result):
             # Ensure usage trackers for all resources affected by this API
@@ -471,9 +497,11 @@ class Controller(object):
             self._notifier.info(request.context,
                                 notifier_method,
                                 create_result)
-            self._send_dhcp_notification(request.context,
-                                         create_result,
-                                         notifier_method)
+            registry.notify(self._resource, events.BEFORE_RESPONSE, self,
+                            context=request.context, data=create_result,
+                            method_name=notifier_method,
+                            collection=self._collection,
+                            action=action, original={})
             return create_result
 
         def do_create(body, bulk=False, emulated=False):
@@ -510,7 +538,7 @@ class Controller(object):
             fields_to_strip = self._exclude_attributes_by_policy(
                 request.context, objs[0])
             return notify({self._collection: [self._filter_attributes(
-                request.context, obj, fields_to_strip=fields_to_strip)
+                obj, fields_to_strip=fields_to_strip)
                 for obj in objs]})
         else:
             if self._collection in body:
@@ -519,8 +547,6 @@ class Controller(object):
                 return notify({self._collection: objs})
             else:
                 obj = do_create(body)
-                self._send_nova_notification(action, {},
-                                             {self._resource: obj})
                 return notify({self._resource: self._view(request.context,
                                                           obj)})
 
@@ -559,14 +585,16 @@ class Controller(object):
         # usage trackers as dirty
         resource_registry.set_resources_dirty(request.context)
         notifier_method = self._resource + '.delete.end'
+        result = {self._resource: self._view(request.context, obj)}
+        notifier_payload = {self._resource + '_id': id}
+        notifier_payload.update(result)
         self._notifier.info(request.context,
                             notifier_method,
-                            {self._resource + '_id': id})
-        result = {self._resource: self._view(request.context, obj)}
-        self._send_nova_notification(action, {}, result)
-        self._send_dhcp_notification(request.context,
-                                     result,
-                                     notifier_method)
+                            notifier_payload)
+        registry.notify(self._resource, events.BEFORE_RESPONSE, self,
+                        context=request.context, data=result,
+                        method_name=notifier_method, action=action,
+                        original={})
 
     def update(self, request, id, body=None, **kwargs):
         """Updates the specified entity's attributes."""
@@ -584,7 +612,7 @@ class Controller(object):
     @db_api.retry_db_errors
     def _update(self, request, id, body, **kwargs):
         body = Controller.prepare_request_body(request.context,
-                                               copy.deepcopy(body), False,
+                                               body, False,
                                                self._resource, self._attr_info,
                                                allow_bulk=self._allow_bulk)
         action = self._plugin_handlers[self.UPDATE]
@@ -605,7 +633,7 @@ class Controller(object):
         # Make a list of attributes to be updated to inform the policy engine
         # which attributes are set explicitly so that it can distinguish them
         # from the ones that are set to their default values.
-        orig_obj[const.ATTRIBUTES_TO_UPDATE] = body[self._resource].keys()
+        orig_obj[n_const.ATTRIBUTES_TO_UPDATE] = body[self._resource].keys()
         try:
             policy.enforce(request.context,
                            action,
@@ -637,10 +665,10 @@ class Controller(object):
         result = {self._resource: self._view(request.context, obj)}
         notifier_method = self._resource + '.update.end'
         self._notifier.info(request.context, notifier_method, result)
-        self._send_dhcp_notification(request.context,
-                                     result,
-                                     notifier_method)
-        self._send_nova_notification(action, orig_object_copy, result)
+        registry.notify(self._resource, events.BEFORE_RESPONSE, self,
+                        context=request.context, data=result,
+                        method_name=notifier_method, action=action,
+                        original=orig_object_copy)
         return result
 
     @staticmethod
