@@ -26,9 +26,9 @@ from oslo_utils import importutils
 from neutron._i18n import _LE, _LW
 from neutron.agent.common import config as agent_config
 from neutron.agent.common import ovs_lib
-from neutron.agent.l3 import agent as l3_agent
-from neutron.agent.l3 import dvr
 from neutron.agent.l3 import dvr_fip_ns
+from neutron.agent.l3 import dvr_snat_ns
+from neutron.agent.l3 import namespaces
 from neutron.agent.linux import dhcp
 from neutron.agent.linux import external_process
 from neutron.agent.linux import interface
@@ -43,7 +43,8 @@ LOG = logging.getLogger(__name__)
 LB_NS_PREFIX = 'qlbaas-'
 NS_PREFIXES = {
     'dhcp': [dhcp.NS_PREFIX],
-    'l3': [l3_agent.NS_PREFIX, dvr.SNAT_NS_PREFIX, dvr_fip_ns.FIP_NS_PREFIX],
+    'l3': [namespaces.NS_PREFIX, dvr_snat_ns.SNAT_NS_PREFIX,
+           dvr_fip_ns.FIP_NS_PREFIX],
     'lbaas': [LB_NS_PREFIX],
 }
 SIGTERM_WAITTIME = 10
@@ -247,8 +248,7 @@ def destroy_namespace(conf, namespace, force=False):
                     # the error and continue with the cleanup
                     LOG.error(_LE('Not all processes were killed in %s'),
                               namespace)
-                for device in ip.get_devices(exclude_loopback=True,
-                                             exclude_gre_devices=True):
+                for device in ip.get_devices(exclude_loopback=True):
                     unplug_device(conf, device)
 
         ip.garbage_collect_namespace()
