@@ -229,12 +229,16 @@ def dhcp_release6_supported():
 
 
 def bridge_firewalling_enabled():
-    cmd = ['sysctl', '-N', 'net.bridge']
-    entries = agent_utils.execute(cmd, run_as_root=True)
     for proto in ('arp', 'ip', 'ip6'):
         knob = 'net.bridge.bridge-nf-call-%stables' % proto
-        if knob not in entries:
-            LOG.debug("sysctl value %s not present on this system.", knob)
+        cmd = ['sysctl', '-b', knob]
+        try:
+            out = agent_utils.execute(cmd)
+        except (OSError, RuntimeError, IndexError, ValueError) as e:
+            LOG.debug("Exception while extracting %(knob)s. "
+                      "Exception: %(e)s", {'knob': knob, 'e': e})
+            return False
+        if out == '0':
             return False
     return True
 
@@ -249,7 +253,7 @@ class KeepalivedIPv6Test(object):
         self.config = None
         self.config_path = None
         self.nsname = "keepalivedtest-" + uuidutils.generate_uuid()
-        self.pm = None
+        self.pm = external_process.ProcessMonitor(cfg.CONF, 'router')
         self.orig_interval = cfg.CONF.AGENT.check_child_processes_interval
 
     def configure(self):
@@ -274,7 +278,6 @@ class KeepalivedIPv6Test(object):
     def start_keepalived_process(self):
         # Disable process monitoring for Keepalived process.
         cfg.CONF.set_override('check_child_processes_interval', 0, 'AGENT')
-        self.pm = external_process.ProcessMonitor(cfg.CONF, 'router')
 
         # Create a temp directory to store keepalived configuration.
         self.config_path = tempfile.mkdtemp()
@@ -301,8 +304,7 @@ class KeepalivedIPv6Test(object):
         return self
 
     def __exit__(self, exc_type, exc_value, exc_tb):
-        if self.pm:
-            self.pm.stop()
+        self.pm.stop()
         if self.manager:
             self.manager.disable()
         if self.config_path:
@@ -378,7 +380,7 @@ def ovs_conntrack_supported():
 
     with ovs_lib.OVSBridge(br_name) as br:
         try:
-            br.set_protocols(["OpenFlow%d" % i for i in range(10, 15)])
+            br.add_protocols(*["OpenFlow%d" % i for i in range(10, 15)])
         except RuntimeError as e:
             LOG.debug("Exception while checking ovs conntrack support: %s", e)
             return False
@@ -432,3 +434,36 @@ def dibbler_version_supported():
         LOG.debug("Exception while checking minimal dibbler version. "
                   "Exception: %s", e)
         return False
+
+
+def _fix_ip_nonlocal_bind_root_value(original_value):
+    current_value = ip_lib.get_ip_nonlocal_bind(namespace=None)
+    if current_value != original_value:
+        ip_lib.set_ip_nonlocal_bind(value=original_value, namespace=None)
+
+
+def ip_nonlocal_bind():
+    ipw = ip_lib.IPWrapper()
+    nsname1 = "ipnonlocalbind1-" + uuidutils.generate_uuid()
+    nsname2 = "ipnonlocalbind2-" + uuidutils.generate_uuid()
+
+    ipw.netns.add(nsname1)
+    try:
+        ipw.netns.add(nsname2)
+        try:
+            original_value = ip_lib.get_ip_nonlocal_bind(namespace=None)
+            try:
+                ip_lib.set_ip_nonlocal_bind(value=0, namespace=nsname1)
+                ip_lib.set_ip_nonlocal_bind(value=1, namespace=nsname2)
+                ns1_value = ip_lib.get_ip_nonlocal_bind(namespace=nsname1)
+            finally:
+                _fix_ip_nonlocal_bind_root_value(original_value)
+        except RuntimeError as e:
+            LOG.debug("Exception while checking ip_nonlocal_bind. "
+                      "Exception: %s", e)
+            return False
+        finally:
+            ipw.netns.delete(nsname2)
+    finally:
+        ipw.netns.delete(nsname1)
+    return ns1_value == 0
