@@ -16,6 +16,7 @@
 import os
 
 import eventlet
+from oslo_utils import importutils
 
 
 def monkey_patch():
@@ -30,3 +31,5 @@ def monkey_patch():
         eventlet.monkey_patch(os=False, thread=False)
     else:
         eventlet.monkey_patch()
+        p_c_e = importutils.import_module('pyroute2.config.eventlet')
+        p_c_e.eventlet_config()
