@@ -18,32 +18,33 @@
 
 """Utilities and helper functions."""
 
-import collections
-import decimal
-import errno
 import functools
 import importlib
-import multiprocessing
 import os
 import os.path
 import random
 import signal
-import socket
 import sys
-import tempfile
-import threading
 import time
 import uuid
+import weakref
 
+import debtcollector
+from debtcollector import removals
 import eventlet
 from eventlet.green import subprocess
 import netaddr
 from neutron_lib import constants as n_const
+from neutron_lib.utils import file as file_utils
+from neutron_lib.utils import helpers
+from neutron_lib.utils import host
+from neutron_lib.utils import net
 from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 from oslo_utils import excutils
+from oslo_utils import fileutils
 from oslo_utils import importutils
 import six
 from stevedore import driver
@@ -56,64 +57,30 @@ TIME_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
 LOG = logging.getLogger(__name__)
 SYNCHRONIZED_PREFIX = 'neutron-'
 
-DEFAULT_THROTTLER_VALUE = 2
-
 synchronized = lockutils.synchronized_with_prefix(SYNCHRONIZED_PREFIX)
 
 
-class LockWithTimer(object):
-    def __init__(self, threshold):
-        self._threshold = threshold
-        self.timestamp = 0
-        self._lock = threading.Lock()
+class WaitTimeout(Exception, eventlet.TimeoutError):
+    """Default exception coming from wait_until_true() function.
 
-    def acquire(self):
-        return self._lock.acquire(False)
-
-    def release(self):
-        return self._lock.release()
-
-    def time_to_wait(self):
-        return self.timestamp - time.time() + self._threshold
-
-
-# REVISIT(jlibosva): Some parts of throttler may be similar to what
-#                    neutron.notifiers.batch_notifier.BatchNotifier does. They
-#                    could be refactored and unified.
-def throttler(threshold=DEFAULT_THROTTLER_VALUE):
-    """Throttle number of calls to a function to only once per 'threshold'.
+    The reason is that eventlet.TimeoutError inherits from BaseException and
+    testtools.TestCase consumes only Exceptions. Which means in case
+    TimeoutError is raised, test runner stops and exits while it still has test
+    cases scheduled for execution.
     """
-    def decorator(f):
-        lock_with_timer = LockWithTimer(threshold)
 
-        @functools.wraps(f)
-        def wrapper(*args, **kwargs):
-            if lock_with_timer.acquire():
-                try:
-                    fname = f.__name__
-                    time_to_wait = lock_with_timer.time_to_wait()
-                    if time_to_wait > 0:
-                        LOG.debug("Call of function %s scheduled, sleeping "
-                                  "%.1f seconds", fname, time_to_wait)
-                        # Decorated function has been called recently, wait.
-                        eventlet.sleep(time_to_wait)
-                    lock_with_timer.timestamp = time.time()
-                finally:
-                    lock_with_timer.release()
-                LOG.debug("Calling throttled function %s", fname)
-                return f(*args, **kwargs)
-        return wrapper
-    return decorator
+    def __str__(self):
+        return Exception.__str__(self)
+
+    def __repr__(self):
+        return Exception.__repr__(self)
 
 
+@removals.remove(
+    message="Use ensure_tree(path, 0o755) from oslo_utils.fileutils")
 def ensure_dir(dir_path):
     """Ensure a directory with 755 permissions mode."""
-    try:
-        os.makedirs(dir_path, 0o755)
-    except OSError as e:
-        # If the directory already existed, don't raise the error.
-        if e.errno != errno.EEXIST:
-            raise
+    fileutils.ensure_tree(dir_path, mode=0o755)
 
 
 def _subprocess_setup():
@@ -130,98 +97,57 @@ def subprocess_popen(args, stdin=None, stdout=None, stderr=None, shell=False,
                             close_fds=close_fds, env=env)
 
 
+@removals.remove(
+    message="Use parse_mappings from neutron_lib.utils.helpers")
 def parse_mappings(mapping_list, unique_values=True, unique_keys=True):
-    """Parse a list of mapping strings into a dictionary.
-
-    :param mapping_list: a list of strings of the form '<key>:<value>'
-    :param unique_values: values must be unique if True
-    :param unique_keys: keys must be unique if True, else implies that keys
-    and values are not unique
-    :returns: a dict mapping keys to values or to list of values
-    """
-    mappings = {}
-    for mapping in mapping_list:
-        mapping = mapping.strip()
-        if not mapping:
-            continue
-        split_result = mapping.split(':')
-        if len(split_result) != 2:
-            raise ValueError(_("Invalid mapping: '%s'") % mapping)
-        key = split_result[0].strip()
-        if not key:
-            raise ValueError(_("Missing key in mapping: '%s'") % mapping)
-        value = split_result[1].strip()
-        if not value:
-            raise ValueError(_("Missing value in mapping: '%s'") % mapping)
-        if unique_keys:
-            if key in mappings:
-                raise ValueError(_("Key %(key)s in mapping: '%(mapping)s' not "
-                                   "unique") % {'key': key,
-                                                'mapping': mapping})
-            if unique_values and value in mappings.values():
-                raise ValueError(_("Value %(value)s in mapping: '%(mapping)s' "
-                                   "not unique") % {'value': value,
-                                                    'mapping': mapping})
-            mappings[key] = value
-        else:
-            mappings.setdefault(key, [])
-            if value not in mappings[key]:
-                mappings[key].append(value)
-    return mappings
+    return helpers.parse_mappings(mapping_list, unique_values=unique_values,
+                                  unique_keys=unique_keys)
 
 
+@removals.remove(
+    message="Use get_hostname from neutron_lib.utils.net")
 def get_hostname():
-    return socket.gethostname()
+    return net.get_hostname()
 
 
 def get_first_host_ip(net, ip_version):
     return str(netaddr.IPAddress(net.first + 1, ip_version))
 
 
+@removals.remove(
+    message="Use compare_elements from neutron_lib.utils.helpers")
 def compare_elements(a, b):
-    """Compare elements if a and b have same elements.
-
-    This method doesn't consider ordering
-    """
-    if a is None:
-        a = []
-    if b is None:
-        b = []
-    return set(a) == set(b)
+    return helpers.compare_elements(a, b)
 
 
+@removals.remove(
+    message="Use safe_sort_key from neutron_lib.utils.helpers")
 def safe_sort_key(value):
-    """Return value hash or build one for dictionaries."""
-    if isinstance(value, collections.Mapping):
-        return sorted(value.items())
-    return value
+    return helpers.safe_sort_key(value)
 
 
+@removals.remove(
+    message="Use dict2str from neutron_lib.utils.helpers")
 def dict2str(dic):
-    return ','.join("%s=%s" % (key, val)
-                    for key, val in sorted(six.iteritems(dic)))
+    return helpers.dict2str(dic)
 
 
+@removals.remove(
+    message="Use str2dict from neutron_lib.utils.helpers")
 def str2dict(string):
-    res_dict = {}
-    for keyvalue in string.split(','):
-        (key, value) = keyvalue.split('=', 1)
-        res_dict[key] = value
-    return res_dict
+    return helpers.str2dict(string)
 
 
+@removals.remove(
+    message="Use dict2tuple from neutron_lib.utils.helpers")
 def dict2tuple(d):
-    items = list(d.items())
-    items.sort()
-    return tuple(items)
+    return helpers.dict2tuple(d)
 
 
+@removals.remove(
+    message="Use diff_list_of_dict from neutron_lib.utils.helpers")
 def diff_list_of_dict(old_list, new_list):
-    new_set = set([dict2str(l) for l in new_list])
-    old_set = set([dict2str(l) for l in old_list])
-    added = new_set - old_set
-    removed = old_set - new_set
-    return [str2dict(a) for a in added], [str2dict(r) for r in removed]
+    return helpers.diff_list_of_dict(old_list, new_list)
 
 
 def is_extension_supported(plugin, ext_alias):
@@ -242,13 +168,6 @@ def get_random_mac(base_mac):
     return ':'.join(["%02x" % x for x in mac])
 
 
-def get_random_string(length):
-    """Get a random hex string of the specified length.
-    """
-
-    return "{0:0{1}x}".format(random.getrandbits(length * 4), length)
-
-
 def get_dhcp_agent_device_id(network_id, host):
     # Split host so as to always use only the hostname and
     # not the domain name. This will guarantee consistency
@@ -258,11 +177,10 @@ def get_dhcp_agent_device_id(network_id, host):
     return 'dhcp%s-%s' % (host_uuid, network_id)
 
 
+@removals.remove(
+    message="Use cpu_count from neutron_lib.utils.host")
 def cpu_count():
-    try:
-        return multiprocessing.cpu_count()
-    except NotImplementedError:
-        return 1
+    return host.cpu_count()
 
 
 class exception_logger(object):
@@ -403,33 +321,34 @@ class DelayedStringRenderer(object):
         return str(self.function(*self.args, **self.kwargs))
 
 
-def camelize(s):
-    return ''.join(s.replace('_', ' ').title().split())
-
-
+@removals.remove(
+    message="Use round_val from neutron_lib.utils.helpers")
 def round_val(val):
-    # we rely on decimal module since it behaves consistently across Python
-    # versions (2.x vs. 3.x)
-    return int(decimal.Decimal(val).quantize(decimal.Decimal('1'),
-                                             rounding=decimal.ROUND_HALF_UP))
+    return helpers.round_val(val)
 
 
+@removals.remove(
+    message="Use replace_file from neutron_lib.utils")
 def replace_file(file_name, data, file_mode=0o644):
-    """Replaces the contents of file_name with data in a safe manner.
+    file_utils.replace_file(file_name, data, file_mode=file_mode)
 
-    First write to a temp file and then rename. Since POSIX renames are
-    atomic, the file is unlikely to be corrupted by competing writes.
 
-    We create the tempfile on the same device to ensure that it can be renamed.
+class _SilentDriverManager(driver.DriverManager):
+    """The lamest of hacks to allow us to pass a kwarg to DriverManager parent.
+
+    DriverManager doesn't accept the warn_on_missing_entrypoint param
+    to pass to its parent on __init__ so we mirror the __init__ here and bypass
+    the one in DriverManager in order to silence the warnings.
+    TODO(kevinbenton): remove once Ia6f5f749fc2f73ca6091fa6d58506fddb058902a
+    is released or we stop supporting loading by class path.
     """
-
-    base_dir = os.path.dirname(os.path.abspath(file_name))
-    with tempfile.NamedTemporaryFile('w+',
-                                     dir=base_dir,
-                                     delete=False) as tmp_file:
-        tmp_file.write(data)
-    os.chmod(tmp_file.name, file_mode)
-    os.rename(tmp_file.name, file_name)
+    def __init__(self, namespace, name):
+        p = super(driver.DriverManager, self)  # pylint: disable=bad-super-call
+        p.__init__(
+            namespace=namespace, names=[name],
+            on_load_failure_callback=self._default_on_load_failure,
+            warn_on_missing_entrypoint=False
+        )
 
 
 def load_class_by_alias_or_classname(namespace, name):
@@ -445,7 +364,7 @@ def load_class_by_alias_or_classname(namespace, name):
         raise ImportError(_("Class not found."))
     try:
         # Try to resolve class by alias
-        mgr = driver.DriverManager(namespace, name)
+        mgr = _SilentDriverManager(namespace, name)
         class_to_load = mgr.driver
     except RuntimeError:
         e1_info = sys.exc_info()
@@ -461,10 +380,10 @@ def load_class_by_alias_or_classname(namespace, name):
     return class_to_load
 
 
+@removals.remove(
+    message="Use safe_decode_utf8 from neutron_lib.utils.helpers")
 def safe_decode_utf8(s):
-    if six.PY3 and isinstance(s, bytes):
-        return s.decode('utf-8', 'surrogateescape')
-    return s
+    return helpers.safe_decode_utf8(s)
 
 
 def _hex_format(port, mask=0):
@@ -813,12 +732,26 @@ def wait_until_true(predicate, timeout=60, sleep=1, exception=None):
     Best practice is to instantiate predicate with functools.partial()
     :param timeout: Timeout in seconds how long should function wait.
     :param sleep: Polling interval for results in seconds.
-    :param exception: Exception class for eventlet.Timeout.
-    (see doc for eventlet.Timeout for more information)
+    :param exception: Exception instance to raise on timeout. If None is passed
+                      (default) then WaitTimeout exception is raised.
     """
-    with eventlet.timeout.Timeout(timeout, exception):
-        while not predicate():
-            eventlet.sleep(sleep)
+    try:
+        with eventlet.timeout.Timeout(timeout):
+            while not predicate():
+                eventlet.sleep(sleep)
+    except eventlet.TimeoutError:
+        if exception is None:
+            debtcollector.deprecate(
+                "Raising eventlet.TimeoutError by default has been deprecated",
+                message="wait_until_true() now raises WaitTimeout error by "
+                        "default.",
+                version="Ocata",
+                removal_version="Pike")
+            exception = WaitTimeout("Timed out after %d seconds" % timeout)
+        #NOTE(jlibosva): In case None is passed exception is instantiated on
+        #                the line above.
+        #pylint: disable=raising-bad-type
+        raise exception
 
 
 class _AuthenticBase(object):
@@ -939,13 +872,28 @@ def get_related_rand_names(prefixes, max_length=None):
     if max_length:
         length = max_length - max(len(p) for p in prefixes)
         if length <= 0:
-            raise ValueError("'max_length' must be longer than all prefixes")
+            raise ValueError(
+                _("'max_length' must be longer than all prefixes"))
     else:
         length = 8
-    rndchrs = get_random_string(length)
+    rndchrs = helpers.get_random_string(length)
     return [p + rndchrs for p in prefixes]
 
 
 def get_related_rand_device_names(prefixes):
     return get_related_rand_names(prefixes,
                                   max_length=n_const.DEVICE_NAME_MAX_LEN)
+
+
+try:
+    # PY3
+    weak_method = weakref.WeakMethod
+except AttributeError:
+    # PY2
+    import weakrefmethod
+    weak_method = weakrefmethod.WeakMethod
+
+
+def make_weak_ref(f):
+    """Make a weak reference to a function accounting for bound methods."""
+    return weak_method(f) if hasattr(f, '__self__') else weakref.ref(f)
