@@ -13,46 +13,24 @@
 # under the License.
 
 import netaddr
-from neutron_lib.db import model_base
 from oslo_db import exception as db_exc
 from oslo_utils import uuidutils
-import sqlalchemy as sa
 from sqlalchemy import orm
-from sqlalchemy import sql
 
 from neutron.api.rpc.agentnotifiers import metering_rpc_agent_api
-from neutron.api.v2 import attributes as attr
+from neutron.common import _deprecate
 from neutron.common import constants
+from neutron.db import _utils as db_utils
+from neutron.db import api as db_api
 from neutron.db import common_db_mixin as base_db
-from neutron.db import l3_db
 from neutron.db import l3_dvr_db
+from neutron.db.models import l3 as l3_models
+from neutron.db.models import metering as metering_models
 from neutron.extensions import metering
 
 
-class MeteringLabelRule(model_base.BASEV2, model_base.HasId):
-    direction = sa.Column(sa.Enum('ingress', 'egress',
-                                  name='meteringlabels_direction'))
-    remote_ip_prefix = sa.Column(sa.String(64))
-    metering_label_id = sa.Column(sa.String(36),
-                                  sa.ForeignKey("meteringlabels.id",
-                                                ondelete="CASCADE"),
-                                  nullable=False)
-    excluded = sa.Column(sa.Boolean, default=False, server_default=sql.false())
-
-
-class MeteringLabel(model_base.BASEV2,
-                    model_base.HasId,
-                    model_base.HasProject):
-    name = sa.Column(sa.String(attr.NAME_MAX_LEN))
-    description = sa.Column(sa.String(attr.LONG_DESCRIPTION_MAX_LEN))
-    rules = orm.relationship(MeteringLabelRule, backref="label",
-                             cascade="delete", lazy="joined")
-    routers = orm.relationship(
-        l3_db.Router,
-        primaryjoin="MeteringLabel.tenant_id==Router.tenant_id",
-        foreign_keys='MeteringLabel.tenant_id',
-        uselist=True)
-    shared = sa.Column(sa.Boolean, default=False, server_default=sql.false())
+_deprecate._moved_global('MeteringLabelRule', new_module=metering_models)
+_deprecate._moved_global('MeteringLabel', new_module=metering_models)
 
 
 class MeteringDbMixin(metering.MeteringPluginBase,
@@ -61,31 +39,35 @@ class MeteringDbMixin(metering.MeteringPluginBase,
     def __init__(self):
         self.meter_rpc = metering_rpc_agent_api.MeteringAgentNotifyAPI()
 
-    def _make_metering_label_dict(self, metering_label, fields=None):
+    @staticmethod
+    def _make_metering_label_dict(metering_label, fields=None):
         res = {'id': metering_label['id'],
                'name': metering_label['name'],
                'description': metering_label['description'],
                'shared': metering_label['shared'],
                'tenant_id': metering_label['tenant_id']}
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     def create_metering_label(self, context, metering_label):
         m = metering_label['metering_label']
 
-        with context.session.begin(subtransactions=True):
-            metering_db = MeteringLabel(id=uuidutils.generate_uuid(),
-                                        description=m['description'],
-                                        tenant_id=m['tenant_id'],
-                                        name=m['name'],
-                                        shared=m['shared'])
+        with db_api.context_manager.writer.using(context):
+            metering_db = metering_models.MeteringLabel(
+                id=uuidutils.generate_uuid(),
+                description=m['description'],
+                tenant_id=m['tenant_id'],
+                name=m['name'],
+                shared=m['shared'])
             context.session.add(metering_db)
 
         return self._make_metering_label_dict(metering_db)
 
     def delete_metering_label(self, context, label_id):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             try:
-                label = self._get_by_id(context, MeteringLabel, label_id)
+                label = self._get_by_id(context,
+                                        metering_models.MeteringLabel,
+                                        label_id)
             except orm.exc.NoResultFound:
                 raise metering.MeteringLabelNotFound(label_id=label_id)
 
@@ -93,7 +75,9 @@ class MeteringDbMixin(metering.MeteringPluginBase,
 
     def get_metering_label(self, context, label_id, fields=None):
         try:
-            metering_label = self._get_by_id(context, MeteringLabel, label_id)
+            metering_label = self._get_by_id(context,
+                                             metering_models.MeteringLabel,
+                                             label_id)
         except orm.exc.NoResultFound:
             raise metering.MeteringLabelNotFound(label_id=label_id)
 
@@ -104,7 +88,7 @@ class MeteringDbMixin(metering.MeteringPluginBase,
                             page_reverse=False):
         marker_obj = self._get_marker_obj(context, 'metering_labels', limit,
                                           marker)
-        return self._get_collection(context, MeteringLabel,
+        return self._get_collection(context, metering_models.MeteringLabel,
                                     self._make_metering_label_dict,
                                     filters=filters, fields=fields,
                                     sorts=sorts,
@@ -112,13 +96,14 @@ class MeteringDbMixin(metering.MeteringPluginBase,
                                     marker_obj=marker_obj,
                                     page_reverse=page_reverse)
 
-    def _make_metering_label_rule_dict(self, metering_label_rule, fields=None):
+    @staticmethod
+    def _make_metering_label_rule_dict(metering_label_rule, fields=None):
         res = {'id': metering_label_rule['id'],
                'metering_label_id': metering_label_rule['metering_label_id'],
                'direction': metering_label_rule['direction'],
                'remote_ip_prefix': metering_label_rule['remote_ip_prefix'],
                'excluded': metering_label_rule['excluded']}
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     def get_metering_label_rules(self, context, filters=None, fields=None,
                                  sorts=None, limit=None, marker=None,
@@ -126,7 +111,7 @@ class MeteringDbMixin(metering.MeteringPluginBase,
         marker_obj = self._get_marker_obj(context, 'metering_label_rules',
                                           limit, marker)
 
-        return self._get_collection(context, MeteringLabelRule,
+        return self._get_collection(context, metering_models.MeteringLabelRule,
                                     self._make_metering_label_rule_dict,
                                     filters=filters, fields=fields,
                                     sorts=sorts,
@@ -136,8 +121,8 @@ class MeteringDbMixin(metering.MeteringPluginBase,
 
     def get_metering_label_rule(self, context, rule_id, fields=None):
         try:
-            metering_label_rule = self._get_by_id(context,
-                                                  MeteringLabelRule, rule_id)
+            metering_label_rule = self._get_by_id(
+                context, metering_models.MeteringLabelRule, rule_id)
         except orm.exc.NoResultFound:
             raise metering.MeteringLabelRuleNotFound(rule_id=rule_id)
 
@@ -163,7 +148,7 @@ class MeteringDbMixin(metering.MeteringPluginBase,
     def create_metering_label_rule(self, context, metering_label_rule):
         m = metering_label_rule['metering_label_rule']
         try:
-            with context.session.begin(subtransactions=True):
+            with db_api.context_manager.writer.using(context):
                 label_id = m['metering_label_id']
                 ip_prefix = m['remote_ip_prefix']
                 direction = m['direction']
@@ -171,21 +156,25 @@ class MeteringDbMixin(metering.MeteringPluginBase,
 
                 self._validate_cidr(context, label_id, ip_prefix, direction,
                                     excluded)
-                metering_db = MeteringLabelRule(id=uuidutils.generate_uuid(),
-                                                metering_label_id=label_id,
-                                                direction=direction,
-                                                excluded=m['excluded'],
-                                                remote_ip_prefix=ip_prefix)
+                metering_db = metering_models.MeteringLabelRule(
+                    id=uuidutils.generate_uuid(),
+                    metering_label_id=label_id,
+                    direction=direction,
+                    excluded=m['excluded'],
+                    remote_ip_prefix=ip_prefix)
                 context.session.add(metering_db)
+
         except db_exc.DBReferenceError:
             raise metering.MeteringLabelNotFound(label_id=label_id)
 
         return self._make_metering_label_rule_dict(metering_db)
 
     def delete_metering_label_rule(self, context, rule_id):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             try:
-                rule = self._get_by_id(context, MeteringLabelRule, rule_id)
+                rule = self._get_by_id(context,
+                                       metering_models.MeteringLabelRule,
+                                       rule_id)
             except orm.exc.NoResultFound:
                 raise metering.MeteringLabelRuleNotFound(rule_id=rule_id)
             context.session.delete(rule)
@@ -221,12 +210,14 @@ class MeteringDbMixin(metering.MeteringPluginBase,
             if label.shared:
                 if not all_routers:
                     all_routers = self._get_collection_query(context,
-                                                             l3_db.Router)
+                                                             l3_models.Router)
                 routers = all_routers
             else:
                 routers = label.routers
 
             for router in routers:
+                if not router['admin_state_up']:
+                    continue
                 router_dict = routers_dict.get(
                     router['id'],
                     self._make_router_dict(router))
@@ -241,11 +232,12 @@ class MeteringDbMixin(metering.MeteringPluginBase,
         return list(routers_dict.values())
 
     def get_sync_data_for_rule(self, context, rule):
-        label = context.session.query(MeteringLabel).get(
-            rule['metering_label_id'])
+        label = context.session.query(
+            metering_models.MeteringLabel).get(
+                rule['metering_label_id'])
 
         if label.shared:
-            routers = self._get_collection_query(context, l3_db.Router)
+            routers = self._get_collection_query(context, l3_models.Router)
         else:
             routers = label.routers
 
@@ -259,13 +251,14 @@ class MeteringDbMixin(metering.MeteringPluginBase,
 
         return list(routers_dict.values())
 
-    def get_sync_data_metering(self, context, label_id=None, router_ids=None):
-        labels = context.session.query(MeteringLabel)
+    def get_sync_data_metering(self, context, label_id=None):
+        labels = context.session.query(metering_models.MeteringLabel)
 
         if label_id:
-            labels = labels.filter(MeteringLabel.id == label_id)
-        elif router_ids:
-            labels = (labels.join(MeteringLabel.routers).
-                      filter(l3_db.Router.id.in_(router_ids)))
+            labels = labels.filter(
+                metering_models.MeteringLabel.id == label_id)
 
         return self._process_sync_metering_data(context, labels)
+
+
+_deprecate._MovedGlobals()
