@@ -15,13 +15,14 @@
 
 import re
 
+from neutron_lib.api import extensions
+from neutron_lib.api import validators
+from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
 import six
 
 from neutron._i18n import _
-from neutron.api import extensions
 from neutron.api.v2 import attributes as attr
-from neutron.common import exceptions as n_exc
 from neutron.extensions import l3
 
 DNS_LABEL_MAX_LEN = 63
@@ -60,7 +61,7 @@ def _validate_dns_name(data, max_len=FQDN_MAX_LEN):
 
 
 def _validate_fip_dns_name(data, max_len=FQDN_MAX_LEN):
-    msg = attr._validate_string(data)
+    msg = validators.validate_string(data)
     if msg:
         return msg
     if not data:
@@ -82,7 +83,7 @@ def _validate_fip_dns_name(data, max_len=FQDN_MAX_LEN):
 
 
 def _validate_dns_domain(data, max_len=FQDN_MAX_LEN):
-    msg = attr._validate_string(data)
+    msg = validators.validate_string(data)
     if msg:
         return msg
     if not data:
@@ -195,11 +196,9 @@ def convert_to_lowercase(data):
     msg = _("'%s' cannot be converted to lowercase string") % data
     raise n_exc.InvalidInput(error_message=msg)
 
-
-attr.validators['type:dns_name'] = (_validate_dns_name)
-attr.validators['type:fip_dns_name'] = (_validate_fip_dns_name)
-attr.validators['type:dns_domain'] = (_validate_dns_domain)
-
+validators.add_validator('dns_name', _validate_dns_name)
+validators.add_validator('fip_dns_name', _validate_fip_dns_name)
+validators.add_validator('dns_domain', _validate_dns_domain)
 
 DNSNAME = 'dns_name'
 DNSDOMAIN = 'dns_domain'
