@@ -14,21 +14,19 @@
 #    under the License.
 
 from neutron_lib.api import converters
+from neutron_lib.api import extensions as api_extensions
 from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_utils import importutils
 import webob
 
 from neutron._i18n import _
 from neutron.api import extensions
-from neutron.api.v2 import attributes
 from neutron.api.v2 import base
 from neutron.api.v2 import resource
 from neutron.common import constants as const
 from neutron.common import exceptions
-from neutron import manager
-from neutron.pecan_wsgi import controllers
-from neutron.pecan_wsgi.controllers import utils as pecan_utils
 from neutron import quota
 from neutron.quota import resource_registry
 from neutron import wsgi
@@ -127,7 +125,7 @@ class QuotaSetsController(wsgi.Controller):
         return {self._resource_name: self._get_quotas(request, id)}
 
 
-class Quotasv2(extensions.ExtensionDescriptor):
+class Quotasv2(api_extensions.ExtensionDescriptor):
     """Quotas management support."""
 
     @classmethod
@@ -153,7 +151,7 @@ class Quotasv2(extensions.ExtensionDescriptor):
     def get_resources(cls):
         """Returns Ext Resources."""
         controller = resource.Resource(
-            QuotaSetsController(manager.NeutronManager.get_plugin()),
+            QuotaSetsController(directory.get_plugin()),
             faults=base.FAULT_MAP)
         return [extensions.ResourceExtension(
             Quotasv2.get_alias(),
@@ -161,14 +159,6 @@ class Quotasv2(extensions.ExtensionDescriptor):
             member_actions={DEFAULT_QUOTAS_ACTION: 'GET'},
             collection_actions={'tenant': 'GET'})]
 
-    @classmethod
-    def get_pecan_resources(cls):
-        # NOTE: quotas in PLURALS is needed because get_resources never sets it
-        attributes.PLURALS[RESOURCE_COLLECTION] = RESOURCE_NAME
-        # NOTE: plugin is not needed for quotas
-        return [pecan_utils.PecanResourceExtension(
-            RESOURCE_COLLECTION, controllers.QuotasController(), None)]
-
     def get_extended_resources(self, version):
         if version == "2.0":
             return EXTENDED_ATTRIBUTES_2_0
