@@ -13,27 +13,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.api import converters
+from neutron_lib.db import constants as db_const
+from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
-from oslo_log import log
 from oslo_utils import importutils
 import pecan
 from pecan import request
-from pecan import response
 
 from neutron._i18n import _
 from neutron.api.v2 import attributes
 from neutron.common import constants
-from neutron.common import exceptions as n_exc
 from neutron.pecan_wsgi.controllers import utils
 from neutron.quota import resource_registry
 
-LOG = log.getLogger(__name__)
 RESOURCE_NAME = "quota"
 TENANT_ID_ATTR = {'tenant_id':
                   {'allow_post': False,
                    'allow_put': False,
                    'required_by_policy': True,
-                   'validate': {'type:string': attributes.TENANT_ID_MAX_LEN},
+                   'validate': {'type:string': db_const.PROJECT_ID_FIELD_SIZE},
                    'is_visible': True}}
 
 
@@ -58,7 +57,7 @@ class QuotasController(utils.NeutronPecanController):
     @utils.expose(generic=True)
     def index(self):
         neutron_context = request.context.get('neutron_context')
-        # FIXME(salv-orlando): There shouldn't be any need to to this eplicit
+        # FIXME(salv-orlando): There shouldn't be any need to do this explicit
         # check. However some behaviours from the "old" extension have
         # been temporarily carried over here
         self._check_admin(neutron_context)
@@ -90,7 +89,7 @@ class QuotaController(utils.NeutronPecanController):
             attr_dict[quota_resource] = {
                 'allow_post': False,
                 'allow_put': True,
-                'convert_to': attributes.convert_to_int,
+                'convert_to': converters.convert_to_int,
                 'validate': {
                     'type:range': [-1, constants.DB_INTEGER_MAX_VALUE]},
                 'is_visible': True}
@@ -113,12 +112,11 @@ class QuotaController(utils.NeutronPecanController):
                 neutron_context, self._tenant_id, key, value)
         return get_tenant_quotas(self._tenant_id, self._driver)
 
-    @utils.when(index, method='DELETE')
+    @utils.when_delete(index)
     def delete(self):
         neutron_context = request.context.get('neutron_context')
         self._driver.delete_tenant_quota(neutron_context,
                                          self._tenant_id)
-        response.status = 204
 
     @utils.when(index, method='POST')
     def not_supported(self):
