@@ -13,15 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib import constants
+
+
 # Neutron well-known service type constants:
-CORE = "CORE"
 DUMMY = "DUMMY"
 LOADBALANCER = "LOADBALANCER"
 LOADBALANCERV2 = "LOADBALANCERV2"
 FIREWALL = "FIREWALL"
 VPN = "VPN"
 METERING = "METERING"
-L3_ROUTER_NAT = "L3_ROUTER_NAT"
 FLAVORS = "FLAVORS"
 QOS = "QOS"
 
@@ -34,7 +35,7 @@ EXT_TO_SERVICE_MAPPING = {
     'fwaas': FIREWALL,
     'vpnaas': VPN,
     'metering': METERING,
-    'router': L3_ROUTER_NAT,
+    'router': constants.L3,
     'qos': QOS,
 }
 
@@ -42,8 +43,10 @@ EXT_TO_SERVICE_MAPPING = {
 DEFAULT_SERVICE_PLUGINS = {
     'auto_allocate': 'auto-allocated-topology',
     'tag': 'tag',
-    'timestamp_core': 'timestamp_core',
-    'network_ip_availability': 'network-ip-availability'
+    'timestamp': 'timestamp',
+    'network_ip_availability': 'network-ip-availability',
+    'flavors': 'flavors',
+    'revisions': 'revisions',
 }
 
 # Service operation status constants
@@ -90,7 +93,13 @@ MIN_VXLAN_VNI = 1
 MAX_VXLAN_VNI = 2 ** 24 - 1
 VXLAN_UDP_PORT = 4789
 
-# Network Type MTU overhead
-GENEVE_ENCAP_MIN_OVERHEAD = 50
-GRE_ENCAP_OVERHEAD = 42
-VXLAN_ENCAP_OVERHEAD = 50
+# Overlay (tunnel) protocol overhead
+GENEVE_ENCAP_MIN_OVERHEAD = 30
+GRE_ENCAP_OVERHEAD = 22
+VXLAN_ENCAP_OVERHEAD = 30
+
+# IP header length
+IP_HEADER_LENGTH = {
+    4: 20,
+    6: 40,
+}
