@@ -38,15 +38,12 @@ class MechanismDriverContext(object):
 class NetworkContext(MechanismDriverContext, api.NetworkContext):
 
     def __init__(self, plugin, plugin_context, network,
-                 original_network=None, segments=None):
+                 original_network=None):
         super(NetworkContext, self).__init__(plugin, plugin_context)
         self._network = network
         self._original_network = original_network
-        if segments is None:
-            self._segments = segments_db.get_network_segments(
-                plugin_context.session, network['id'])
-        else:
-            self._segments = segments
+        self._segments = segments_db.get_network_segments(
+            plugin_context, network['id'])
 
     @property
     def current(self):
@@ -91,11 +88,8 @@ class PortContext(MechanismDriverContext, api.PortContext):
         super(PortContext, self).__init__(plugin, plugin_context)
         self._port = port
         self._original_port = original_port
-        if isinstance(network, NetworkContext):
-            self._network_context = network
-        else:
-            self._network_context = NetworkContext(
-                plugin, plugin_context, network) if network else None
+        self._network_context = NetworkContext(plugin, plugin_context,
+                                               network)
         self._binding = binding
         self._binding_levels = binding_levels
         self._segments_to_bind = None
@@ -157,11 +151,6 @@ class PortContext(MechanismDriverContext, api.PortContext):
 
     @property
     def network(self):
-        if not self._network_context:
-            network = self._plugin.get_network(
-                self._plugin_context, self.current['network_id'])
-            self._network_context = NetworkContext(
-                self._plugin, self._plugin_context, network)
         return self._network_context
 
     @property
@@ -203,13 +192,7 @@ class PortContext(MechanismDriverContext, api.PortContext):
                 self._original_binding_levels[-1].segment_id)
 
     def _expand_segment(self, segment_id):
-        for s in self.network.network_segments:
-            if s['id'] == segment_id:
-                return s
-        # TODO(kevinbenton): eliminate the query below. The above should
-        # always return since the port is bound to a network segment. Leaving
-        # in for now for minimally invasive change for back-port.
-        segment = segments_db.get_segment_by_id(self._plugin_context.session,
+        segment = segments_db.get_segment_by_id(self._plugin_context,
                                                 segment_id)
         if not segment:
             LOG.warning(_LW("Could not expand segment %s"), segment_id)
@@ -280,4 +263,4 @@ class PortContext(MechanismDriverContext, api.PortContext):
 
     def release_dynamic_segment(self, segment_id):
         return self._plugin.type_manager.release_dynamic_segment(
-                self._plugin_context.session, segment_id)
+                self._plugin_context, segment_id)
