@@ -13,24 +13,25 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib import constants as const
+from neutron_lib import exceptions
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log as logging
 
-from neutron._i18n import _LW
-from neutron.common import constants as const
+from neutron._i18n import _, _LW
+from neutron.conf.plugins.ml2.drivers import l2pop as config
 from neutron import context as n_context
 from neutron.db import api as db_api
 from neutron.db import l3_hamode_db
-from neutron import manager
-from neutron.plugins.common import constants as service_constants
-from neutron.plugins.ml2.common import exceptions as ml2_exc
 from neutron.plugins.ml2 import driver_api as api
-from neutron.plugins.ml2.drivers.l2pop import config  # noqa
 from neutron.plugins.ml2.drivers.l2pop import db as l2pop_db
 from neutron.plugins.ml2.drivers.l2pop import rpc as l2pop_rpc
 
 LOG = logging.getLogger(__name__)
 
+config.register_l2_population_opts()
+
 
 class L2populationMechanismDriver(api.MechanismDriver):
 
@@ -51,6 +52,10 @@ class L2populationMechanismDriver(api.MechanismDriver):
                                    ip_address=ip['ip_address'])
                 for ip in port['fixed_ips']]
 
+    def check_vlan_transparency(self, context):
+        """L2population driver vlan transparency support."""
+        return True
+
     def _get_ha_port_agents_fdb(
             self, session, network_id, router_id):
         other_fdb_ports = {}
@@ -66,10 +71,10 @@ class L2populationMechanismDriver(api.MechanismDriver):
     def delete_port_postcommit(self, context):
         port = context.current
         agent_host = context.host
-        fdb_entries = self._get_agent_fdb(context.bottom_bound_segment,
-                                          port, agent_host)
+        fdb_entries = self._get_agent_fdb(
+            context, context.bottom_bound_segment, port, agent_host)
         if port['device_owner'] in l2pop_db.HA_ROUTER_PORTS and fdb_entries:
-            session = db_api.get_session()
+            session = db_api.get_reader_session()
             network_id = port['network_id']
             other_fdb_ports = self._get_ha_port_agents_fdb(
                 session, network_id, port['device_id'])
@@ -78,6 +83,12 @@ class L2populationMechanismDriver(api.MechanismDriver):
         self.L2populationAgentNotify.remove_fdb_entries(self.rpc_ctx,
             fdb_entries)
 
+    def filter_hosts_with_segment_access(
+            self, context, segments, candidate_hosts, agent_getter):
+        # NOTE(cbrandily): let other mechanisms (openvswitch, linuxbridge, ...)
+        # perform the filtering
+        return set()
+
     def _get_diff_ips(self, orig, port):
         orig_ips = set([ip['ip_address'] for ip in orig['fixed_ips']])
         port_ips = set([ip['ip_address'] for ip in port['fixed_ips']])
@@ -100,7 +111,7 @@ class L2populationMechanismDriver(api.MechanismDriver):
         if not agent_host:
             return
 
-        agent_ip = l2pop_db.get_agent_ip_by_host(db_api.get_session(),
+        agent_ip = l2pop_db.get_agent_ip_by_host(db_api.get_reader_session(),
                                                  agent_host)
 
         orig_mac_ip = [l2pop_rpc.PortInfo(mac_address=port['mac_address'],
@@ -130,14 +141,14 @@ class L2populationMechanismDriver(api.MechanismDriver):
 
         if (orig['mac_address'] != port['mac_address'] and
             context.status == const.PORT_STATUS_ACTIVE):
-            LOG.warning(_LW("unable to modify mac_address of ACTIVE port "
-                            "%s"), port['id'])
-            raise ml2_exc.MechanismDriverError(method='update_port_precommit')
+            msg = _("unable to modify mac_address of ACTIVE port "
+                    "%s") % port['id']
+            raise exceptions.InvalidInput(error_message=msg)
 
     def update_port_postcommit(self, context):
         port = context.current
         orig = context.original
-        if l3_hamode_db.is_ha_router_port(port['device_owner'],
+        if l3_hamode_db.is_ha_router_port(context, port['device_owner'],
                                           port['device_id']):
             return
         diff_ips = self._get_diff_ips(orig, port)
@@ -149,7 +160,8 @@ class L2populationMechanismDriver(api.MechanismDriver):
             if context.status == const.PORT_STATUS_DOWN:
                 agent_host = context.host
                 fdb_entries = self._get_agent_fdb(
-                        context.bottom_bound_segment, port, agent_host)
+                    context, context.bottom_bound_segment, port,
+                    agent_host)
                 self.L2populationAgentNotify.remove_fdb_entries(
                     self.rpc_ctx, fdb_entries)
         elif (context.host != context.original_host
@@ -158,7 +170,7 @@ class L2populationMechanismDriver(api.MechanismDriver):
             # The port has been migrated. Send notification about port
             # removal from old host.
             fdb_entries = self._get_agent_fdb(
-                context.original_bottom_bound_segment,
+                context, context.original_bottom_bound_segment,
                 orig, context.original_host)
             self.L2populationAgentNotify.remove_fdb_entries(
                 self.rpc_ctx, fdb_entries)
@@ -167,7 +179,8 @@ class L2populationMechanismDriver(api.MechanismDriver):
                 self.update_port_up(context)
             elif context.status == const.PORT_STATUS_DOWN:
                 fdb_entries = self._get_agent_fdb(
-                    context.bottom_bound_segment, port, context.host)
+                    context, context.bottom_bound_segment, port,
+                    context.host)
                 self.L2populationAgentNotify.remove_fdb_entries(
                     self.rpc_ctx, fdb_entries)
 
@@ -193,8 +206,8 @@ class L2populationMechanismDriver(api.MechanismDriver):
         tunnel_network_ports = (
             l2pop_db.get_distributed_active_network_ports(session, network_id))
         fdb_network_ports = (
-            l2pop_db.get_nondistributed_active_network_ports(
-                session, network_id))
+            l2pop_db.get_nondistributed_active_network_ports(session,
+                                                             network_id))
         ports = agent_fdb_entries[network_id]['ports']
         ports.update(self._get_tunnels(
             fdb_network_ports + tunnel_network_ports,
@@ -208,7 +221,7 @@ class L2populationMechanismDriver(api.MechanismDriver):
 
     def _get_tunnels(self, tunnel_network_ports, exclude_host):
         agents = {}
-        for _, agent in tunnel_network_ports:
+        for __, agent in tunnel_network_ports:
             if agent.host == exclude_host:
                 continue
 
@@ -226,8 +239,7 @@ class L2populationMechanismDriver(api.MechanismDriver):
     def update_port_down(self, context):
         port = context.current
         agent_host = context.host
-        l3plugin = manager.NeutronManager.get_service_plugins().get(
-            service_constants.L3_ROUTER_NAT)
+        l3plugin = directory.get_plugin(const.L3)
         # when agent transitions to backup, don't remove flood flows
         if agent_host and l3plugin and getattr(
             l3plugin, "list_router_ids_on_host", None):
@@ -236,14 +248,14 @@ class L2populationMechanismDriver(api.MechanismDriver):
                 admin_context, agent_host, [port['device_id']]):
                 return
             fdb_entries = self._get_agent_fdb(
-                context.bottom_bound_segment, port, agent_host)
+                context, context.bottom_bound_segment, port, agent_host)
             self.L2populationAgentNotify.remove_fdb_entries(
                 self.rpc_ctx, fdb_entries)
 
     def update_port_up(self, context):
         port = context.current
         agent_host = context.host
-        session = db_api.get_session()
+        session = db_api.get_reader_session()
         agent = l2pop_db.get_agent_by_host(session, agent_host)
         if not agent:
             LOG.warning(_LW("Unable to retrieve active L2 agent on host %s"),
@@ -281,24 +293,25 @@ class L2populationMechanismDriver(api.MechanismDriver):
 
         # Notify other agents to add fdb rule for current port
         if (port['device_owner'] != const.DEVICE_OWNER_DVR_INTERFACE and
-            not l3_hamode_db.is_ha_router_port(port['device_owner'],
-                                               port['device_id'])):
+            not l3_hamode_db.is_ha_router_port(
+                context, port['device_owner'], port['device_id'])):
             other_fdb_ports[agent_ip] += self._get_port_fdb_entries(port)
 
         self.L2populationAgentNotify.add_fdb_entries(self.rpc_ctx,
                                                      other_fdb_entries)
 
-    def _get_agent_fdb(self, segment, port, agent_host):
+    def _get_agent_fdb(self, context, segment, port, agent_host):
         if not agent_host:
             return
 
         network_id = port['network_id']
 
-        session = db_api.get_session()
+        session = db_api.get_reader_session()
         agent_active_ports = l2pop_db.get_agent_network_active_port_count(
             session, agent_host, network_id)
 
-        agent = l2pop_db.get_agent_by_host(db_api.get_session(), agent_host)
+        agent = l2pop_db.get_agent_by_host(session,
+                                           agent_host)
         if not self._validate_segment(segment, port['id'], agent):
             return
 
@@ -312,7 +325,8 @@ class L2populationMechanismDriver(api.MechanismDriver):
                 const.FLOODING_ENTRY)
         # Notify other agents to remove fdb rules for current port
         if (port['device_owner'] != const.DEVICE_OWNER_DVR_INTERFACE and
-            not l3_hamode_db.is_ha_router_port(port['device_owner'],
+            not l3_hamode_db.is_ha_router_port(context,
+                                               port['device_owner'],
                                                port['device_id'])):
             fdb_entries = self._get_port_fdb_entries(port)
             other_fdb_entries[network_id]['ports'][agent_ip] += fdb_entries
