@@ -23,6 +23,7 @@ import sys
 
 import netaddr
 from neutron_lib import constants
+from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
@@ -33,13 +34,11 @@ from six import moves
 from neutron._i18n import _LE, _LI, _LW
 from neutron.agent.linux import bridge_lib
 from neutron.agent.linux import ip_lib
-from neutron.agent.linux import utils
-from neutron.agent import securitygroups_rpc as sg_rpc
+from neutron.api.rpc.handlers import securitygroups_rpc as sg_rpc
 from neutron.common import config as common_config
 from neutron.common import exceptions
 from neutron.common import profiler as setup_profiler
 from neutron.common import topics
-from neutron.common import utils as n_utils
 from neutron.plugins.common import constants as p_const
 from neutron.plugins.common import utils as p_utils
 from neutron.plugins.ml2.drivers.agent import _agent_manager_base as amb
@@ -470,14 +469,11 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
 
         if network_type == p_const.TYPE_LOCAL:
             self.ensure_local_bridge(network_id, bridge_name)
-        else:
-            phy_dev_name = self.ensure_physical_in_bridge(network_id,
-                                                          network_type,
-                                                          physical_network,
-                                                          segmentation_id)
-            if not phy_dev_name:
-                return False
-            self.ensure_tap_mtu(tap_device_name, phy_dev_name)
+        elif not self.ensure_physical_in_bridge(network_id,
+                                                network_type,
+                                                physical_network,
+                                                segmentation_id):
+            return False
         # Avoid messing with plugging devices into a bridge that the agent
         # does not own
         if not device_owner.startswith(constants.DEVICE_OWNER_COMPUTE_PREFIX):
@@ -499,11 +495,6 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
                       "thus added elsewhere.", data)
         return True
 
-    def ensure_tap_mtu(self, tap_dev_name, phy_dev_name):
-        """Ensure the MTU on the tap is the same as the physical device."""
-        phy_dev_mtu = ip_lib.IPDevice(phy_dev_name).link.mtu
-        ip_lib.IPDevice(tap_dev_name).link.set_mtu(phy_dev_mtu)
-
     def plug_interface(self, network_id, network_segment, tap_name,
                        device_owner):
         return self.add_tap_interface(network_id, network_segment.network_type,
@@ -675,10 +666,10 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
         LOG.debug('Using %s VXLAN mode', self.vxlan_mode)
 
     def fdb_ip_entry_exists(self, mac, ip, interface):
-        entries = utils.execute(['ip', 'neigh', 'show', 'to', ip,
-                                 'dev', interface],
-                                run_as_root=True)
-        return mac in entries
+        ip_version = ip_lib.get_ip_version(ip)
+        entry = ip_lib.dump_neigh_entries(ip_version, interface, dst=ip,
+                                          lladdr=mac)
+        return entry != []
 
     def fdb_bridge_entry_exists(self, mac, interface, agent_ip=None):
         entries = bridge_lib.FdbInterface.show(interface)
@@ -689,11 +680,11 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
 
     def add_fdb_ip_entry(self, mac, ip, interface):
         if cfg.CONF.VXLAN.arp_responder:
-            ip_lib.IPDevice(interface).neigh.add(ip, mac)
+            ip_lib.add_neigh_entry(ip, mac, interface)
 
     def remove_fdb_ip_entry(self, mac, ip, interface):
         if cfg.CONF.VXLAN.arp_responder:
-            ip_lib.IPDevice(interface).neigh.delete(ip, mac)
+            ip_lib.delete_neigh_entry(ip, mac, interface)
 
     def add_fdb_entries(self, agent_ip, ports, interface):
         for mac, ip in ports:
@@ -721,14 +712,12 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
 
     def get_agent_id(self):
         if self.bridge_mappings:
-            mac = utils.get_interface_mac(
+            mac = ip_lib.get_device_mac(
                 list(self.bridge_mappings.values())[0])
         else:
             devices = ip_lib.IPWrapper().get_devices(True)
-            for device in devices:
-                mac = utils.get_interface_mac(device.name)
-                if mac:
-                    break
+            if devices:
+                mac = ip_lib.get_device_mac(devices[0].name)
             else:
                 LOG.error(_LE("Unable to obtain MAC address for unique ID. "
                               "Agent terminated!"))
@@ -913,7 +902,7 @@ def main():
 
     common_config.setup_logging()
     try:
-        interface_mappings = n_utils.parse_mappings(
+        interface_mappings = helpers.parse_mappings(
             cfg.CONF.LINUX_BRIDGE.physical_interface_mappings)
     except ValueError as e:
         LOG.error(_LE("Parsing physical_interface_mappings failed: %s. "
@@ -922,7 +911,7 @@ def main():
     LOG.info(_LI("Interface mappings: %s"), interface_mappings)
 
     try:
-        bridge_mappings = n_utils.parse_mappings(
+        bridge_mappings = helpers.parse_mappings(
             cfg.CONF.LINUX_BRIDGE.bridge_mappings)
     except ValueError as e:
         LOG.error(_LE("Parsing bridge_mappings failed: %s. "
