@@ -16,12 +16,12 @@
 import os
 import re
 
+from neutron_lib.utils import helpers
 from oslo_log import log as logging
 import six
 
 from neutron._i18n import _, _LE, _LW
 from neutron.agent.linux import ip_link_support
-from neutron.common import utils
 from neutron.plugins.ml2.drivers.mech_sriov.agent.common \
     import exceptions as exc
 from neutron.plugins.ml2.drivers.mech_sriov.agent import pci_lib
@@ -197,7 +197,7 @@ class EmbSwitch(object):
         if rate_kbps > 0 and rate_kbps < 1000:
             rate_mbps = 1
         else:
-            rate_mbps = utils.round_val(rate_kbps / 1000.0)
+            rate_mbps = helpers.round_val(rate_kbps / 1000.0)
 
         log_dict = {
             'rate_mbps': rate_mbps,
@@ -419,8 +419,26 @@ class ESwitchManager(object):
                 embedded_switch = None
         return embedded_switch
 
-    def clear_rate(self, pci_slot, rate_type):
-        """Clear the VF rate
+    def clear_max_rate(self, pci_slot):
+        """Clear the VF "rate" parameter
+
+        Clear the "rate" configuration from VF by setting it to 0.
+        @param pci_slot: VF PCI slot
+        """
+        self._clear_rate(pci_slot,
+            ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_RATE)
+
+    def clear_min_tx_rate(self, pci_slot):
+        """Clear the VF "min_tx_rate" parameter
+
+        Clear the "min_tx_rate" configuration from VF by setting it to 0.
+        @param pci_slot: VF PCI slot
+        """
+        self._clear_rate(pci_slot,
+            ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_MIN_TX_RATE)
+
+    def _clear_rate(self, pci_slot, rate_type):
+        """Clear the VF rate parameter specified in rate_type
 
         Clear the rate configuration from VF by setting it to 0.
         @param pci_slot: VF PCI slot
