@@ -13,24 +13,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.api import validators
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log as logging
 
 from neutron._i18n import _LE, _LI
-from neutron.api.v2 import attributes
 from neutron.callbacks import events
 from neutron.callbacks import registry
 from neutron.callbacks import resources
-from neutron.db import dns_db
 from neutron.db import models_v2
+from neutron.db import segments_db
 from neutron.extensions import dns
-from neutron import manager
+from neutron.objects import network as net_obj
+from neutron.objects import ports as port_obj
 from neutron.plugins.common import utils as plugin_utils
-from neutron.plugins.ml2 import db
 from neutron.plugins.ml2 import driver_api as api
 from neutron.services.externaldns import driver
 
 LOG = logging.getLogger(__name__)
+DNS_DOMAIN_DEFAULT = 'openstacklocal.'
 
 
 class DNSExtensionDriver(api.ExtensionDriver):
@@ -42,17 +44,18 @@ class DNSExtensionDriver(api.ExtensionDriver):
 
     def process_create_network(self, plugin_context, request_data, db_data):
         dns_domain = request_data.get(dns.DNSDOMAIN)
-        if not attributes.is_attr_set(dns_domain):
+        if not validators.is_attr_set(dns_domain):
             return
 
         if dns_domain:
-            plugin_context.session.add(dns_db.NetworkDNSDomain(
-                network_id=db_data['id'], dns_domain=dns_domain))
+            net_obj.NetworkDNSDomain(plugin_context,
+                                     network_id=db_data['id'],
+                                     dns_domain=dns_domain).create()
         db_data[dns.DNSDOMAIN] = dns_domain
 
     def process_update_network(self, plugin_context, request_data, db_data):
         new_value = request_data.get(dns.DNSDOMAIN)
-        if not attributes.is_attr_set(new_value):
+        if not validators.is_attr_set(new_value):
             return
 
         current_dns_domain = db_data.get(dns.DNSDOMAIN)
@@ -61,46 +64,51 @@ class DNSExtensionDriver(api.ExtensionDriver):
 
         net_id = db_data['id']
         if current_dns_domain:
-            net_dns_domain = plugin_context.session.query(
-                dns_db.NetworkDNSDomain).filter_by(network_id=net_id).one()
+            net_dns_domain = net_obj.NetworkDNSDomain.get_object(
+                plugin_context,
+                network_id=net_id)
             if new_value:
                 net_dns_domain['dns_domain'] = new_value
                 db_data[dns.DNSDOMAIN] = new_value
+                net_dns_domain.update()
             else:
-                plugin_context.session.delete(net_dns_domain)
+                net_dns_domain.delete()
                 db_data[dns.DNSDOMAIN] = ''
         elif new_value:
-            plugin_context.session.add(dns_db.NetworkDNSDomain(
-                network_id=net_id, dns_domain=new_value))
+            net_obj.NetworkDNSDomain(plugin_context,
+                                     network_id=net_id,
+                                     dns_domain=new_value).create()
             db_data[dns.DNSDOMAIN] = new_value
 
     def process_create_port(self, plugin_context, request_data, db_data):
-        if not request_data[dns.DNSNAME]:
+        if not request_data.get(dns.DNSNAME):
+            return
+        dns_name, is_dns_domain_default = self._get_request_dns_name(
+            request_data)
+        if is_dns_domain_default:
             return
         network = self._get_network(plugin_context, db_data['network_id'])
-        if not network[dns.DNSDOMAIN]:
-            return
-        if self.external_dns_not_needed(plugin_context, network):
-            return
-        plugin_context.session.add(dns_db.PortDNS(
-            port_id=db_data['id'],
-            current_dns_name=request_data[dns.DNSNAME],
-            current_dns_domain=network[dns.DNSDOMAIN],
-            previous_dns_name='', previous_dns_domain=''))
+        if self.external_dns_not_needed(
+                plugin_context, network) or not network[dns.DNSDOMAIN]:
+            current_dns_name = ''
+            current_dns_domain = ''
+        else:
+            current_dns_name = dns_name
+            current_dns_domain = network[dns.DNSDOMAIN]
 
-    def process_update_port(self, plugin_context, request_data, db_data):
-        dns_name = request_data.get(dns.DNSNAME)
-        has_fixed_ips = 'fixed_ips' in request_data
-        if dns_name is None and not has_fixed_ips:
-            return
-        network = self._get_network(plugin_context, db_data['network_id'])
-        if not network[dns.DNSDOMAIN]:
-            return
-        if self.external_dns_not_needed(plugin_context, network):
-            return
-        dns_domain = network[dns.DNSDOMAIN]
-        dns_data_db = plugin_context.session.query(dns_db.PortDNS).filter_by(
-            port_id=db_data['id']).one_or_none()
+        port_obj.PortDNS(plugin_context,
+                         port_id=db_data['id'],
+                         current_dns_name=current_dns_name,
+                         current_dns_domain=current_dns_domain,
+                         previous_dns_name='',
+                         previous_dns_domain='',
+                         dns_name=dns_name).create()
+
+    def _update_dns_db(self, dns_name, dns_domain, db_data,
+                      plugin_context, has_fixed_ips):
+        dns_data_db = port_obj.PortDNS.get_object(
+            plugin_context,
+            port_id=db_data['id'])
         if dns_data_db:
             is_dns_name_changed = (dns_name is not None and
                     dns_data_db['current_dns_name'] != dns_name)
@@ -112,18 +120,72 @@ class DNSExtensionDriver(api.ExtensionDriver):
                 dns_data_db['previous_dns_domain'] = (
                     dns_data_db['current_dns_domain'])
                 if is_dns_name_changed:
+                    dns_data_db[dns.DNSNAME] = dns_name
                     dns_data_db['current_dns_name'] = dns_name
                     if dns_name:
                         dns_data_db['current_dns_domain'] = dns_domain
                     else:
                         dns_data_db['current_dns_domain'] = ''
-            return
+
+            dns_data_db.update()
+            return dns_data_db
         if dns_name:
-            plugin_context.session.add(dns_db.PortDNS(
-                port_id=db_data['id'],
-                current_dns_name=dns_name,
-                current_dns_domain=dns_domain,
-                previous_dns_name='', previous_dns_domain=''))
+            dns_data_db = port_obj.PortDNS(plugin_context,
+                                           port_id=db_data['id'],
+                                           current_dns_name=dns_name,
+                                           current_dns_domain=dns_domain,
+                                           previous_dns_name='',
+                                           previous_dns_domain='',
+                                           dns_name=dns_name)
+            dns_data_db.create()
+        return dns_data_db
+
+    def process_update_port(self, plugin_context, request_data, db_data):
+        dns_name = request_data.get(dns.DNSNAME)
+        has_fixed_ips = 'fixed_ips' in request_data
+        if dns_name is None and not has_fixed_ips:
+            return
+        if dns_name is not None:
+            dns_name, is_dns_domain_default = self._get_request_dns_name(
+                request_data)
+            if is_dns_domain_default:
+                self._extend_port_dict(plugin_context.session, db_data,
+                                       db_data, None)
+                return
+        network = self._get_network(plugin_context, db_data['network_id'])
+        dns_domain = network[dns.DNSDOMAIN]
+        dns_data_db = None
+        if not dns_domain or self.external_dns_not_needed(plugin_context,
+                                                          network):
+            # No need to update external DNS service. Only process the port's
+            # dns_name attribute if necessary
+            if dns_name is not None:
+                dns_data_db = self._process_only_dns_name_update(
+                    plugin_context, db_data, dns_name)
+        else:
+            dns_data_db = self._update_dns_db(dns_name, dns_domain, db_data,
+                                              plugin_context, has_fixed_ips)
+        self._extend_port_dict(plugin_context.session, db_data, db_data,
+                               dns_data_db)
+
+    def _process_only_dns_name_update(self, plugin_context, db_data, dns_name):
+        dns_data_db = port_obj.PortDNS.get_object(
+            plugin_context,
+            port_id=db_data['id'])
+        if dns_data_db:
+            dns_data_db['dns_name'] = dns_name
+            dns_data_db.update()
+            return dns_data_db
+        if dns_name:
+            dns_data_db = port_obj.PortDNS(plugin_context,
+                                           port_id=db_data['id'],
+                                           current_dns_name='',
+                                           current_dns_domain='',
+                                           previous_dns_name='',
+                                           previous_dns_domain='',
+                                           dns_name=dns_name)
+            dns_data_db.create()
+        return dns_data_db
 
     def external_dns_not_needed(self, context, network):
         """Decide if ports in network need to be sent to the DNS service.
@@ -140,12 +202,69 @@ class DNSExtensionDriver(api.ExtensionDriver):
             response_data[dns.DNSDOMAIN] = db_data.dns_domain[dns.DNSDOMAIN]
         return response_data
 
-    def extend_port_dict(self, session, db_data, response_data):
-        response_data[dns.DNSNAME] = db_data[dns.DNSNAME]
+    def _get_dns_domain(self):
+        if not cfg.CONF.dns_domain:
+            return ''
+        if cfg.CONF.dns_domain.endswith('.'):
+            return cfg.CONF.dns_domain
+        return '%s.' % cfg.CONF.dns_domain
+
+    def _get_request_dns_name(self, port):
+        dns_domain = self._get_dns_domain()
+        if ((dns_domain and dns_domain != DNS_DOMAIN_DEFAULT)):
+            return (port.get(dns.DNSNAME, ''), False)
+        return ('', True)
+
+    def _get_request_dns_name_and_domain_name(self, dns_data_db):
+        dns_domain = self._get_dns_domain()
+        dns_name = ''
+        if ((dns_domain and dns_domain != DNS_DOMAIN_DEFAULT)):
+            if dns_data_db:
+                dns_name = dns_data_db.dns_name
+        return dns_name, dns_domain
+
+    def _get_dns_names_for_port(self, ips, dns_data_db):
+        dns_assignment = []
+        dns_name, dns_domain = self._get_request_dns_name_and_domain_name(
+            dns_data_db)
+        for ip in ips:
+            if dns_name:
+                hostname = dns_name
+                fqdn = dns_name
+                if not dns_name.endswith('.'):
+                    fqdn = '%s.%s' % (dns_name, dns_domain)
+            else:
+                hostname = 'host-%s' % ip['ip_address'].replace(
+                    '.', '-').replace(':', '-')
+                fqdn = hostname
+                if dns_domain:
+                    fqdn = '%s.%s' % (hostname, dns_domain)
+            dns_assignment.append({'ip_address': ip['ip_address'],
+                                   'hostname': hostname,
+                                   'fqdn': fqdn})
+        return dns_assignment
+
+    def _get_dns_name_for_port_get(self, port, dns_data_db):
+        if port['fixed_ips']:
+            return self._get_dns_names_for_port(port['fixed_ips'], dns_data_db)
+        return []
+
+    def _extend_port_dict(self, session, db_data, response_data, dns_data_db):
+        if not dns_data_db:
+            response_data[dns.DNSNAME] = ''
+        else:
+            response_data[dns.DNSNAME] = dns_data_db[dns.DNSNAME]
+        response_data['dns_assignment'] = self._get_dns_name_for_port_get(
+            db_data, dns_data_db)
         return response_data
 
+    def extend_port_dict(self, session, db_data, response_data):
+        dns_data_db = db_data.dns
+        return self._extend_port_dict(session, db_data, response_data,
+                                      dns_data_db)
+
     def _get_network(self, context, network_id):
-        plugin = manager.NeutronManager.get_plugin()
+        plugin = directory.get_plugin()
         return plugin.get_network(context, network_id)
 
 
@@ -187,7 +306,7 @@ class DNSExtensionDriverML2(DNSExtensionDriver):
             return True
         if network['router:external']:
             return True
-        segments = db.get_network_segments(context.session, network['id'])
+        segments = segments_db.get_network_segments(context, network['id'])
         if len(segments) > 1:
             return False
         provider_net = segments[0]
@@ -229,9 +348,9 @@ def _create_port_in_external_dns_service(resource, event, trigger, **kwargs):
         return
     context = kwargs['context']
     port = kwargs['port']
-    dns_data_db = context.session.query(dns_db.PortDNS).filter_by(
-        port_id=port['id']).one_or_none()
-    if not dns_data_db:
+    dns_data_db = port_obj.PortDNS.get_object(
+        context, port_id=port['id'])
+    if not (dns_data_db and dns_data_db['current_dns_name']):
         return
     records = [ip['ip_address'] for ip in port['fixed_ips']]
     _send_data_to_external_dns_service(context, dns_driver,
@@ -247,10 +366,10 @@ def _send_data_to_external_dns_service(context, dns_driver, dns_domain,
     except (dns.DNSDomainNotFound, dns.DuplicateRecordSet) as e:
         LOG.exception(_LE("Error publishing port data in external DNS "
                           "service. Name: '%(name)s'. Domain: '%(domain)s'. "
-                          "DNS service driver message '%(message)s'")
-                      % {"name": dns_name,
-                         "domain": dns_domain,
-                         "message": e.msg})
+                          "DNS service driver message '%(message)s'"),
+                      {"name": dns_name,
+                       "domain": dns_domain,
+                       "message": e.msg})
 
 
 def _remove_data_from_external_dns_service(context, dns_driver, dns_domain,
@@ -261,11 +380,11 @@ def _remove_data_from_external_dns_service(context, dns_driver, dns_domain,
         LOG.exception(_LE("Error deleting port data from external DNS "
                           "service. Name: '%(name)s'. Domain: '%(domain)s'. "
                           "IP addresses '%(ips)s'. DNS service driver message "
-                          "'%(message)s'")
-                      % {"name": dns_name,
-                         "domain": dns_domain,
-                         "message": e.msg,
-                         "ips": ', '.join(records)})
+                          "'%(message)s'"),
+                      {"name": dns_name,
+                       "domain": dns_domain,
+                       "message": e.msg,
+                       "ips": ', '.join(records)})
 
 
 def _update_port_in_external_dns_service(resource, event, trigger, **kwargs):
@@ -285,9 +404,10 @@ def _update_port_in_external_dns_service(resource, event, trigger, **kwargs):
     if (updated_port[dns.DNSNAME] == original_port[dns.DNSNAME] and
             not original_port[dns.DNSNAME]):
         return
-    dns_data_db = context.session.query(dns_db.PortDNS).filter_by(
-        port_id=updated_port['id']).one_or_none()
-    if not dns_data_db:
+    dns_data_db = port_obj.PortDNS.get_object(
+        context, port_id=updated_port['id'])
+    if not (dns_data_db and (dns_data_db['previous_dns_name'] or dns_data_db[
+        'current_dns_name'])):
         return
     if dns_data_db['previous_dns_name']:
         _remove_data_from_external_dns_service(
@@ -306,8 +426,8 @@ def _delete_port_in_external_dns_service(resource, event, trigger, **kwargs):
         return
     context = kwargs['context']
     port_id = kwargs['port_id']
-    dns_data_db = context.session.query(dns_db.PortDNS).filter_by(
-        port_id=port_id).one_or_none()
+    dns_data_db = port_obj.PortDNS.get_object(
+        context, port_id=port_id)
     if not dns_data_db:
         return
     if dns_data_db['current_dns_name']:
