@@ -13,6 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.api import validators
+from neutron_lib import constants
+from neutron_lib import exceptions as exc
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import excutils
@@ -20,19 +23,16 @@ import six
 import stevedore
 
 from neutron._i18n import _, _LE, _LI, _LW
-from neutron.api.v2 import attributes
-from neutron.common import exceptions as exc
 from neutron.db import api as db_api
+from neutron.db import segments_db
 from neutron.extensions import external_net
 from neutron.extensions import multiprovidernet as mpnet
 from neutron.extensions import portbindings
 from neutron.extensions import providernet as provider
 from neutron.extensions import vlantransparent
 from neutron.plugins.ml2.common import exceptions as ml2_exc
-from neutron.plugins.ml2 import db
 from neutron.plugins.ml2 import driver_api as api
 from neutron.plugins.ml2 import models
-from neutron.services.qos import qos_consts
 
 LOG = log.getLogger(__name__)
 
@@ -92,7 +92,7 @@ class TypeManager(stevedore.named.NamedExtensionManager):
          segmentation_id) = (self._get_attribute(segment, attr)
                              for attr in provider.ATTRIBUTES)
 
-        if attributes.is_attr_set(network_type):
+        if validators.is_attr_set(network_type):
             segment = {api.NETWORK_TYPE: network_type,
                        api.PHYSICAL_NETWORK: physical_network,
                        api.SEGMENTATION_ID: segmentation_id}
@@ -103,15 +103,15 @@ class TypeManager(stevedore.named.NamedExtensionManager):
         raise exc.InvalidInput(error_message=msg)
 
     def _process_provider_create(self, network):
-        if any(attributes.is_attr_set(network.get(attr))
+        if any(validators.is_attr_set(network.get(attr))
                for attr in provider.ATTRIBUTES):
             # Verify that multiprovider and provider attributes are not set
             # at the same time.
-            if attributes.is_attr_set(network.get(mpnet.SEGMENTS)):
+            if validators.is_attr_set(network.get(mpnet.SEGMENTS)):
                 raise mpnet.SegmentsSetInConjunctionWithProviders()
             segment = self._get_provider_segment(network)
             return [self._process_provider_segment(segment)]
-        elif attributes.is_attr_set(network.get(mpnet.SEGMENTS)):
+        elif validators.is_attr_set(network.get(mpnet.SEGMENTS)):
             segments = [self._process_provider_segment(s)
                         for s in network[mpnet.SEGMENTS]]
             mpnet.check_duplicate_segments(segments, self.is_partial_segment)
@@ -133,10 +133,10 @@ class TypeManager(stevedore.named.NamedExtensionManager):
     def network_matches_filters(self, network, filters):
         if not filters:
             return True
-        if any(attributes.is_attr_set(network.get(attr))
+        if any(validators.is_attr_set(network.get(attr))
                for attr in provider.ATTRIBUTES):
             segments = [self._get_provider_segment(network)]
-        elif attributes.is_attr_set(network.get(mpnet.SEGMENTS)):
+        elif validators.is_attr_set(network.get(mpnet.SEGMENTS)):
             segments = self._get_attribute(network, mpnet.SEGMENTS)
         else:
             return True
@@ -144,7 +144,7 @@ class TypeManager(stevedore.named.NamedExtensionManager):
 
     def _get_attribute(self, attrs, key):
         value = attrs.get(key)
-        if value is attributes.ATTR_NOT_SPECIFIED:
+        if value is constants.ATTR_NOT_SPECIFIED:
             value = None
         return value
 
@@ -155,7 +155,7 @@ class TypeManager(stevedore.named.NamedExtensionManager):
 
     def extend_networks_dict_provider(self, context, networks):
         ids = [network['id'] for network in networks]
-        net_segments = db.get_networks_segments(context.session, ids)
+        net_segments = segments_db.get_networks_segments(context, ids)
         for network in networks:
             segments = net_segments[network['id']]
             self._extend_network_dict_provider(network, segments)
@@ -182,33 +182,50 @@ class TypeManager(stevedore.named.NamedExtensionManager):
             LOG.info(_LI("Initializing driver for type '%s'"), network_type)
             driver.obj.initialize()
 
-    def _add_network_segment(self, session, network_id, segment, mtu,
+    def _add_network_segment(self, context, network_id, segment,
                              segment_index=0):
-        db.add_network_segment(session, network_id, segment, segment_index)
-        if segment.get(api.MTU, 0) > 0:
-            mtu.append(segment[api.MTU])
+        segments_db.add_network_segment(
+            context, network_id, segment, segment_index)
 
     def create_network_segments(self, context, network, tenant_id):
         """Call type drivers to create network segments."""
         segments = self._process_provider_create(network)
         session = context.session
-        mtu = []
         with session.begin(subtransactions=True):
             network_id = network['id']
             if segments:
                 for segment_index, segment in enumerate(segments):
                     segment = self.reserve_provider_segment(
-                        session, segment)
-                    self._add_network_segment(session, network_id, segment,
-                                              mtu, segment_index)
+                        context, segment)
+                    self._add_network_segment(context, network_id, segment,
+                                              segment_index)
             elif (cfg.CONF.ml2.external_network_type and
                   self._get_attribute(network, external_net.EXTERNAL)):
-                segment = self._allocate_ext_net_segment(session)
-                self._add_network_segment(session, network_id, segment, mtu)
+                segment = self._allocate_ext_net_segment(context)
+                self._add_network_segment(context, network_id, segment)
             else:
-                segment = self._allocate_tenant_net_segment(session)
-                self._add_network_segment(session, network_id, segment, mtu)
-        network[api.MTU] = min(mtu) if mtu else 0
+                segment = self._allocate_tenant_net_segment(context)
+                self._add_network_segment(context, network_id, segment)
+
+    def reserve_network_segment(self, context, segment_data):
+        """Call type drivers to reserve a network segment."""
+        # Validate the data of segment
+        if not validators.is_attr_set(segment_data[api.NETWORK_TYPE]):
+            msg = _("network_type required")
+            raise exc.InvalidInput(error_message=msg)
+
+        net_type = self._get_attribute(segment_data, api.NETWORK_TYPE)
+        phys_net = self._get_attribute(segment_data, api.PHYSICAL_NETWORK)
+        seg_id = self._get_attribute(segment_data, api.SEGMENTATION_ID)
+        segment = {api.NETWORK_TYPE: net_type,
+                   api.PHYSICAL_NETWORK: phys_net,
+                   api.SEGMENTATION_ID: seg_id}
+
+        self.validate_provider_segment(segment)
+
+        # Reserve segment in type driver
+        with context.session.begin(subtransactions=True):
+            return self.reserve_provider_segment(context, segment)
 
     def is_partial_segment(self, segment):
         network_type = segment[api.NETWORK_TYPE]
@@ -228,65 +245,87 @@ class TypeManager(stevedore.named.NamedExtensionManager):
             msg = _("network_type value '%s' not supported") % network_type
             raise exc.InvalidInput(error_message=msg)
 
-    def reserve_provider_segment(self, session, segment):
+    def reserve_provider_segment(self, context, segment):
         network_type = segment.get(api.NETWORK_TYPE)
         driver = self.drivers.get(network_type)
-        return driver.obj.reserve_provider_segment(session, segment)
+        if isinstance(driver.obj, api.TypeDriver):
+            return driver.obj.reserve_provider_segment(context.session,
+                                                       segment)
+        else:
+            return driver.obj.reserve_provider_segment(context,
+                                                       segment)
 
-    def _allocate_segment(self, session, network_type):
+    def _allocate_segment(self, context, network_type):
         driver = self.drivers.get(network_type)
-        return driver.obj.allocate_tenant_segment(session)
+        if isinstance(driver.obj, api.TypeDriver):
+            return driver.obj.allocate_tenant_segment(context.session)
+        else:
+            return driver.obj.allocate_tenant_segment(context)
 
-    def _allocate_tenant_net_segment(self, session):
+    def _allocate_tenant_net_segment(self, context):
         for network_type in self.tenant_network_types:
-            segment = self._allocate_segment(session, network_type)
+            segment = self._allocate_segment(context, network_type)
             if segment:
                 return segment
         raise exc.NoNetworkAvailable()
 
-    def _allocate_ext_net_segment(self, session):
+    def _allocate_ext_net_segment(self, context):
         network_type = cfg.CONF.ml2.external_network_type
-        segment = self._allocate_segment(session, network_type)
+        segment = self._allocate_segment(context, network_type)
         if segment:
             return segment
         raise exc.NoNetworkAvailable()
 
-    def release_network_segments(self, session, network_id):
-        segments = db.get_network_segments(session, network_id,
-                                           filter_dynamic=None)
+    def release_network_segments(self, context, network_id):
+        segments = segments_db.get_network_segments(context, network_id,
+                                                    filter_dynamic=None)
 
         for segment in segments:
-            network_type = segment.get(api.NETWORK_TYPE)
-            driver = self.drivers.get(network_type)
-            if driver:
-                driver.obj.release_segment(session, segment)
-            else:
-                LOG.error(_LE("Failed to release segment '%s' because "
-                              "network type is not supported."), segment)
+            self.release_network_segment(context, segment)
 
-    def allocate_dynamic_segment(self, session, network_id, segment):
+    def release_network_segment(self, context, segment):
+        network_type = segment.get(api.NETWORK_TYPE)
+        driver = self.drivers.get(network_type)
+        if driver:
+            if isinstance(driver.obj, api.TypeDriver):
+                driver.obj.release_segment(context.session, segment)
+            else:
+                driver.obj.release_segment(context, segment)
+        else:
+            LOG.error(_LE("Failed to release segment '%s' because "
+                          "network type is not supported."), segment)
+
+    def allocate_dynamic_segment(self, context, network_id, segment):
         """Allocate a dynamic segment using a partial or full segment dict."""
-        dynamic_segment = db.get_dynamic_segment(
-            session, network_id, segment.get(api.PHYSICAL_NETWORK),
+        dynamic_segment = segments_db.get_dynamic_segment(
+            context, network_id, segment.get(api.PHYSICAL_NETWORK),
             segment.get(api.SEGMENTATION_ID))
 
         if dynamic_segment:
             return dynamic_segment
 
         driver = self.drivers.get(segment.get(api.NETWORK_TYPE))
-        dynamic_segment = driver.obj.reserve_provider_segment(session, segment)
-        db.add_network_segment(session, network_id, dynamic_segment,
-                               is_dynamic=True)
+        if isinstance(driver.obj, api.TypeDriver):
+            dynamic_segment = driver.obj.reserve_provider_segment(
+                context.session, segment)
+        else:
+            dynamic_segment = driver.obj.reserve_provider_segment(
+                context, segment)
+        segments_db.add_network_segment(context, network_id, dynamic_segment,
+                                        is_dynamic=True)
         return dynamic_segment
 
-    def release_dynamic_segment(self, session, segment_id):
+    def release_dynamic_segment(self, context, segment_id):
         """Delete a dynamic segment."""
-        segment = db.get_segment_by_id(session, segment_id)
+        segment = segments_db.get_segment_by_id(context, segment_id)
         if segment:
             driver = self.drivers.get(segment.get(api.NETWORK_TYPE))
             if driver:
-                driver.obj.release_segment(session, segment)
-                db.delete_network_segment(session, segment_id)
+                if isinstance(driver.obj, api.TypeDriver):
+                    driver.obj.release_segment(context.session, segment)
+                else:
+                    driver.obj.release_segment(context, segment)
+                segments_db.delete_network_segment(context, segment_id)
             else:
                 LOG.error(_LE("Failed to release segment '%s' because "
                               "network type is not supported."), segment)
@@ -312,6 +351,10 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
                                                name_order=True)
         LOG.info(_LI("Loaded mechanism driver names: %s"), self.names())
         self._register_mechanisms()
+        self.host_filtering_supported = self.is_host_filtering_supported()
+        if not self.host_filtering_supported:
+            LOG.info(_LI("No mechanism drivers provide segment reachability "
+                         "information for agent scheduling."))
 
     def _register_mechanisms(self):
         """Register all mechanism drivers.
@@ -325,48 +368,6 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         LOG.info(_LI("Registered mechanism drivers: %s"),
                  [driver.name for driver in self.ordered_mech_drivers])
 
-    @property
-    def supported_qos_rule_types(self):
-        if not self.ordered_mech_drivers:
-            return []
-
-        rule_types = set(qos_consts.VALID_RULE_TYPES)
-        binding_driver_found = False
-
-        # Recalculate on every call to allow drivers determine supported rule
-        # types dynamically
-        for driver in self.ordered_mech_drivers:
-            driver_obj = driver.obj
-            if driver_obj._supports_port_binding:
-                binding_driver_found = True
-                if hasattr(driver_obj, 'supported_qos_rule_types'):
-                    new_rule_types = \
-                        rule_types & set(driver_obj.supported_qos_rule_types)
-                    dropped_rule_types = new_rule_types - rule_types
-                    if dropped_rule_types:
-                        LOG.info(
-                            _LI("%(rule_types)s rule types disabled for ml2 "
-                                "because %(driver)s does not support them"),
-                            {'rule_types': ', '.join(dropped_rule_types),
-                             'driver': driver.name})
-                    rule_types = new_rule_types
-                else:
-                    # at least one of drivers does not support QoS, meaning
-                    # there are no rule types supported by all of them
-                    LOG.warning(
-                        _LW("%s does not support QoS; "
-                            "no rule types available"),
-                        driver.name)
-                    return []
-
-        if binding_driver_found:
-            rule_types = list(rule_types)
-        else:
-            rule_types = []
-        LOG.debug("Supported QoS rule types "
-                  "(common subset for all mech drivers): %s", rule_types)
-        return rule_types
-
     def initialize(self):
         for driver in self.ordered_mech_drivers:
             LOG.info(_LI("Initializing mechanism driver '%s'"), driver.name)
@@ -401,7 +402,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         raise_db_retriable=False. See neutron.db.api.is_retriable for
         what db exception is retriable
         """
-        error = False
+        errors = []
         for driver in self.ordered_mech_drivers:
             try:
                 getattr(driver.obj, method_name)(context)
@@ -416,12 +417,13 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
                     _LE("Mechanism driver '%(name)s' failed in %(method)s"),
                     {'name': driver.name, 'method': method_name}
                 )
-                error = True
+                errors.append(e)
                 if not continue_on_failure:
                     break
-        if error:
+        if errors:
             raise ml2_exc.MechanismDriverError(
-                method=method_name
+                method=method_name,
+                errors=errors
             )
 
     def create_network_precommit(self, context):
@@ -433,7 +435,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         if any mechanism driver create_network_precommit call fails.
 
         Called within the database transaction. If a mechanism driver
-        raises an exception, then a MechanismDriverError is propogated
+        raises an exception, then a MechanismDriverError is propagated
         to the caller, triggering a rollback. There is no guarantee
         that all mechanism drivers are called in this case.
         """
@@ -464,7 +466,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         if any mechanism driver update_network_precommit call fails.
 
         Called within the database transaction. If a mechanism driver
-        raises an exception, then a MechanismDriverError is propogated
+        raises an exception, then a MechanismDriverError is propagated
         to the caller, triggering a rollback. There is no guarantee
         that all mechanism drivers are called in this case.
         """
@@ -494,7 +496,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         if any mechanism driver delete_network_precommit call fails.
 
         Called within the database transaction. If a mechanism driver
-        raises an exception, then a MechanismDriverError is propogated
+        raises an exception, then a MechanismDriverError is propagated
         to the caller, triggering a rollback. There is no guarantee
         that all mechanism drivers are called in this case.
         """
@@ -528,7 +530,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         if any mechanism driver create_subnet_precommit call fails.
 
         Called within the database transaction. If a mechanism driver
-        raises an exception, then a MechanismDriverError is propogated
+        raises an exception, then a MechanismDriverError is propagated
         to the caller, triggering a rollback. There is no guarantee
         that all mechanism drivers are called in this case.
         """
@@ -558,7 +560,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         if any mechanism driver update_subnet_precommit call fails.
 
         Called within the database transaction. If a mechanism driver
-        raises an exception, then a MechanismDriverError is propogated
+        raises an exception, then a MechanismDriverError is propagated
         to the caller, triggering a rollback. There is no guarantee
         that all mechanism drivers are called in this case.
         """
@@ -588,7 +590,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         if any mechanism driver delete_subnet_precommit call fails.
 
         Called within the database transaction. If a mechanism driver
-        raises an exception, then a MechanismDriverError is propogated
+        raises an exception, then a MechanismDriverError is propagated
         to the caller, triggering a rollback. There is no guarantee
         that all mechanism drivers are called in this case.
         """
@@ -622,7 +624,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         if any mechanism driver create_port_precommit call fails.
 
         Called within the database transaction. If a mechanism driver
-        raises an exception, then a MechanismDriverError is propogated
+        raises an exception, then a MechanismDriverError is propagated
         to the caller, triggering a rollback. There is no guarantee
         that all mechanism drivers are called in this case.
         """
@@ -652,7 +654,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         if any mechanism driver update_port_precommit call fails.
 
         Called within the database transaction. If a mechanism driver
-        raises an exception, then a MechanismDriverError is propogated
+        raises an exception, then a MechanismDriverError is propagated
         to the caller, triggering a rollback. There is no guarantee
         that all mechanism drivers are called in this case.
         """
@@ -682,7 +684,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
         if any mechanism driver delete_port_precommit call fails.
 
         Called within the database transaction. If a mechanism driver
-        raises an exception, then a MechanismDriverError is propogated
+        raises an exception, then a MechanismDriverError is propagated
         to the caller, triggering a rollback. There is no guarantee
         that all mechanism drivers are called in this case.
         """
@@ -797,15 +799,44 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
                                   "bind_port"),
                               driver.name)
 
+    def is_host_filtering_supported(self):
+        return all(driver.obj.is_host_filtering_supported()
+                   for driver in self.ordered_mech_drivers)
+
+    def filter_hosts_with_segment_access(
+            self, context, segments, candidate_hosts, agent_getter):
+        """Filter hosts with access to at least one segment.
+
+        :returns: a subset of candidate_hosts.
+
+        This method returns all hosts from candidate_hosts with access to a
+        segment according to at least one driver.
+        """
+        candidate_hosts = set(candidate_hosts)
+        if not self.host_filtering_supported:
+            return candidate_hosts
+
+        hosts_with_access = set()
+        for driver in self.ordered_mech_drivers:
+            hosts = driver.obj.filter_hosts_with_segment_access(
+                context, segments, candidate_hosts, agent_getter)
+            hosts_with_access |= hosts
+            candidate_hosts -= hosts
+            if not candidate_hosts:
+                break
+        return hosts_with_access
+
     def _check_driver_to_bind(self, driver, segments_to_bind, binding_levels):
         # To prevent a possible binding loop, don't try to bind with
         # this driver if the same driver has already bound at a higher
         # level to one of the segments we are currently trying to
         # bind. Note that it is OK for the same driver to bind at
         # multiple levels using different segments.
+        segment_ids_to_bind = {s[api.SEGMENTATION_ID]
+                               for s in segments_to_bind}
         for level in binding_levels:
             if (level.driver == driver and
-                level.segment_id in segments_to_bind):
+                level.segment_id in segment_ids_to_bind):
                 return False
         return True
 
