@@ -16,6 +16,9 @@
 import collections
 import re
 
+from neutron_lib import constants
+from neutron_lib import exceptions
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
@@ -26,8 +29,8 @@ import six
 
 from neutron._i18n import _, _LE, _LW
 from neutron.api.v2 import attributes
+from neutron.common import cache_utils as cache
 from neutron.common import constants as const
-from neutron.common import exceptions
 
 
 LOG = logging.getLogger(__name__)
@@ -89,9 +92,9 @@ def _is_attribute_explicitly_set(attribute_name, resource, target, action):
         # default value of an attribute, but check whether it was explicitly
         # marked as being updated instead.
         return (attribute_name in target[const.ATTRIBUTES_TO_UPDATE] and
-                target[attribute_name] is not attributes.ATTR_NOT_SPECIFIED)
+                target[attribute_name] is not constants.ATTR_NOT_SPECIFIED)
     result = (attribute_name in target and
-              target[attribute_name] is not attributes.ATTR_NOT_SPECIFIED)
+              target[attribute_name] is not constants.ATTR_NOT_SPECIFIED)
     if result and 'default' in resource[attribute_name]:
         return target[attribute_name] != resource[attribute_name]['default']
     return result
@@ -111,7 +114,7 @@ def _build_subattr_match_rule(attr_name, attr, action, target):
     # typing for API attributes
     # Expect a dict as type descriptor
     validate = attr['validate']
-    key = list(filter(lambda k: k.startswith('type:dict'), validate.keys()))
+    key = [k for k in validate.keys() if k.startswith('type:dict')]
     if not key:
         LOG.warning(_LW("Unable to find data type descriptor "
                         "for attribute %s"),
@@ -207,8 +210,35 @@ class OwnerCheck(policy.Check):
             raise exceptions.PolicyInitError(
                 policy="%s:%s" % (kind, match),
                 reason=err_reason)
+        self._cache = cache._get_memory_cache_region(expiration_time=5)
         super(OwnerCheck, self).__init__(kind, match)
 
+    @cache.cache_method_results
+    def _extract(self, resource_type, resource_id, field):
+        # NOTE(salv-orlando): This check currently assumes the parent
+        # resource is handled by the core plugin. It might be worth
+        # having a way to map resources to plugins so to make this
+        # check more general
+        f = getattr(directory.get_plugin(), 'get_%s' % resource_type)
+        # f *must* exist, if not found it is better to let neutron
+        # explode. Check will be performed with admin context
+        context = importutils.import_module('neutron.context')
+        try:
+            data = f(context.get_admin_context(),
+                     resource_id,
+                     fields=[field])
+        except exceptions.NotFound as e:
+            # NOTE(kevinbenton): a NotFound exception can occur if a
+            # list operation is happening at the same time as one of
+            # the parents and its children being deleted. So we issue
+            # a RetryRequest so the API will redo the lookup and the
+            # problem items will be gone.
+            raise db_exc.RetryRequest(e)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                LOG.exception(_LE('Policy check error while calling %s!'), f)
+        return data[field]
+
     def __call__(self, target, creds, enforcer):
         if self.target_field not in target:
             # policy needs a plugin check
@@ -246,34 +276,10 @@ class OwnerCheck(policy.Check):
                 raise exceptions.PolicyCheckError(
                     policy="%s:%s" % (self.kind, self.match),
                     reason=err_reason)
-            # NOTE(salv-orlando): This check currently assumes the parent
-            # resource is handled by the core plugin. It might be worth
-            # having a way to map resources to plugins so to make this
-            # check more general
-            # NOTE(ihrachys): if import is put in global, circular
-            # import failure occurs
-            manager = importutils.import_module('neutron.manager')
-            f = getattr(manager.NeutronManager.get_instance().plugin,
-                        'get_%s' % parent_res)
-            # f *must* exist, if not found it is better to let neutron
-            # explode. Check will be performed with admin context
-            context = importutils.import_module('neutron.context')
-            try:
-                data = f(context.get_admin_context(),
-                         target[parent_foreign_key],
-                         fields=[parent_field])
-                target[self.target_field] = data[parent_field]
-            except exceptions.NotFound as e:
-                # NOTE(kevinbenton): a NotFound exception can occur if a
-                # list operation is happening at the same time as one of
-                # the parents and its children being deleted. So we issue
-                # a RetryRequest so the API will redo the lookup and the
-                # problem items will be gone.
-                raise db_exc.RetryRequest(e)
-            except Exception:
-                with excutils.save_and_reraise_exception():
-                    LOG.exception(_LE('Policy check error while calling %s!'),
-                                  f)
+
+            target[self.target_field] = self._extract(
+                parent_res, target[parent_foreign_key], parent_field)
+
         match = self.match % target
         if self.kind in creds:
             return match == six.text_type(creds[self.kind])
@@ -320,7 +326,7 @@ def _prepare_check(context, action, target, pluralized):
     if target is None:
         target = {}
     match_rule = _build_match_rule(action, target, pluralized)
-    credentials = context.to_dict()
+    credentials = context.to_policy_values()
     return match_rule, target, credentials
 
 
@@ -409,7 +415,7 @@ def check_is_admin(context):
     """Verify context has admin rights according to policy settings."""
     init()
     # the target is user-self
-    credentials = context.to_dict()
+    credentials = context.to_policy_values()
     if ADMIN_CTX_POLICY not in _ENFORCER.rules:
         return False
     return _ENFORCER.enforce(ADMIN_CTX_POLICY, credentials, credentials)
@@ -419,7 +425,7 @@ def check_is_advsvc(context):
     """Verify context has advsvc rights according to policy settings."""
     init()
     # the target is user-self
-    credentials = context.to_dict()
+    credentials = context.to_policy_values()
     if ADVSVC_CTX_POLICY not in _ENFORCER.rules:
         return False
     return _ENFORCER.enforce(ADVSVC_CTX_POLICY, credentials, credentials)
