@@ -219,11 +219,11 @@ class TrunkPlugin(service_base.ServicePluginBase,
                                         id=uuidutils.generate_uuid(),
                                         name=trunk.get('name', ""),
                                         description=trunk_description,
-                                        tenant_id=trunk['tenant_id'],
+                                        project_id=trunk['tenant_id'],
                                         port_id=trunk['port_id'],
                                         status=constants.DOWN_STATUS,
                                         sub_ports=sub_ports)
-        with db_api.autonested_transaction(context.session):
+        with db_api.context_manager.writer.using(context):
             trunk_obj.create()
             payload = callbacks.TrunkPayload(context, trunk_obj.id,
                                              current_trunk=trunk_obj)
@@ -238,7 +238,7 @@ class TrunkPlugin(service_base.ServicePluginBase,
     def update_trunk(self, context, trunk_id, trunk):
         """Update information for the specified trunk."""
         trunk_data = trunk['trunk']
-        with db_api.autonested_transaction(context.session):
+        with db_api.context_manager.writer.using(context):
             trunk_obj = self._get_trunk(context, trunk_id)
             original_trunk = copy.deepcopy(trunk_obj)
             # NOTE(status_police): a trunk status should not change during an
@@ -258,7 +258,7 @@ class TrunkPlugin(service_base.ServicePluginBase,
 
     def delete_trunk(self, context, trunk_id):
         """Delete the specified trunk."""
-        with db_api.autonested_transaction(context.session):
+        with db_api.context_manager.writer.using(context):
             trunk = self._get_trunk(context, trunk_id)
             rules.trunk_can_be_managed(context, trunk)
             trunk_port_validator = rules.TrunkPortValidator(trunk.port_id)
@@ -280,7 +280,7 @@ class TrunkPlugin(service_base.ServicePluginBase,
     @db_base_plugin_common.convert_result_to_dict
     def add_subports(self, context, trunk_id, subports):
         """Add one or more subports to trunk."""
-        with db_api.autonested_transaction(context.session):
+        with db_api.context_manager.writer.using(context):
             trunk = self._get_trunk(context, trunk_id)
 
             # Check for basic validation since the request body here is not
@@ -334,7 +334,7 @@ class TrunkPlugin(service_base.ServicePluginBase,
     def remove_subports(self, context, trunk_id, subports):
         """Remove one or more subports from trunk."""
         subports = subports['sub_ports']
-        with db_api.autonested_transaction(context.session):
+        with db_api.context_manager.writer.using(context):
             trunk = self._get_trunk(context, trunk_id)
             original_trunk = copy.deepcopy(trunk)
             rules.trunk_can_be_managed(context, trunk)
@@ -409,8 +409,9 @@ class TrunkPlugin(service_base.ServicePluginBase,
         new_vif_type = updated_port.get(portbindings.VIF_TYPE)
         vif_type_changed = orig_vif_type != new_vif_type
         if vif_type_changed and new_vif_type == portbindings.VIF_TYPE_UNBOUND:
-            trunk = self._get_trunk(context, trunk_details['trunk_id'])
+            trunk_id = trunk_details['trunk_id']
             # NOTE(status_police) Trunk status goes to DOWN when the parent
             # port is unbound. This means there are no more physical resources
             # associated with the logical resource.
-            trunk.update(status=constants.DOWN_STATUS)
+            self.update_trunk(context, trunk_id,
+                              {'trunk': {'status': constants.DOWN_STATUS}})
