@@ -15,6 +15,7 @@
 import functools
 
 import fixtures
+import netaddr
 from neutron_lib import constants
 from oslo_utils import uuidutils
 
@@ -371,14 +372,12 @@ class OVSBaseConnectionTester(ConnectionTester):
 
     @staticmethod
     def set_tag(port_name, bridge, tag):
-        bridge.set_db_attribute('Port', port_name, 'tag', tag)
-        other_config = bridge.db_get_val(
-            'Port', port_name, 'other_config')
-        if 'tag' in other_config:
-            return
-        other_config['tag'] = str(tag)
-        bridge.set_db_attribute(
-            'Port', port_name, 'other_config', other_config)
+        ovsdb = bridge.ovsdb
+        with ovsdb.transaction() as txn:
+            txn.add(ovsdb.db_set('Port', port_name, ('tag', tag)))
+            txn.add(
+                ovsdb.db_add(
+                    'Port', port_name, 'other_config', {'tag': str(tag)}))
 
 
 class OVSConnectionTester(OVSBaseConnectionTester):
@@ -464,13 +463,11 @@ class OVSTrunkConnectionTester(OVSBaseConnectionTester):
 
         """
 
-        ip_wrap = ip_lib.IPWrapper(self._vm.namespace)
-        dev_name = self._vm.port.name + ".%d" % vlan
-        ip_wrap.add_vlan(dev_name, self._vm.port.name, vlan)
-        dev = ip_wrap.device(dev_name)
-        dev.addr.add(ip_cidr)
-        dev.link.set_up()
-        self._ip_vlan = ip_cidr.partition('/')[0]
+        network = netaddr.IPNetwork(ip_cidr)
+        net_helpers.create_vlan_interface(
+            self._vm.namespace, self._vm.port.name,
+            self.vm_mac_address, network, vlan)
+        self._ip_vlan = str(network.ip)
         ip_cidr = net_helpers.increment_ip_cidr(ip_cidr, 1)
         self._peer2 = self.useFixture(machine_fixtures.FakeMachine(
             self.bridge, ip_cidr))
@@ -517,18 +514,9 @@ class LinuxBridgeConnectionTester(ConnectionTester):
 
     """
 
-    def __init__(self, *args, **kwargs):
-        self.bridge_name = kwargs.pop('bridge_name', None)
-        super(LinuxBridgeConnectionTester, self).__init__(*args, **kwargs)
-
     def _setUp(self):
         super(LinuxBridgeConnectionTester, self)._setUp()
-        bridge_args = {}
-        if self.bridge_name:
-            bridge_args = {'prefix': self.bridge_name,
-                           'prefix_is_full_name': True}
-        self.bridge = self.useFixture(
-            net_helpers.LinuxBridgeFixture(**bridge_args)).bridge
+        self.bridge = self.useFixture(net_helpers.LinuxBridgeFixture()).bridge
         machines = self.useFixture(
             machine_fixtures.PeerMachines(
                 self.bridge, self.ip_cidr)).machines
