@@ -15,10 +15,11 @@
 import copy
 
 import netaddr
+from neutron_lib import constants as lib_constants
 from oslo_utils import uuidutils
 from six import moves
 
-from neutron.common import constants as l3_constants
+from neutron.common import constants as n_const
 
 _uuid = uuidutils.generate_uuid
 
@@ -32,7 +33,7 @@ def get_ha_interface(ip='169.254.192.1', mac='12:34:56:78:2b:5d'):
     subnet_id = _uuid()
     return {'admin_state_up': True,
             'device_id': _uuid(),
-            'device_owner': l3_constants.DEVICE_OWNER_ROUTER_HA_INTF,
+            'device_owner': lib_constants.DEVICE_OWNER_ROUTER_HA_INTF,
             'fixed_ips': [{'ip_address': ip,
                            'prefixlen': 18,
                            'subnet_id': subnet_id}],
@@ -102,12 +103,12 @@ def prepare_router_data(ip_version=4, enable_snat=None, num_internal_ports=1,
     router = {
         'id': router_id,
         'distributed': False,
-        l3_constants.INTERFACE_KEY: [],
+        lib_constants.INTERFACE_KEY: [],
         'routes': routes,
         'gw_port': ex_gw_port}
 
     if enable_floating_ip:
-        router[l3_constants.FLOATINGIP_KEY] = [{
+        router[lib_constants.FLOATINGIP_KEY] = [{
             'id': _uuid(),
             'port_id': _uuid(),
             'status': 'DOWN',
@@ -119,7 +120,7 @@ def prepare_router_data(ip_version=4, enable_snat=None, num_internal_ports=1,
     if enable_ha:
         router['ha'] = True
         router['ha_vr_id'] = 1
-        router[l3_constants.HA_INTERFACE_KEY] = (get_ha_interface())
+        router[lib_constants.HA_INTERFACE_KEY] = (get_ha_interface())
 
     if enable_snat is not None:
         router['enable_snat'] = enable_snat
@@ -131,18 +132,29 @@ def get_subnet_id(port):
 
 
 def router_append_interface(router, count=1, ip_version=4, ra_mode=None,
-                            addr_mode=None, dual_stack=False):
-    interfaces = router[l3_constants.INTERFACE_KEY]
+                            addr_mode=None, dual_stack=False, same_port=False):
+    interfaces = router[lib_constants.INTERFACE_KEY]
     current = sum(
         [netaddr.IPNetwork(subnet['cidr']).version == ip_version
          for p in interfaces for subnet in p['subnets']])
 
+    # If dual_stack=True, create IPv4 and IPv6 subnets on each port
+    # If same_port=True, create ip_version number of subnets on a single port
+    # Else create just an ip_version subnet on each port
+    if dual_stack:
+        ip_versions = [4, 6]
+    elif same_port:
+        ip_versions = [ip_version] * count
+        count = 1
+    else:
+        ip_versions = [ip_version]
+
     mac_address = netaddr.EUI('ca:fe:de:ad:be:ef')
     mac_address.dialect = netaddr.mac_unix
     for i in range(current, current + count):
         fixed_ips = []
         subnets = []
-        for loop_version in (4, 6):
+        for loop_version in ip_versions:
             if loop_version == 4 and (ip_version == 4 or dual_stack):
                 ip_pool = '35.4.%i.4'
                 cidr_pool = '35.4.%i.0/24'
@@ -202,7 +214,7 @@ def router_append_subnet(router, count=1, ip_version=4,
     else:
         raise ValueError("Invalid ip_version: %s" % ip_version)
 
-    interfaces = copy.deepcopy(router.get(l3_constants.INTERFACE_KEY, []))
+    interfaces = copy.deepcopy(router.get(lib_constants.INTERFACE_KEY, []))
     if interface_id:
         try:
             interface = next(i for i in interfaces
@@ -247,11 +259,11 @@ def router_append_subnet(router, count=1, ip_version=4,
              'fixed_ips': fixed_ips,
              'subnets': subnets})
 
-    router[l3_constants.INTERFACE_KEY] = interfaces
+    router[lib_constants.INTERFACE_KEY] = interfaces
 
 
 def router_append_pd_enabled_subnet(router, count=1):
-    interfaces = router[l3_constants.INTERFACE_KEY]
+    interfaces = router[lib_constants.INTERFACE_KEY]
     current = sum(netaddr.IPNetwork(subnet['cidr']).version == 6
                   for p in interfaces for subnet in p['subnets'])
 
@@ -268,10 +280,10 @@ def router_append_pd_enabled_subnet(router, count=1):
                                'subnet_id': subnet_id}],
                 'mac_address': str(mac_address),
                 'subnets': [{'id': subnet_id,
-                             'cidr': l3_constants.PROVISIONAL_IPV6_PD_PREFIX,
+                             'cidr': n_const.PROVISIONAL_IPV6_PD_PREFIX,
                              'gateway_ip': '::1',
-                             'ipv6_ra_mode': l3_constants.IPV6_SLAAC,
-                             'subnetpool_id': l3_constants.IPV6_PD_POOL_ID}]}
+                             'ipv6_ra_mode': lib_constants.IPV6_SLAAC,
+                             'subnetpool_id': lib_constants.IPV6_PD_POOL_ID}]}
         interfaces.append(intf)
         pd_intfs.append(intf)
         mac_address.value += 1
