@@ -16,6 +16,7 @@
 import abc
 from concurrent import futures
 import contextlib
+import functools
 import os
 import random
 import re
@@ -242,6 +243,27 @@ def create_patch_ports(source, destination):
     destination.add_patch_port(destination_name, source_name)
 
 
+def create_vlan_interface(
+        namespace, port_name, mac_address, ip_address, vlan_tag):
+    """Create a VLAN interface in namespace with IP address.
+
+    :param namespace: Namespace in which VLAN interface should be created.
+    :param port_name: Name of the port to which VLAN should be added.
+    :param ip_address: IPNetwork instance containing the VLAN interface IP
+                       address.
+    :param vlan_tag: VLAN tag for VLAN interface.
+    """
+    ip_wrap = ip_lib.IPWrapper(namespace)
+    dev_name = "%s.%d" % (port_name, vlan_tag)
+    ip_wrap.add_vlan(dev_name, port_name, vlan_tag)
+    dev = ip_wrap.device(dev_name)
+    dev.addr.add(str(ip_address))
+    dev.link.set_address(mac_address)
+    dev.link.set_up()
+
+    return dev
+
+
 class RootHelperProcess(subprocess.Popen):
     def __init__(self, cmd, *args, **kwargs):
         for arg in ('stdin', 'stdout', 'stderr'):
@@ -266,9 +288,12 @@ class RootHelperProcess(subprocess.Popen):
     @staticmethod
     def _read_stream(stream, timeout):
         if timeout:
-            rready, _wready, _xready = select.select([stream], [], [], timeout)
-            if not rready:
-                raise RuntimeError('No output in %.2f seconds' % timeout)
+            poller = select.poll()
+            poller.register(stream.fileno())
+            poll_predicate = functools.partial(poller.poll, 1)
+            common_utils.wait_until_true(poll_predicate, timeout, 0.1,
+                                  RuntimeError(
+                                      'No output in %.2f seconds' % timeout))
         return stream.readline()
 
     def writeline(self, data):
@@ -482,8 +507,14 @@ class NetcatTester(object):
 
         return message == testing_string
 
+    def test_no_connectivity(self, respawn=False):
+        try:
+            return not self.test_connectivity(respawn)
+        except RuntimeError:
+            return True
+
     def _spawn_nc_in_namespace(self, namespace, address, listen=False):
-        cmd = ['nc', address, self.dst_port]
+        cmd = ['ncat', address, self.dst_port]
         if self.protocol == self.UDP:
             cmd.append('-u')
         elif self.protocol == self.SCTP:
