@@ -12,6 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import os
+import shutil
 import tempfile
 
 import fixtures
@@ -46,6 +48,9 @@ class ConfigFixture(fixtures.Fixture):
         self.useFixture(cfg_fixture)
         self.filename = cfg_fixture.filename
 
+    def _generate_namespace_suffix(self):
+        return utils.get_rand_name(prefix='test')
+
 
 class NeutronConfigFixture(ConfigFixture):
 
@@ -54,7 +59,7 @@ class NeutronConfigFixture(ConfigFixture):
         super(NeutronConfigFixture, self).__init__(
             env_desc, host_desc, temp_dir, base_filename='neutron.conf')
 
-        service_plugins = ['router']
+        service_plugins = ['router', 'trunk']
         if env_desc.qos:
             service_plugins.append('qos')
 
@@ -67,6 +72,7 @@ class NeutronConfigFixture(ConfigFixture):
                 'service_plugins': ','.join(service_plugins),
                 'auth_strategy': 'noauth',
                 'debug': 'True',
+                'agent_down_time': env_desc.agent_down_time,
                 'transport_url':
                     'rabbit://%(user)s:%(password)s@%(host)s:5672/%(vhost)s' %
                     {'user': rabbitmq_environment.user,
@@ -133,9 +139,10 @@ class ML2ConfigFixture(ConfigFixture):
             },
         })
 
+        extension_drivers = ['port_security']
         if env_desc.qos:
-            self.config['ml2']['extension_drivers'] =\
-                    qos_ext.QOS_EXT_DRIVER_ALIAS
+            extension_drivers.append(qos_ext.QOS_EXT_DRIVER_ALIAS)
+        self.config['ml2']['extension_drivers'] = ','.join(extension_drivers)
 
 
 class OVSConfigFixture(ConfigFixture):
@@ -154,7 +161,7 @@ class OVSConfigFixture(ConfigFixture):
                 'ovsdb_interface': host_desc.ovsdb_interface,
             },
             'securitygroup': {
-                'firewall_driver': 'noop',
+                'firewall_driver': host_desc.firewall_driver,
             },
             'agent': {
                 'l2_population': str(self.env_desc.l2_pop),
@@ -222,6 +229,9 @@ class LinuxBridgeConfigFixture(ConfigFixture):
                 'enable_vxlan': str(self.env_desc.tunneling_enabled),
                 'local_ip': local_ip,
                 'l2_population': str(self.env_desc.l2_pop),
+            },
+            'securitygroup': {
+                'firewall_driver': host_desc.firewall_driver,
             }
         })
         if env_desc.qos:
@@ -290,5 +300,49 @@ class L3ConfigFixture(ConfigFixture):
     def get_external_bridge(self):
         return self.config.DEFAULT.external_network_bridge
 
-    def _generate_namespace_suffix(self):
-        return utils.get_rand_name(prefix='test')
+
+class DhcpConfigFixture(ConfigFixture):
+
+    def __init__(self, env_desc, host_desc, temp_dir, integration_bridge=None):
+        super(DhcpConfigFixture, self).__init__(
+            env_desc, host_desc, temp_dir, base_filename='dhcp_agent.ini')
+
+        if host_desc.l2_agent_type == constants.AGENT_TYPE_OVS:
+            self._prepare_config_with_ovs_agent(integration_bridge)
+        elif host_desc.l2_agent_type == constants.AGENT_TYPE_LINUXBRIDGE:
+            self._prepare_config_with_linuxbridge_agent()
+        self.config['DEFAULT'].update({
+            'debug': 'True',
+            'dhcp_confs': self._generate_dhcp_path(),
+            'test_namespace_suffix': self._generate_namespace_suffix()
+        })
+
+    def _setUp(self):
+        super(DhcpConfigFixture, self)._setUp()
+        self.addCleanup(self._clean_dhcp_path)
+
+    def _prepare_config_with_ovs_agent(self, integration_bridge):
+        self.config.update({
+            'DEFAULT': {
+                'interface_driver': 'openvswitch',
+                'ovs_integration_bridge': integration_bridge,
+            }
+        })
+
+    def _prepare_config_with_linuxbridge_agent(self):
+        self.config.update({
+            'DEFAULT': {
+                'interface_driver': 'linuxbridge',
+            }
+        })
+
+    def _generate_dhcp_path(self):
+        # NOTE(slaweq): dhcp_conf path needs to be directory with read
+        # permission for everyone, otherwise dnsmasq process will not be able
+        # to read his configs
+        self.dhcp_path = tempfile.mkdtemp(prefix="dhcp_configs_", dir="/tmp/")
+        os.chmod(self.dhcp_path, 0o755)
+        return self.dhcp_path
+
+    def _clean_dhcp_path(self):
+        shutil.rmtree(self.dhcp_path, ignore_errors=True)
