@@ -29,6 +29,18 @@ from neutron.tests.functional.agent.l2 import base
 
 TEST_POLICY_ID1 = "a2d72369-4246-4f19-bd3c-af51ec8d70cd"
 TEST_POLICY_ID2 = "46ebaec0-0570-43ac-82f6-60d2b03168c5"
+TEST_DSCP_MARK_1 = 14
+TEST_DSCP_MARK_2 = 30
+TEST_DSCP_MARKING_RULE_1 = rule.QosDscpMarkingRule(
+        context=None,
+        qos_policy_id=TEST_POLICY_ID1,
+        id="9f126d84-551a-4dcf-bb01-0e9c0df0c793",
+        dscp_mark=TEST_DSCP_MARK_1)
+TEST_DSCP_MARKING_RULE_2 = rule.QosDscpMarkingRule(
+        context=None,
+        qos_policy_id=TEST_POLICY_ID2,
+        id="7f126d84-551a-4dcf-bb01-0e9c0df0c793",
+        dscp_mark=TEST_DSCP_MARK_2)
 TEST_BW_LIMIT_RULE_1 = rule.QosBandwidthLimitRule(
         context=None,
         qos_policy_id=TEST_POLICY_ID1,
@@ -48,8 +60,12 @@ class OVSAgentQoSExtensionTestFramework(base.OVSAgentTestFramework):
         super(OVSAgentQoSExtensionTestFramework, self).setUp()
         self.config.set_override('extensions', ['qos'], 'agent')
         self._set_pull_mock()
-        self.set_test_qos_rules(TEST_POLICY_ID1, [TEST_BW_LIMIT_RULE_1])
-        self.set_test_qos_rules(TEST_POLICY_ID2, [TEST_BW_LIMIT_RULE_2])
+        self.set_test_qos_rules(TEST_POLICY_ID1,
+                                [TEST_BW_LIMIT_RULE_1,
+                                TEST_DSCP_MARKING_RULE_1])
+        self.set_test_qos_rules(TEST_POLICY_ID2,
+                                [TEST_BW_LIMIT_RULE_2,
+                                TEST_DSCP_MARKING_RULE_2])
 
     def _set_pull_mock(self):
 
@@ -68,7 +84,7 @@ class OVSAgentQoSExtensionTestFramework(base.OVSAgentTestFramework):
 
         qos_policy = policy.QosPolicy(
             context=None,
-            tenant_id=uuidutils.generate_uuid(),
+            project_id=uuidutils.generate_uuid(),
             id=policy_id,
             name="Test Policy Name",
             description="This is a policy for testing purposes",
@@ -107,6 +123,27 @@ class OVSAgentQoSExtensionTestFramework(base.OVSAgentTestFramework):
         l2_extensions.wait_until_bandwidth_limit_rule_applied(
             self.agent.int_br, port['vif_name'], rule)
 
+    def _assert_dscp_marking_rule_is_set(self, port, dscp_rule):
+        port_num = self.agent.int_br._get_port_ofport(port['vif_name'])
+
+        flows = self.agent.int_br.dump_flows_for(table='0',
+                                                 in_port=str(port_num))
+        tos_mark = l2_extensions.extract_mod_nw_tos_action(flows)
+        self.assertEqual(dscp_rule.dscp_mark << 2, tos_mark)
+
+    def _assert_dscp_marking_rule_not_set(self, port):
+        port_num = self.agent.int_br._get_port_ofport(port['vif_name'])
+
+        flows = self.agent.int_br.dump_flows_for(table='0',
+                                                 in_port=str(port_num))
+
+        tos_mark = l2_extensions.extract_mod_nw_tos_action(flows)
+        self.assertIsNone(tos_mark)
+
+    def wait_until_dscp_marking_rule_applied(self, port, dscp_mark):
+        l2_extensions.wait_until_dscp_marking_rule_applied_ovs(
+            self.agent.int_br, port['vif_name'], dscp_mark)
+
     def _create_port_with_qos(self):
         port_dict = self._create_test_port_dict()
         port_dict['qos_policy_id'] = TEST_POLICY_ID1
@@ -150,19 +187,57 @@ class TestOVSAgentQosExtension(OVSAgentQoSExtensionTestFramework):
 
         self._assert_bandwidth_limit_rule_not_set(self.ports[2])
 
+    def test_port_creation_with_dscp_marking(self):
+        """Make sure dscp marking rules are set in low level to ports."""
+
+        self.setup_agent_and_ports(
+            port_dicts=self.create_test_ports(amount=1,
+                                              policy_id=TEST_POLICY_ID1))
+        self.wait_until_ports_state(self.ports, up=True)
+
+        for port in self.ports:
+            self._assert_dscp_marking_rule_is_set(
+                port, TEST_DSCP_MARKING_RULE_1)
+
+    def test_port_creation_with_different_dscp_markings(self):
+        """Make sure different types of policies end on the right ports."""
+
+        port_dicts = self.create_test_ports(amount=3)
+
+        port_dicts[0]['qos_policy_id'] = TEST_POLICY_ID1
+        port_dicts[1]['qos_policy_id'] = TEST_POLICY_ID2
+
+        self.setup_agent_and_ports(port_dicts)
+        self.wait_until_ports_state(self.ports, up=True)
+
+        self._assert_dscp_marking_rule_is_set(self.ports[0],
+                                              TEST_DSCP_MARKING_RULE_1)
+
+        self._assert_dscp_marking_rule_is_set(self.ports[1],
+                                              TEST_DSCP_MARKING_RULE_2)
+
+        self._assert_dscp_marking_rule_not_set(self.ports[2])
+
     def test_simple_port_policy_update(self):
         self.setup_agent_and_ports(
             port_dicts=self.create_test_ports(amount=1,
                                               policy_id=TEST_POLICY_ID1))
         self.wait_until_ports_state(self.ports, up=True)
+        self._assert_dscp_marking_rule_is_set(self.ports[0],
+                                              TEST_DSCP_MARKING_RULE_1)
         policy_copy = copy.deepcopy(self.qos_policies[TEST_POLICY_ID1])
         policy_copy.rules[0].max_kbps = 500
         policy_copy.rules[0].max_burst_kbps = 5
-        consumer_reg.push(resources.QOS_POLICY, policy_copy, events.UPDATED)
+        policy_copy.rules[1].dscp_mark = TEST_DSCP_MARK_2
+        context = mock.Mock()
+        consumer_reg.push(context, resources.QOS_POLICY,
+                          [policy_copy], events.UPDATED)
         self.wait_until_bandwidth_limit_rule_applied(self.ports[0],
                                                      policy_copy.rules[0])
         self._assert_bandwidth_limit_rule_is_set(self.ports[0],
                                                  policy_copy.rules[0])
+        self._assert_dscp_marking_rule_is_set(self.ports[0],
+                                              TEST_DSCP_MARKING_RULE_2)
 
     def test_port_qos_disassociation(self):
         """Test that qos_policy_id set to None will remove all qos rules from
@@ -192,6 +267,8 @@ class TestOVSAgentQosExtension(OVSAgentQoSExtensionTestFramework):
 
         policy_copy = copy.deepcopy(self.qos_policies[TEST_POLICY_ID1])
         policy_copy.rules = list()
-        consumer_reg.push(resources.QOS_POLICY, policy_copy, events.UPDATED)
+        context = mock.Mock()
+        consumer_reg.push(context, resources.QOS_POLICY, [policy_copy],
+                          events.UPDATED)
 
         self.wait_until_bandwidth_limit_rule_applied(port_dict, None)
