@@ -24,14 +24,12 @@ import netaddr
 from neutron_lib import constants
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import uuidutils
 import testscenarios
 
 from neutron.agent import firewall
 from neutron.agent.linux import iptables_firewall
 from neutron.agent.linux import openvswitch_firewall
 from neutron.cmd.sanity import checks
-from neutron.common import constants as n_const
 from neutron.conf.agent import securitygroups_rpc as security_config
 from neutron.tests.common import conn_testers
 from neutron.tests.common import helpers
@@ -93,11 +91,10 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
     scenarios = scenarios_iptables + scenarios_ovs_fw_interfaces
 
     ip_cidr = None
-    vlan_range = set(range(test_constants.VLAN_COUNT))
+    vlan_range = set(range(1, test_constants.VLAN_COUNT - 1))
 
     def setUp(self):
         security_config.register_securitygroups_opts()
-        self.net_id = uuidutils.generate_uuid()
         super(BaseFirewallTestCase, self).setUp()
         self.tester, self.firewall = getattr(self, self.initialize)()
         if self.firewall_name == "openvswitch":
@@ -106,8 +103,7 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
             self.tester.vm_port_id,
             [self.tester.vm_ip_address],
             self.tester.vm_mac_address,
-            [self.FAKE_SECURITY_GROUP_ID],
-            self.net_id)
+            [self.FAKE_SECURITY_GROUP_ID])
         # FIXME(jlibosva): We should consider to call prepare_port_filter with
         # deferred bridge depending on its performance
         self.firewall.prepare_port_filter(self.src_port_desc)
@@ -115,10 +111,8 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
     def initialize_iptables(self):
         cfg.CONF.set_override('enable_ipset', self.enable_ipset,
                               'SECURITYGROUP')
-        br_name = ('brq' + self.net_id)[:n_const.LINUX_DEV_LEN]
         tester = self.useFixture(
-            conn_testers.LinuxBridgeConnectionTester(self.ip_cidr,
-                                                     bridge_name=br_name))
+            conn_testers.LinuxBridgeConnectionTester(self.ip_cidr))
         firewall_drv = iptables_firewall.IptablesFirewallDriver(
             namespace=tester.bridge_namespace)
         return tester, firewall_drv
@@ -146,8 +140,7 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
         self.tester.set_peer_tag(vlan)
 
     @staticmethod
-    def _create_port_description(port_id, ip_addresses, mac_address, sg_ids,
-                                 net_id):
+    def _create_port_description(port_id, ip_addresses, mac_address, sg_ids):
         return {'admin_state_up': True,
                 'device': port_id,
                 'device_owner': DEVICE_OWNER_COMPUTE,
@@ -155,8 +148,7 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
                 'mac_address': mac_address,
                 'port_security_enabled': True,
                 'security_groups': sg_ids,
-                'status': 'ACTIVE',
-                'network_id': net_id}
+                'status': 'ACTIVE'}
 
     def _apply_security_group_rules(self, sg_id, sg_rules):
         with self.firewall.defer_apply():
@@ -398,7 +390,7 @@ class FirewallTestCase(BaseFirewallTestCase):
         not_allowed_ip = "%s/24" % (allowed_ip + 1)
         self.src_port_desc['allowed_address_pairs'] = [
             {'mac_address': port_mac,
-             'ip_address': "%s/32" % allowed_ip}]
+             'ip_address': allowed_ip}]
         allowed_ip = "%s/24" % allowed_ip
 
         self.firewall.update_port_filter(self.src_port_desc)
@@ -547,8 +539,7 @@ class FirewallTestCase(BaseFirewallTestCase):
             self.tester.peer_port_id,
             [self.tester.peer_ip_address],
             self.tester.peer_mac_address,
-            [remote_sg_id],
-            self.net_id)
+            [remote_sg_id])
 
         vm_sg_members = {'IPv4': [self.tester.peer_ip_address]}
         peer_sg_rules = [{'ethertype': 'IPv4', 'direction': 'egress',
@@ -582,7 +573,7 @@ class FirewallTestCase(BaseFirewallTestCase):
 
         """
         # Enable ip forwarding on the interface in order to reply with
-        # destionation net unreachable
+        # destination net unreachable
         self.tester._peer.execute([
             'sysctl', '-w', 'net.ipv4.conf.%s.forwarding=1' %
             self.tester._peer.port.name])
