@@ -18,6 +18,7 @@ from oslo_utils import uuidutils
 import testtools
 
 from neutron.agent.linux import ip_lib
+from neutron.common import utils
 from neutron.objects import trunk
 from neutron.plugins.ml2.drivers.linuxbridge.agent import \
     linuxbridge_neutron_agent
@@ -59,6 +60,19 @@ class LinuxBridgeAgentTests(test_ip_lib.IpLibTestFramework):
                                                     name='br-eth1'))
         lba.LinuxBridgeManager(mappings, {})
 
+    def test_set_port_mac(self):
+        attr = self.generate_device_details()
+        self.manage_device(attr)
+        plumber = trunk_plumber.Plumber(namespace=attr.namespace)
+        # force it to return name of above
+        plumber._get_tap_device_name = lambda x: attr.name
+        new_mac = utils.get_random_mac('fa:16:3e:00:00:00'.split(':'))
+        self.assertTrue(plumber.set_port_mac('port_id', new_mac))
+        self.assertFalse(plumber.set_port_mac('port_id', new_mac))
+        new_mac = utils.get_random_mac('fa:16:3e:00:00:00'.split(':'))
+        self.assertTrue(plumber.set_port_mac('port_id', new_mac))
+        self.assertFalse(plumber.set_port_mac('port_id', new_mac))
+
     def test_vlan_subinterfaces(self):
         attr = self.generate_device_details()
         device = self.manage_device(attr)
@@ -133,7 +147,7 @@ class LinuxBridgeAgentTests(test_ip_lib.IpLibTestFramework):
     def _gen_trunk(self):
         trunk_obj = trunk.Trunk(id=uuidutils.generate_uuid(),
                                 port_id=uuidutils.generate_uuid(),
-                                tenant_id=uuidutils.generate_uuid())
+                                project_id=uuidutils.generate_uuid())
         subports = [trunk.SubPort(id=uuidutils.generate_uuid(),
                                   port_id=uuidutils.generate_uuid(),
                                   segmentation_type='vlan',
