@@ -18,6 +18,7 @@ from neutron_lib import exceptions
 from oslo_utils import uuidutils
 
 from neutron.agent.l2.extensions import qos
+from neutron.agent.l2.extensions import qos_linux
 from neutron.api.rpc.callbacks.consumer import registry
 from neutron.api.rpc.callbacks import events
 from neutron.api.rpc.callbacks import resources
@@ -55,7 +56,7 @@ FAKE_RULE_ID = uuidutils.generate_uuid()
 REALLY_FAKE_RULE_ID = uuidutils.generate_uuid()
 
 
-class FakeDriver(qos.QosAgentDriver):
+class FakeDriver(qos_linux.QosLinuxAgentDriver):
 
     SUPPORTED_RULES = {qos_consts.RULE_TYPE_BANDWIDTH_LIMIT}
 
@@ -152,7 +153,7 @@ class QosExtensionBaseTestCase(base.BaseTestCase):
         # Don't rely on used driver
         mock.patch(
             'neutron.manager.NeutronManager.load_class_for_provider',
-            return_value=lambda: mock.Mock(spec=qos.QosAgentDriver)
+            return_value=lambda: mock.Mock(spec=qos_linux.QosLinuxAgentDriver)
         ).start()
 
 
@@ -230,7 +231,8 @@ class QosExtensionRpcTestCase(QosExtensionBaseTestCase):
             self.qos_ext, '_process_update_policy') as update_mock:
 
             for event_type in set(events.VALID) - {events.UPDATED}:
-                self.qos_ext._handle_notification(object(), event_type)
+                self.qos_ext._handle_notification(mock.Mock(), 'QOS',
+                                                  object(), event_type)
                 self.assertFalse(update_mock.called)
 
     def test__handle_notification_passes_update_events(self):
@@ -238,7 +240,8 @@ class QosExtensionRpcTestCase(QosExtensionBaseTestCase):
             self.qos_ext, '_process_update_policy') as update_mock:
 
             policy_obj = mock.Mock()
-            self.qos_ext._handle_notification([policy_obj], events.UPDATED)
+            self.qos_ext._handle_notification(mock.Mock(), 'QOS',
+                                              [policy_obj], events.UPDATED)
             update_mock.assert_called_with(policy_obj)
 
     def test__process_update_policy(self):
@@ -298,7 +301,7 @@ class QosExtensionRpcTestCase(QosExtensionBaseTestCase):
 
 class QosExtensionInitializeTestCase(QosExtensionBaseTestCase):
 
-    @mock.patch.object(registry, 'subscribe')
+    @mock.patch.object(registry, 'register')
     @mock.patch.object(resources_rpc, 'ResourcesPushRpcCallback')
     def test_initialize_subscribed_to_rpc(self, rpc_mock, subscribe_mock):
         self.qos_ext.initialize(
