@@ -12,10 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import errno
 import os.path
 import random
-import re
 import sys
 
 import eventlet
@@ -680,31 +678,6 @@ class TestDelayedStringRenderer(base.BaseTestCase):
         self.assertTrue(my_func.called)
 
 
-class TestEnsureDir(base.BaseTestCase):
-    @mock.patch('os.makedirs')
-    def test_ensure_dir_no_fail_if_exists(self, makedirs):
-        error = OSError()
-        error.errno = errno.EEXIST
-        makedirs.side_effect = error
-        utils.ensure_dir("/etc/create/concurrently")
-
-    @mock.patch('os.makedirs')
-    def test_ensure_dir_calls_makedirs(self, makedirs):
-        utils.ensure_dir("/etc/create/directory")
-        makedirs.assert_called_once_with("/etc/create/directory", 0o755)
-
-
-class TestCamelize(base.BaseTestCase):
-    def test_camelize(self):
-        data = {'bandwidth_limit': 'BandwidthLimit',
-                'test': 'Test',
-                'some__more__dashes': 'SomeMoreDashes',
-                'a_penguin_walks_into_a_bar': 'APenguinWalksIntoABar'}
-
-        for s, expected in data.items():
-            self.assertEqual(expected, utils.camelize(s))
-
-
 class TestRoundVal(base.BaseTestCase):
     def test_round_val_ok(self):
         for expected, value in ((0, 0),
@@ -715,15 +688,6 @@ class TestRoundVal(base.BaseTestCase):
             self.assertEqual(expected, utils.round_val(value))
 
 
-class TestGetRandomString(base.BaseTestCase):
-    def test_get_random_string(self):
-        length = 127
-        random_string = utils.get_random_string(length)
-        self.assertEqual(length, len(random_string))
-        regex = re.compile('^[0-9a-fA-F]+$')
-        self.assertIsNotNone(regex.match(random_string))
-
-
 class TestSafeDecodeUtf8(base.BaseTestCase):
 
     @helpers.requires_py2
@@ -854,53 +818,3 @@ class ImportModulesRecursivelyTestCase(base.BaseTestCase):
         for module in expected_modules:
             self.assertIn(module, modules)
             self.assertIn(module, sys.modules)
-
-
-class TestThrottler(base.BaseTestCase):
-    def test_throttler(self):
-        threshold = 1
-        orig_function = mock.Mock()
-        # Add this magic name as it's required by functools
-        orig_function.__name__ = 'mock_func'
-        throttled_func = utils.throttler(threshold)(orig_function)
-
-        throttled_func()
-
-        sleep = utils.eventlet.sleep
-
-        def sleep_mock(amount_to_sleep):
-            sleep(amount_to_sleep)
-            self.assertTrue(threshold > amount_to_sleep)
-
-        with mock.patch.object(utils.eventlet, "sleep",
-                               side_effect=sleep_mock):
-            throttled_func()
-
-        self.assertEqual(2, orig_function.call_count)
-
-        lock_with_timer = six.get_function_closure(
-            throttled_func)[1].cell_contents
-        timestamp = lock_with_timer.timestamp - threshold
-        lock_with_timer.timestamp = timestamp
-
-        throttled_func()
-
-        self.assertEqual(3, orig_function.call_count)
-        self.assertTrue(timestamp < lock_with_timer.timestamp)
-
-    def test_method_docstring_is_preserved(self):
-        class Klass(object):
-            @utils.throttler()
-            def method(self):
-                """Docstring"""
-
-        self.assertEqual("Docstring", Klass.method.__doc__)
-
-    def test_method_still_callable(self):
-        class Klass(object):
-            @utils.throttler()
-            def method(self):
-                pass
-
-        obj = Klass()
-        obj.method()
