@@ -112,12 +112,10 @@ class TestDbBasePluginIpam(test_db_base.NeutronDbPluginV2TestCase):
 
     def _get_allocate_mock(self, subnet_id, auto_ip='10.0.0.2',
                            fail_ip='127.0.0.1',
-                           exception=None):
-        if exception is None:
-            exception = n_exc.InvalidInput(error_message='SomeError')
-
+                           exception=n_exc.InvalidInput(
+                               error_message='SomeError')):
         def allocate_mock(request):
-            if type(request) == ipam_req.SpecificAddressRequest:
+            if isinstance(request, ipam_req.SpecificAddressRequest):
                 if request.address == netaddr.IPAddress(fail_ip):
                     raise exception
                 else:
@@ -127,15 +125,14 @@ class TestDbBasePluginIpam(test_db_base.NeutronDbPluginV2TestCase):
 
         return allocate_mock
 
-    def _get_deallocate_mock(self, fail_ip='127.0.0.1', exception=None):
-            if exception is None:
-                exception = n_exc.InvalidInput(error_message='SomeError')
+    def _get_deallocate_mock(self, fail_ip='127.0.0.1',
+                             exception=n_exc.InvalidInput(
+                                 error_message='SomeError')):
+        def deallocate_mock(ip):
+            if str(ip) == fail_ip:
+                raise exception
 
-            def deallocate_mock(ip):
-                if str(ip) == fail_ip:
-                    raise exception
-
-            return deallocate_mock
+        return deallocate_mock
 
     def _validate_allocate_calls(self, expected_calls, mocks):
         self.assertTrue(mocks['subnets'].allocate.called)
@@ -648,7 +645,7 @@ class TestDbBasePluginIpam(test_db_base.NeutronDbPluginV2TestCase):
         fixed_ips_mock = mock.Mock(return_value=changes.add)
         mocks['ipam'] = ipam_pluggable_backend.IpamPluggableBackend()
         mocks['ipam']._get_changed_ips_for_port = changes_mock
-        mocks['ipam']._ipam_get_subnets = mock.Mock(return_value=[])
+        mocks['ipam']._ipam_get_subnets = mock.Mock()
         mocks['ipam']._test_fixed_ips_for_port = fixed_ips_mock
         mocks['ipam']._update_ips_for_pd_subnet = mock.Mock(return_value=[])
 
@@ -787,7 +784,7 @@ class TestDbBasePluginIpam(test_db_base.NeutronDbPluginV2TestCase):
         mocks['ipam']._ipam_deallocate_ips = mock.Mock(side_effect=ValueError)
         mocks['ipam']._ipam_allocate_ips = mock.Mock(side_effect=ValueError)
 
-        # Validate original exception (DBDeadlock) is not overriden by
+        # Validate original exception (DBDeadlock) is not overridden by
         # exception raised on rollback (ValueError)
         self.assertRaises(db_exc.DBDeadlock,
                           mocks['ipam'].update_port_with_ips,
