@@ -14,20 +14,22 @@
 # limitations under the License.
 
 import mock
-from neutron_lib import constants as l3_const
+from neutron_lib import constants as const
 from neutron_lib import exceptions
+from neutron_lib.plugins import directory
 from oslo_utils import uuidutils
-import testtools
 
+from neutron.callbacks import events
+from neutron.callbacks import registry
+from neutron.callbacks import resources
 from neutron.common import constants as n_const
 from neutron import context
 from neutron.db import agents_db
 from neutron.db import common_db_mixin
 from neutron.db import l3_agentschedulers_db
 from neutron.db import l3_dvr_db
+from neutron.extensions import l3
 from neutron.extensions import portbindings
-from neutron import manager
-from neutron.plugins.common import constants as plugin_const
 from neutron.tests.unit.db import test_db_base_plugin_v2
 
 _uuid = uuidutils.generate_uuid
@@ -44,7 +46,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
 
     def setUp(self):
         super(L3DvrTestCase, self).setUp(plugin='ml2')
-        self.core_plugin = manager.NeutronManager.get_plugin()
+        self.core_plugin = directory.get_plugin()
         self.ctx = context.get_admin_context()
         self.mixin = FakeL3Plugin()
 
@@ -79,7 +81,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             'distributed': True
         }
         router_db = self._create_router(router)
-        self.assertIsNone(self.mixin._validate_router_migration(
+        self.assertFalse(self.mixin._validate_router_migration(
             self.ctx, router_db, {'name': 'foo_router_2'}))
 
     def test__validate_router_migration_raise_error(self):
@@ -122,7 +124,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                          self.mixin._migrate_router_ports.call_count)
 
     def _test_get_device_owner(self, is_distributed=False,
-                               expected=l3_const.DEVICE_OWNER_ROUTER_INTF,
+                               expected=const.DEVICE_OWNER_ROUTER_INTF,
                                pass_router_id=True):
         router = {
             'name': 'foo_router',
@@ -145,7 +147,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
     def test__get_device_owner_distributed(self):
         self._test_get_device_owner(
             is_distributed=True,
-            expected=l3_const.DEVICE_OWNER_DVR_INTERFACE,
+            expected=const.DEVICE_OWNER_DVR_INTERFACE,
             pass_router_id=False)
 
     def _test__is_distributed_router(self, router, expected):
@@ -170,34 +172,32 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         self._test__is_distributed_router(router, True)
 
     def test__get_agent_gw_ports_exist_for_network(self):
-        with mock.patch.object(manager.NeutronManager, 'get_plugin') as gp:
-            plugin = mock.Mock()
-            gp.return_value = plugin
-            plugin.get_ports.return_value = []
-            self.mixin._get_agent_gw_ports_exist_for_network(
-                self.ctx, 'network_id', 'host', 'agent_id')
+        plugin = mock.Mock()
+        directory.add_plugin(const.CORE, plugin)
+        plugin.get_ports.return_value = []
+        self.mixin._get_agent_gw_ports_exist_for_network(
+            self.ctx, 'network_id', 'host', 'agent_id')
         plugin.get_ports.assert_called_with(self.ctx, {
             'network_id': ['network_id'],
             'device_id': ['agent_id'],
-            'device_owner': [l3_const.DEVICE_OWNER_AGENT_GW]})
+            'device_owner': [const.DEVICE_OWNER_AGENT_GW]})
 
     def _test_prepare_direct_delete_dvr_internal_ports(self, port):
-        with mock.patch.object(manager.NeutronManager, 'get_plugin') as gp:
-            plugin = mock.Mock()
-            gp.return_value = plugin
-            plugin.get_port.return_value = port
-            self.mixin._router_exists = mock.Mock(return_value=True)
-            self.assertRaises(exceptions.ServicePortInUse,
-                              self.mixin.prevent_l3_port_deletion,
-                              self.ctx,
-                              port['id'])
+        plugin = mock.Mock()
+        directory.add_plugin(const.CORE, plugin)
+        plugin.get_port.return_value = port
+        self.mixin._router_exists = mock.Mock(return_value=True)
+        self.assertRaises(exceptions.ServicePortInUse,
+                          self.mixin.prevent_l3_port_deletion,
+                          self.ctx,
+                          port['id'])
 
     def test_prevent_delete_floatingip_agent_gateway_port(self):
         port = {
             'id': 'my_port_id',
             'fixed_ips': mock.ANY,
             'device_id': 'r_id',
-            'device_owner': l3_const.DEVICE_OWNER_AGENT_GW
+            'device_owner': const.DEVICE_OWNER_AGENT_GW
         }
         self._test_prepare_direct_delete_dvr_internal_ports(port)
 
@@ -206,7 +206,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             'id': 'my_port_id',
             'fixed_ips': mock.ANY,
             'device_id': 'r_id',
-            'device_owner': l3_const.DEVICE_OWNER_ROUTER_SNAT
+            'device_owner': const.DEVICE_OWNER_ROUTER_SNAT
         }
         self._test_prepare_direct_delete_dvr_internal_ports(port)
 
@@ -235,53 +235,27 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         routers = self.mixin._build_routers_list(self.ctx, routers, gw_ports)
         self.assertIsNone(routers[0].get('gw_port'))
 
-    def setup_port_has_ipv6_address(self, port):
-        with mock.patch.object(l3_dvr_db.l3_db.L3_NAT_db_mixin,
-                               '_port_has_ipv6_address') as pv6:
-            pv6.return_value = True
-            result = self.mixin._port_has_ipv6_address(port)
-            return result, pv6
-
-    def test__port_has_ipv6_address_for_dvr_snat_port(self):
-        port = {
-            'id': 'my_port_id',
-            'device_owner': l3_const.DEVICE_OWNER_ROUTER_SNAT,
-        }
-        result, pv6 = self.setup_port_has_ipv6_address(port)
-        self.assertFalse(result)
-        self.assertFalse(pv6.called)
-
-    def test__port_has_ipv6_address_for_non_snat_ports(self):
-        port = {
-            'id': 'my_port_id',
-            'device_owner': l3_const.DEVICE_OWNER_DVR_INTERFACE,
-        }
-        result, pv6 = self.setup_port_has_ipv6_address(port)
-        self.assertTrue(result)
-        self.assertTrue(pv6.called)
-
     def _helper_delete_floatingip_agent_gateway_port(self, port_host):
         ports = [{
             'id': 'my_port_id',
             portbindings.HOST_ID: 'foo_host',
             'network_id': 'ext_network_id',
-            'device_owner': l3_const.DEVICE_OWNER_ROUTER_GW
+            'device_owner': const.DEVICE_OWNER_ROUTER_GW
         },
                 {
             'id': 'my_new_port_id',
             portbindings.HOST_ID: 'my_foo_host',
             'network_id': 'ext_network_id',
-            'device_owner': l3_const.DEVICE_OWNER_ROUTER_GW
+            'device_owner': const.DEVICE_OWNER_ROUTER_GW
         }]
-        with mock.patch.object(manager.NeutronManager, 'get_plugin') as gp:
-            plugin = mock.Mock()
-            gp.return_value = plugin
-            plugin.get_ports.return_value = ports
-            self.mixin.delete_floatingip_agent_gateway_port(
-                self.ctx, port_host, 'ext_network_id')
+        plugin = mock.Mock()
+        directory.add_plugin(const.CORE, plugin)
+        plugin.get_ports.return_value = ports
+        self.mixin.delete_floatingip_agent_gateway_port(
+            self.ctx, port_host, 'ext_network_id')
         plugin.get_ports.assert_called_with(self.ctx, filters={
             'network_id': ['ext_network_id'],
-            'device_owner': [l3_const.DEVICE_OWNER_AGENT_GW]})
+            'device_owner': [const.DEVICE_OWNER_AGENT_GW]})
         if port_host:
             plugin.ipam.delete_port.assert_called_once_with(
                 self.ctx, 'my_port_id')
@@ -296,23 +270,28 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         self._helper_delete_floatingip_agent_gateway_port(
             'foo_host')
 
-    def _setup_delete_current_gw_port_deletes_fip_agent_gw_port(
-        self, port=None, gw_port=True):
+    def _setup_delete_current_gw_port_deletes_dvr_internal_ports(
+        self, port=None, gw_port=True, new_network_id='ext_net_id_2'):
         router = mock.MagicMock()
         router.extra_attributes.distributed = True
         if gw_port:
             gw_port_db = {
                 'id': 'my_gw_id',
                 'network_id': 'ext_net_id',
-                'device_owner': l3_const.DEVICE_OWNER_ROUTER_GW
+                'device_owner': const.DEVICE_OWNER_ROUTER_GW,
+                'fixed_ips': [{'ip_address': '1.2.3.4'}]
             }
             router.gw_port = gw_port_db
         else:
             router.gw_port = None
 
-        with mock.patch.object(manager.NeutronManager, 'get_plugin') as gp,\
+        plugin = mock.Mock()
+        directory.add_plugin(const.CORE, plugin)
+        with mock.patch.object(l3_dvr_db.l3_db.L3_NAT_db_mixin,
+                               'router_gw_port_has_floating_ips',
+                               return_value=False),\
             mock.patch.object(l3_dvr_db.l3_db.L3_NAT_db_mixin,
-                              '_delete_current_gw_port'),\
+                              '_delete_router_gw_port_db'),\
             mock.patch.object(
                 self.mixin,
                 '_get_router') as grtr,\
@@ -325,22 +304,18 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             mock.patch.object(
                 self.mixin.l3_rpc_notifier,
                 'delete_fipnamespace_for_ext_net') as del_fip:
-            plugin = mock.Mock()
-            gp.return_value = plugin
             plugin.get_ports.return_value = port
             grtr.return_value = router
             self.mixin._delete_current_gw_port(
-                self.ctx, router['id'], router, 'ext_network_id')
+                self.ctx, router['id'], router, new_network_id)
             return router, plugin, del_csnat_port, del_agent_gw_port, del_fip
 
     def test_delete_current_gw_port_deletes_fip_agent_gw_port_and_fipnamespace(
             self):
         rtr, plugin, d_csnat_port, d_agent_gw_port, del_fip = (
-            self._setup_delete_current_gw_port_deletes_fip_agent_gw_port())
-        self.assertTrue(d_csnat_port.called)
+            self._setup_delete_current_gw_port_deletes_dvr_internal_ports())
+        self.assertFalse(d_csnat_port.called)
         self.assertTrue(d_agent_gw_port.called)
-        d_csnat_port.assert_called_once_with(
-            mock.ANY, rtr)
         d_agent_gw_port.assert_called_once_with(mock.ANY, None, 'ext_net_id')
         del_fip.assert_called_once_with(mock.ANY, 'ext_net_id')
 
@@ -348,30 +323,38 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         port = [{
             'id': 'my_port_id',
             'network_id': 'ext_net_id',
-            'device_owner': l3_const.DEVICE_OWNER_ROUTER_GW
+            'device_owner': const.DEVICE_OWNER_ROUTER_GW
         },
                 {
             'id': 'my_new_port_id',
             'network_id': 'ext_net_id',
-            'device_owner': l3_const.DEVICE_OWNER_ROUTER_GW
+            'device_owner': const.DEVICE_OWNER_ROUTER_GW
         }]
         rtr, plugin, d_csnat_port, d_agent_gw_port, del_fip = (
-            self._setup_delete_current_gw_port_deletes_fip_agent_gw_port(
+            self._setup_delete_current_gw_port_deletes_dvr_internal_ports(
                 port=port))
-        self.assertTrue(d_csnat_port.called)
+        self.assertFalse(d_csnat_port.called)
         self.assertFalse(d_agent_gw_port.called)
         self.assertFalse(del_fip.called)
-        d_csnat_port.assert_called_once_with(
-            mock.ANY, rtr)
 
     def test_delete_current_gw_port_never_calls_delete_fipnamespace(self):
         rtr, plugin, d_csnat_port, d_agent_gw_port, del_fip = (
-            self._setup_delete_current_gw_port_deletes_fip_agent_gw_port(
+            self._setup_delete_current_gw_port_deletes_dvr_internal_ports(
                 gw_port=False))
         self.assertFalse(d_csnat_port.called)
         self.assertFalse(d_agent_gw_port.called)
         self.assertFalse(del_fip.called)
 
+    def test_delete_current_gw_port_deletes_csnat_port(self):
+        rtr, plugin, d_csnat_port, d_agent_gw_port, del_fip = (
+            self._setup_delete_current_gw_port_deletes_dvr_internal_ports(
+                new_network_id=None))
+        self.assertTrue(d_csnat_port.called)
+        self.assertTrue(d_agent_gw_port.called)
+        d_csnat_port.assert_called_once_with(mock.ANY, rtr)
+        d_agent_gw_port.assert_called_once_with(mock.ANY, None, 'ext_net_id')
+        del_fip.assert_called_once_with(mock.ANY, 'ext_net_id')
+
     def _floatingip_on_port_test_setup(self, hostid):
         router = {'id': 'foo_router_id', 'distributed': True}
         floatingip = {
@@ -408,7 +391,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
     def test_floatingip_on_port_not_host(self):
         router, fip = self._floatingip_on_port_test_setup(None)
 
-        self.assertNotIn(l3_const.FLOATINGIP_KEY, router)
+        self.assertNotIn(const.FLOATINGIP_KEY, router)
         self.assertNotIn(n_const.FLOATINGIP_AGENT_INTF_KEY, router)
 
     def test_floatingip_on_port_with_host(self):
@@ -416,9 +399,9 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
 
         self.assertTrue(self.mixin._get_fip_sync_interfaces.called)
 
-        self.assertIn(l3_const.FLOATINGIP_KEY, router)
+        self.assertIn(const.FLOATINGIP_KEY, router)
         self.assertIn(n_const.FLOATINGIP_AGENT_INTF_KEY, router)
-        self.assertIn(fip, router[l3_const.FLOATINGIP_KEY])
+        self.assertIn(fip, router[const.FLOATINGIP_KEY])
         self.assertIn('fip_interface',
             router[n_const.FLOATINGIP_AGENT_INTF_KEY])
 
@@ -445,8 +428,11 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             grtr.return_value = router_db
             vmp.return_value = 'my-host'
             mvmp.return_value = 'my-future-host'
-            self.mixin._update_fip_assoc(
-                self.ctx, fip, floatingip_db, port)
+            registry.notify(resources.FLOATING_IP, events.AFTER_UPDATE, self,
+                            context=mock.Mock(), router_id=router_db['id'],
+                            fixed_port_id=port['id'], floating_ip_id=fip['id'],
+                            floating_network_id=fip['floating_network_id'],
+                            fixed_ip_address='1.2.3.4')
             return c_fip
 
     def test_create_floatingip_agent_gw_port_with_dvr_router(self):
@@ -457,6 +443,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         router = {'id': 'foo_router_id', 'distributed': True}
         fip = {
             'id': _uuid(),
+            'floating_network_id': _uuid(),
             'port_id': _uuid()
         }
         create_fip = (
@@ -472,6 +459,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         router = {'id': 'foo_router_id', 'distributed': False}
         fip = {
             'id': _uuid(),
+            'floating_network_id': _uuid(),
             'port_id': _uuid()
         }
         create_fip = (
@@ -479,6 +467,38 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                 fip, floatingip, router))
         self.assertFalse(create_fip.called)
 
+    def test_update_router_gw_info_external_network_change(self):
+        router_dict = {'name': 'test_router', 'admin_state_up': True,
+                       'distributed': True}
+        router = self._create_router(router_dict)
+        with self.network() as net_ext_1,\
+                self.network() as net_ext_2,\
+                self.subnet() as subnet:
+            ext_net_1_id = net_ext_1['network']['id']
+            self.core_plugin.update_network(
+                self.ctx, ext_net_1_id,
+                {'network': {'router:external': True}})
+            self.mixin.update_router(
+                self.ctx, router['id'],
+                {'router': {'external_gateway_info':
+                            {'network_id': ext_net_1_id}}})
+            self.mixin.add_router_interface(self.ctx, router['id'],
+                {'subnet_id': subnet['subnet']['id']})
+
+            ext_net_2_id = net_ext_2['network']['id']
+            self.core_plugin.update_network(
+                self.ctx, ext_net_2_id,
+                {'network': {'router:external': True}})
+            self.mixin.update_router(
+                self.ctx, router['id'],
+                {'router': {'external_gateway_info':
+                            {'network_id': ext_net_2_id}}})
+
+            csnat_filters = {'device_owner': [const.DEVICE_OWNER_ROUTER_SNAT]}
+            csnat_ports = self.core_plugin.get_ports(
+                self.ctx, filters=csnat_filters)
+            self.assertEqual(1, len(csnat_ports))
+
     def test_remove_router_interface_csnat_ports_removal(self):
         router_dict = {'name': 'test_router', 'admin_state_up': True,
                        'distributed': True}
@@ -501,24 +521,20 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                 {'subnet_id': subnet2['subnet']['id']})
 
             csnat_filters = {'device_owner':
-                             [l3_const.DEVICE_OWNER_ROUTER_SNAT]}
+                             [const.DEVICE_OWNER_ROUTER_SNAT]}
             csnat_ports = self.core_plugin.get_ports(
                 self.ctx, filters=csnat_filters)
             self.assertEqual(2, len(csnat_ports))
 
             dvr_filters = {'device_owner':
-                           [l3_const.DEVICE_OWNER_DVR_INTERFACE]}
+                           [const.DEVICE_OWNER_DVR_INTERFACE]}
             dvr_ports = self.core_plugin.get_ports(
                 self.ctx, filters=dvr_filters)
             self.assertEqual(2, len(dvr_ports))
 
-            with mock.patch.object(manager.NeutronManager,
-                                  'get_service_plugins') as get_svc_plugin:
-                get_svc_plugin.return_value = {
-                    plugin_const.L3_ROUTER_NAT: plugin}
-                self.mixin.manager = manager
-                self.mixin.remove_router_interface(
-                    self.ctx, router['id'], {'port_id': dvr_ports[0]['id']})
+            directory.add_plugin(const.L3, plugin)
+            self.mixin.remove_router_interface(
+                self.ctx, router['id'], {'port_id': dvr_ports[0]['id']})
 
             csnat_ports = self.core_plugin.get_ports(
                 self.ctx, filters=csnat_filters)
@@ -553,11 +569,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                     {'subnet_id': subnet_v4['subnet']['id']})
                 self.mixin.add_router_interface(self.ctx, router['id'],
                     {'subnet_id': subnet_v6['subnet']['id']})
-                get_svc_plugin = mock.patch.object(
-                    manager.NeutronManager, 'get_service_plugins').start()
-                get_svc_plugin.return_value = {
-                    plugin_const.L3_ROUTER_NAT: plugin}
-                self.mixin.manager = manager
+                directory.add_plugin(const.L3, plugin)
                 return router, subnet_v4, subnet_v6
 
     def test_undo_router_interface_change_on_csnat_error(self):
@@ -588,9 +600,8 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
 
         with self.subnet(network=net, cidr='fe81::/64',
                          gateway_ip='fe81::1', ip_version=6) as subnet2_v6:
-            with testtools.ExpectedException(RuntimeError):
-                self.mixin.add_router_interface(self.ctx, router['id'],
-                    {'subnet_id': subnet2_v6['subnet']['id']})
+            self.mixin.add_router_interface(self.ctx, router['id'],
+                {'subnet_id': subnet2_v6['subnet']['id']})
             if fail_revert:
                 # a revert failure will mean the interface is still added
                 # so we can't re-add it
@@ -602,12 +613,12 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
     def test_remove_router_interface_csnat_ports_removal_with_ipv6(self):
         router, subnet_v4, subnet_v6 = self._setup_router_with_v4_and_v6()
         csnat_filters = {'device_owner':
-                         [l3_const.DEVICE_OWNER_ROUTER_SNAT]}
+                         [const.DEVICE_OWNER_ROUTER_SNAT]}
         csnat_ports = self.core_plugin.get_ports(
             self.ctx, filters=csnat_filters)
         self.assertEqual(2, len(csnat_ports))
         dvr_filters = {'device_owner':
-                       [l3_const.DEVICE_OWNER_DVR_INTERFACE]}
+                       [const.DEVICE_OWNER_DVR_INTERFACE]}
         dvr_ports = self.core_plugin.get_ports(
             self.ctx, filters=dvr_filters)
         self.assertEqual(2, len(dvr_ports))
@@ -634,7 +645,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             self.ctx, router['id'],
             {'subnet_id': subnet_v4['subnet']['id']})
         csnat_filters = {'device_owner':
-                         [l3_const.DEVICE_OWNER_ROUTER_SNAT]}
+                         [const.DEVICE_OWNER_ROUTER_SNAT]}
         csnat_ports = self.core_plugin.get_ports(
             self.ctx, filters=csnat_filters)
         self.core_plugin.update_port(self.ctx, csnat_ports[0]['id'],
@@ -665,8 +676,9 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             interface_info = {'subnet_id': sub['subnet']['id']}
             self.mixin.add_router_interface(self.ctx, router_db.id,
                                             interface_info)
-            mock_notify.assert_called_once_with(self.ctx, router_db.id,
-                                                sub['subnet']['network_id'])
+            mock_notify.assert_called_once_with(self.ctx, router_db=router_db,
+                                                port=mock.ANY,
+                                                interface_info=interface_info)
 
     def test_validate_add_router_interface_by_port_notify_advanced_services(
         self):
@@ -681,54 +693,54 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             interface_info = {'port_id': port['port']['id']}
             self.mixin.add_router_interface(self.ctx, router_db.id,
                                             interface_info)
-            mock_notify.assert_called_once_with(self.ctx, router_db.id,
-                                                net['network']['id'])
+            mock_notify.assert_called_once_with(self.ctx, router_db=router_db,
+                                                port=mock.ANY,
+                                                interface_info=interface_info)
 
     def _test_update_arp_entry_for_dvr_service_port(
             self, device_owner, action):
         router_dict = {'name': 'test_router', 'admin_state_up': True,
                        'distributed': True}
         router = self._create_router(router_dict)
-        with mock.patch.object(manager.NeutronManager, 'get_plugin') as gp:
-            plugin = mock.Mock()
-            l3_notify = self.mixin.l3_rpc_notifier = mock.Mock()
-            gp.return_value = plugin
-            port = {
-                'id': 'my_port_id',
-                'fixed_ips': [
-                    {'subnet_id': '51edc9e0-24f9-47f2-8e1e-2a41cb691323',
-                     'ip_address': '10.0.0.11'},
-                    {'subnet_id': '2b7c8a07-6f8e-4937-8701-f1d5da1a807c',
-                     'ip_address': '10.0.0.21'},
-                    {'subnet_id': '48534187-f077-4e81-93ff-81ec4cc0ad3b',
-                     'ip_address': 'fd45:1515:7e0:0:f816:3eff:fe1a:1111'}],
-                'mac_address': 'my_mac',
-                'device_owner': device_owner
-            }
-            dvr_port = {
-                'id': 'dvr_port_id',
-                'fixed_ips': mock.ANY,
-                'device_owner': l3_const.DEVICE_OWNER_DVR_INTERFACE,
-                'device_id': router['id']
-            }
-            plugin.get_ports.return_value = [dvr_port]
-            if action == 'add':
-                self.mixin.update_arp_entry_for_dvr_service_port(
-                    self.ctx, port)
-                self.assertEqual(3, l3_notify.add_arp_entry.call_count)
-            elif action == 'del':
-                self.mixin.delete_arp_entry_for_dvr_service_port(
-                    self.ctx, port)
-                self.assertEqual(3, l3_notify.del_arp_entry.call_count)
+        plugin = mock.Mock()
+        directory.add_plugin(const.CORE, plugin)
+        l3_notify = self.mixin.l3_rpc_notifier = mock.Mock()
+        port = {
+            'id': 'my_port_id',
+            'fixed_ips': [
+                {'subnet_id': '51edc9e0-24f9-47f2-8e1e-2a41cb691323',
+                 'ip_address': '10.0.0.11'},
+                {'subnet_id': '2b7c8a07-6f8e-4937-8701-f1d5da1a807c',
+                 'ip_address': '10.0.0.21'},
+                {'subnet_id': '48534187-f077-4e81-93ff-81ec4cc0ad3b',
+                 'ip_address': 'fd45:1515:7e0:0:f816:3eff:fe1a:1111'}],
+            'mac_address': 'my_mac',
+            'device_owner': device_owner
+        }
+        dvr_port = {
+            'id': 'dvr_port_id',
+            'fixed_ips': mock.ANY,
+            'device_owner': const.DEVICE_OWNER_DVR_INTERFACE,
+            'device_id': router['id']
+        }
+        plugin.get_ports.return_value = [dvr_port]
+        if action == 'add':
+            self.mixin.update_arp_entry_for_dvr_service_port(
+                self.ctx, port)
+            self.assertEqual(3, l3_notify.add_arp_entry.call_count)
+        elif action == 'del':
+            self.mixin.delete_arp_entry_for_dvr_service_port(
+                self.ctx, port)
+            self.assertEqual(3, l3_notify.del_arp_entry.call_count)
 
     def test_update_arp_entry_for_dvr_service_port_added(self):
         action = 'add'
-        device_owner = l3_const.DEVICE_OWNER_LOADBALANCER
+        device_owner = const.DEVICE_OWNER_LOADBALANCER
         self._test_update_arp_entry_for_dvr_service_port(device_owner, action)
 
     def test_update_arp_entry_for_dvr_service_port_deleted(self):
         action = 'del'
-        device_owner = l3_const.DEVICE_OWNER_LOADBALANCER
+        device_owner = const.DEVICE_OWNER_LOADBALANCER
         self._test_update_arp_entry_for_dvr_service_port(device_owner, action)
 
     def test_add_router_interface_csnat_ports_failure(self):
@@ -749,7 +761,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                 self.mixin, '_add_csnat_router_interface_port') as f:
                 f.side_effect = RuntimeError()
                 self.assertRaises(
-                    RuntimeError,
+                    l3.RouterInterfaceAttachmentConflict,
                     self.mixin.add_router_interface,
                     self.ctx, router['id'],
                     {'subnet_id': subnet['subnet']['id']})
@@ -758,7 +770,7 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                 }
                 router_ports = self.core_plugin.get_ports(self.ctx, filters)
                 self.assertEqual(1, len(router_ports))
-                self.assertEqual(l3_const.DEVICE_OWNER_ROUTER_GW,
+                self.assertEqual(const.DEVICE_OWNER_ROUTER_GW,
                                  router_ports[0]['device_owner'])
 
     def test_add_router_interface_by_port_failure(self):
