@@ -14,6 +14,8 @@
 #    under the License.
 
 import mock
+from neutron_lib import constants
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_utils import uuidutils
 from webob import exc as web_exc
@@ -23,7 +25,6 @@ from neutron.api import extensions
 from neutron.api.v2 import router
 from neutron import context
 from neutron.extensions import providernet as pnet
-from neutron import manager
 from neutron import quota
 from neutron.tests import tools
 from neutron.tests.unit.api import test_extensions
@@ -60,17 +61,15 @@ class ProvidernetExtensionTestCase(testlib_api.WebTestCase):
         self.useFixture(tools.AttributeMapMemento())
 
         # Update the plugin and extensions path
-        self.setup_coreplugin(plugin)
-        cfg.CONF.set_override('allow_pagination', True)
-        cfg.CONF.set_override('allow_sorting', True)
+        self.setup_coreplugin(plugin, load_plugins=False)
         self._plugin_patcher = mock.patch(plugin, autospec=True)
         self.plugin = self._plugin_patcher.start()
         # Ensure Quota checks never fail because of mock
         instance = self.plugin.return_value
         instance.get_networks_count.return_value = 1
-        # Instantiate mock plugin and enable the 'provider' extension
-        manager.NeutronManager.get_plugin().supported_extension_aliases = (
-            ["provider"])
+        # Register mock plugin and enable the 'provider' extension
+        instance.supported_extension_aliases = ["provider"]
+        directory.add_plugin(constants.CORE, instance)
         ext_mgr = ProviderExtensionManager()
         self.ext_mdw = test_extensions.setup_extensions_middleware(ext_mgr)
         self.addCleanup(self._plugin_patcher.stop)
