@@ -15,10 +15,15 @@
 #    under the License.
 
 import mock
+from neutron_lib import constants
 from oslo_config import cfg
+import testtools
 
 from neutron.agent.linux import bridge_lib
-from neutron.common import constants
+from neutron.callbacks import events
+from neutron.callbacks import registry
+from neutron.callbacks import resources
+from neutron.common import constants as n_const
 from neutron.plugins.ml2.drivers.agent import _agent_manager_base as amb
 from neutron.plugins.ml2.drivers.agent import _common_agent as ca
 from neutron.tests import base
@@ -66,6 +71,36 @@ class TestCommonAgentLoop(base.BaseTestCase):
             with mock.patch.object(self.agent, "daemon_loop"):
                 self.agent.start()
 
+    def test_treat_devices_removed_notify(self):
+        handler = mock.Mock()
+        registry.subscribe(handler, resources.PORT_DEVICE, events.AFTER_DELETE)
+        devices = [DEVICE_1]
+        self.agent.treat_devices_removed(devices)
+        handler.assert_called_once_with(mock.ANY, mock.ANY, self.agent,
+                                        context=mock.ANY, device=DEVICE_1,
+                                        port_id=mock.ANY)
+
+    def test_treat_devices_added_updated_notify(self):
+        handler = mock.Mock()
+        registry.subscribe(handler, resources.PORT_DEVICE, events.AFTER_UPDATE)
+        agent = self.agent
+        mock_details = {'device': 'dev123',
+                        'port_id': 'port123',
+                        'network_id': 'net123',
+                        'admin_state_up': True,
+                        'network_type': 'vlan',
+                        'segmentation_id': 100,
+                        'physical_network': 'physnet1',
+                        'device_owner': 'horse'}
+        agent.plugin_rpc = mock.Mock()
+        agent.plugin_rpc.get_devices_details_list.return_value = [mock_details]
+        agent.mgr = mock.Mock()
+        agent.mgr.plug_interface.return_value = True
+        agent.treat_devices_added_updated(set(['dev123']))
+        handler.assert_called_once_with(mock.ANY, mock.ANY, self.agent,
+                                        context=mock.ANY,
+                                        device_details=mock_details)
+
     def test_treat_devices_removed_with_existed_device(self):
         agent = self.agent
         agent.mgr.ensure_port_admin_state = mock.Mock()
@@ -86,9 +121,7 @@ class TestCommonAgentLoop(base.BaseTestCase):
                 self.assertTrue(fn_udd.called)
                 self.assertTrue(fn_rdf.called)
                 self.assertTrue(ext_mgr_delete_port.called)
-                self.assertTrue(
-                    PORT_DATA not in agent.network_ports[NETWORK_ID]
-                )
+                self.assertNotIn(PORT_DATA, agent.network_ports[NETWORK_ID])
 
     def test_treat_devices_removed_with_not_existed_device(self):
         agent = self.agent
@@ -109,9 +142,7 @@ class TestCommonAgentLoop(base.BaseTestCase):
                 self.assertTrue(fn_udd.called)
                 self.assertTrue(fn_rdf.called)
                 self.assertTrue(ext_mgr_delete_port.called)
-                self.assertTrue(
-                    PORT_DATA not in agent.network_ports[NETWORK_ID]
-                )
+                self.assertNotIn(PORT_DATA, agent.network_ports[NETWORK_ID])
 
     def test_treat_devices_removed_failed(self):
         agent = self.agent
@@ -129,9 +160,7 @@ class TestCommonAgentLoop(base.BaseTestCase):
             self.assertTrue(fn_udd.called)
             self.assertTrue(fn_rdf.called)
             self.assertTrue(ext_mgr_delete_port.called)
-            self.assertTrue(
-                PORT_DATA not in agent.network_ports[NETWORK_ID]
-            )
+            self.assertNotIn(PORT_DATA, agent.network_ports[NETWORK_ID])
 
     def test_treat_devices_removed_with_prevent_arp_spoofing_true(self):
         agent = self.agent
@@ -423,9 +452,9 @@ class TestCommonAgentLoop(base.BaseTestCase):
                 constants.DEVICE_OWNER_NETWORK_PREFIX)
             self.assertTrue(agent.plugin_rpc.update_device_up.called)
             self.assertTrue(agent.ext_manager.handle_port.called)
-            self.assertTrue(mock_port_data in agent.network_ports[
+            self.assertIn(mock_port_data, agent.network_ports[
                 mock_details['network_id']]
-                            )
+                          )
 
     def test_treat_devices_added_updated_prevent_arp_spoofing_true(self):
         agent = self.agent
@@ -447,6 +476,36 @@ class TestCommonAgentLoop(base.BaseTestCase):
             agent.treat_devices_added_updated(set(['tap1']))
             set_arp.assert_called_with(mock_details['device'], mock_details)
 
+    def test__process_device_if_exists_missing_intf(self):
+        mock_details = {'device': 'dev123',
+                        'port_id': 'port123',
+                        'network_id': 'net123',
+                        'admin_state_up': True,
+                        'network_type': 'vlan',
+                        'segmentation_id': 100,
+                        'physical_network': 'physnet1',
+                        'device_owner': constants.DEVICE_OWNER_NETWORK_PREFIX}
+        self.agent.mgr = mock.Mock()
+        self.agent.mgr.get_all_devices.return_value = []
+        self.agent.mgr.plug_interface.side_effect = RuntimeError()
+        self.agent._process_device_if_exists(mock_details)
+
+    def test__process_device_if_exists_error(self):
+        mock_details = {'device': 'dev123',
+                        'port_id': 'port123',
+                        'network_id': 'net123',
+                        'admin_state_up': True,
+                        'network_type': 'vlan',
+                        'segmentation_id': 100,
+                        'physical_network': 'physnet1',
+                        'device_owner': constants.DEVICE_OWNER_NETWORK_PREFIX}
+        self.agent.mgr = mock.Mock()
+        self.agent.mgr.get_all_devices.return_value = ['dev123']
+        self.agent.mgr.plug_interface.side_effect = RuntimeError()
+        with testtools.ExpectedException(RuntimeError):
+            # device exists so it should raise
+            self.agent._process_device_if_exists(mock_details)
+
     def test_set_rpc_timeout(self):
         self.agent.stop()
         for rpc_client in (self.agent.plugin_rpc.client,
@@ -464,7 +523,7 @@ class TestCommonAgentLoop(base.BaseTestCase):
     def test_report_state_revived(self):
         with mock.patch.object(self.agent.state_rpc,
                                "report_state") as report_st:
-            report_st.return_value = constants.AGENT_REVIVED
+            report_st.return_value = n_const.AGENT_REVIVED
             self.agent._report_state()
             self.assertTrue(self.agent.fullsync)
 
@@ -485,12 +544,8 @@ class TestCommonAgentLoop(base.BaseTestCase):
         self.agent._update_network_ports(
             NETWORK_2_ID, port_2_data['port_id'], port_2_data['device']
         )
-        self.assertTrue(
-            port_2_data not in self.agent.network_ports[NETWORK_ID]
-        )
-        self.assertTrue(
-            port_2_data in self.agent.network_ports[NETWORK_2_ID]
-        )
+        self.assertNotIn(port_2_data, self.agent.network_ports[NETWORK_ID])
+        self.assertIn(port_2_data, self.agent.network_ports[NETWORK_2_ID])
 
     def test_clean_network_ports(self):
         port_1_data = PORT_DATA
@@ -506,21 +561,13 @@ class TestCommonAgentLoop(base.BaseTestCase):
         )
         #check removing port from network when other ports are still there:
         cleaned_port_id = self.agent._clean_network_ports(DEVICE_1)
-        self.assertTrue(
-            NETWORK_ID in self.agent.network_ports.keys()
-        )
-        self.assertTrue(
-            port_1_data not in self.agent.network_ports[NETWORK_ID]
-        )
-        self.assertTrue(
-            port_2_data in self.agent.network_ports[NETWORK_ID]
-        )
+        self.assertIn(NETWORK_ID, self.agent.network_ports.keys())
+        self.assertNotIn(port_1_data, self.agent.network_ports[NETWORK_ID])
+        self.assertIn(port_2_data, self.agent.network_ports[NETWORK_ID])
         self.assertEqual(PORT_1, cleaned_port_id)
         #and now remove last port from network:
         cleaned_port_id = self.agent._clean_network_ports(
             port_2_data['device']
         )
-        self.assertTrue(
-            NETWORK_ID not in self.agent.network_ports.keys()
-        )
+        self.assertNotIn(NETWORK_ID, self.agent.network_ports.keys())
         self.assertEqual(port_2_data['port_id'], cleaned_port_id)
