@@ -559,11 +559,7 @@ class TestLinuxBridgeManager(base.BaseTestCase):
                                                             "tap1",
                                                             dev_owner_prefix))
             with mock.patch.object(self.lbm,
-                                   "ensure_physical_in_bridge") as ens_fn,\
-                    mock.patch.object(self.lbm,
-                                      "ensure_tap_mtu") as en_mtu_fn,\
-                    mock.patch.object(bridge_lib.BridgeDevice,
-                                      "get_interface_bridge") as get_br:
+                                   "ensure_physical_in_bridge") as ens_fn:
                 ens_fn.return_value = False
                 self.assertFalse(self.lbm.add_tap_interface("123",
                                                             p_const.TYPE_VLAN,
@@ -571,14 +567,6 @@ class TestLinuxBridgeManager(base.BaseTestCase):
                                                             "tap1",
                                                             dev_owner_prefix))
 
-                ens_fn.return_value = "eth0.1"
-                get_br.return_value = "brq123"
-                self.lbm.add_tap_interface("123", p_const.TYPE_VLAN,
-                                           "physnet1", "1", "tap1",
-                                           dev_owner_prefix)
-                en_mtu_fn.assert_called_once_with("tap1", "eth0.1")
-                bridge_device.addif.assert_called_once_with("tap1")
-
     def test_add_tap_interface_owner_network(self):
         self._test_add_tap_interface(constants.DEVICE_OWNER_NETWORK_PREFIX)
 
@@ -639,7 +627,8 @@ class TestLinuxBridgeManager(base.BaseTestCase):
                                   "get_interface_details") as if_det_fn,\
                 mock.patch.object(self.lbm,
                                   "update_interface_ip_details") as updif_fn,\
-                mock.patch.object(self.lbm, "delete_interface") as del_interface,\
+                mock.patch.object(self.lbm,
+                                  "delete_interface") as del_interface,\
                 mock.patch.object(bridge_lib, "BridgeDevice",
                                   return_value=bridge_device):
             de_fn.return_value = True
@@ -658,7 +647,8 @@ class TestLinuxBridgeManager(base.BaseTestCase):
                                   "get_interface_details") as if_det_fn,\
                 mock.patch.object(self.lbm,
                                   "update_interface_ip_details") as updif_fn,\
-                mock.patch.object(self.lbm, "delete_interface") as del_interface,\
+                mock.patch.object(self.lbm,
+                                  "delete_interface") as del_interface,\
                 mock.patch.object(bridge_lib, "BridgeDevice",
                                   return_value=bridge_device):
             de_fn.return_value = True
@@ -694,7 +684,8 @@ class TestLinuxBridgeManager(base.BaseTestCase):
         bridge_device = mock.Mock()
         with mock.patch.object(ip_lib, "device_exists") as de_fn,\
                 mock.patch.object(self.lbm, "remove_interface"),\
-                mock.patch.object(self.lbm, "get_interface_details") as if_det_fn,\
+                mock.patch.object(self.lbm,
+                                  "get_interface_details") as if_det_fn,\
                 mock.patch.object(self.lbm, "delete_interface") as del_int,\
                 mock.patch.object(bridge_lib, "BridgeDevice",
                                   return_value=bridge_device):
@@ -739,7 +730,8 @@ class TestLinuxBridgeManager(base.BaseTestCase):
 
     def test_delete_interface(self):
         with mock.patch.object(ip_lib.IPDevice, "exists") as de_fn,\
-                mock.patch.object(ip_lib.IpLinkCommand, "set_down") as down_fn,\
+                mock.patch.object(ip_lib.IpLinkCommand,
+                                  "set_down") as down_fn,\
                 mock.patch.object(ip_lib.IpLinkCommand, "delete") as delete_fn:
             de_fn.return_value = False
             self.lbm.delete_interface("eth1.1")
@@ -857,8 +849,8 @@ class TestLinuxBridgeManager(base.BaseTestCase):
 
     def test_get_agent_id_bridge_mappings(self):
         lbm = get_linuxbridge_manager(BRIDGE_MAPPINGS, INTERFACE_MAPPINGS)
-        with mock.patch.object(utils,
-                               "get_interface_mac",
+        with mock.patch.object(ip_lib,
+                               "get_device_mac",
                                return_value='16:63:69:10:a0:59') as mock_gim:
 
             agent_id = lbm.get_agent_id()
@@ -878,13 +870,13 @@ class TestLinuxBridgeManager(base.BaseTestCase):
                               'get_devices',
                               return_value=devices_mock), \
                 mock.patch.object(
-                    utils,
-                    "get_interface_mac",
-                    side_effect=[None, '16:63:69:10:a0:59']) as mock_gim:
+                    ip_lib,
+                    "get_device_mac",
+                    return_value='16:63:69:10:a0:59') as mock_gim:
 
             agent_id = lbm.get_agent_id()
             self.assertEqual("lb16636910a059", agent_id)
-            mock_gim.assert_has_calls([mock.call("eth1"), mock.call("eth2")])
+            mock_gim.assert_called_with("eth1")
 
 
 class TestLinuxBridgeRpcCallbacks(base.BaseTestCase):
@@ -962,7 +954,7 @@ class TestLinuxBridgeRpcCallbacks(base.BaseTestCase):
 
         with mock.patch.object(utils, 'execute',
                                return_value='') as execute_fn, \
-                mock.patch.object(ip_lib.IpNeighCommand, 'add',
+                mock.patch.object(ip_lib, 'add_neigh_entry',
                                   return_value='') as add_fn:
             self.lb_rpc.fdb_add(None, fdb_entries)
 
@@ -981,7 +973,7 @@ class TestLinuxBridgeRpcCallbacks(base.BaseTestCase):
             ]
             execute_fn.assert_has_calls(expected)
             if proxy_enabled:
-                add_fn.assert_called_with('port_ip', 'port_mac')
+                add_fn.assert_called_with('port_ip', 'port_mac', 'vxlan-1')
             else:
                 add_fn.assert_not_called()
 
@@ -1031,7 +1023,7 @@ class TestLinuxBridgeRpcCallbacks(base.BaseTestCase):
 
         with mock.patch.object(utils, 'execute',
                                return_value='') as execute_fn, \
-                mock.patch.object(ip_lib.IpNeighCommand, 'delete',
+                mock.patch.object(ip_lib, 'delete_neigh_entry',
                                   return_value='') as del_fn:
             self.lb_rpc.fdb_remove(None, fdb_entries)
 
@@ -1048,7 +1040,7 @@ class TestLinuxBridgeRpcCallbacks(base.BaseTestCase):
             ]
             execute_fn.assert_has_calls(expected)
             if proxy_enabled:
-                del_fn.assert_called_with('port_ip', 'port_mac')
+                del_fn.assert_called_with('port_ip', 'port_mac', 'vxlan-1')
             else:
                 del_fn.assert_not_called()
 
@@ -1066,15 +1058,15 @@ class TestLinuxBridgeRpcCallbacks(base.BaseTestCase):
                          {'before': [['port_mac', 'port_ip_1']],
                           'after': [['port_mac', 'port_ip_2']]}}}}
 
-        with mock.patch.object(ip_lib.IpNeighCommand, 'add',
+        with mock.patch.object(ip_lib, 'add_neigh_entry',
                                return_value='') as add_fn, \
-                mock.patch.object(ip_lib.IpNeighCommand, 'delete',
+                mock.patch.object(ip_lib, 'delete_neigh_entry',
                                   return_value='') as del_fn:
             self.lb_rpc.fdb_update(None, fdb_entries)
 
             if proxy_enabled:
-                del_fn.assert_called_with('port_ip_1', 'port_mac')
-                add_fn.assert_called_with('port_ip_2', 'port_mac')
+                del_fn.assert_called_with('port_ip_1', 'port_mac', 'vxlan-1')
+                add_fn.assert_called_with('port_ip_2', 'port_mac', 'vxlan-1')
             else:
                 del_fn.assert_not_called()
                 add_fn.assert_not_called()
