@@ -18,8 +18,12 @@ from oslo_utils import uuidutils
 from neutron import context
 from neutron.objects.qos import policy
 from neutron.objects.qos import rule
+from neutron.plugins.ml2.drivers.openvswitch.agent import (
+        ovs_agent_extension_api as ovs_ext_api)
 from neutron.plugins.ml2.drivers.openvswitch.agent.extension_drivers import (
     qos_driver)
+from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.ovs_ofctl import (
+    ovs_bridge)
 from neutron.tests.unit.plugins.ml2.drivers.openvswitch.agent import (
     ovs_test_base)
 
@@ -28,19 +32,29 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
 
     def setUp(self):
         super(QosOVSAgentDriverTestCase, self).setUp()
+        conn_patcher = mock.patch(
+            'neutron.agent.ovsdb.native.connection.Connection.start')
+        conn_patcher.start()
+        self.addCleanup(conn_patcher.stop)
         self.context = context.get_admin_context()
         self.qos_driver = qos_driver.QosOVSAgentDriver()
+        self.agent_api = ovs_ext_api.OVSAgentExtensionAPI(
+                         ovs_bridge.OVSAgentBridge('br-int'),
+                         ovs_bridge.OVSAgentBridge('br-tun'))
+        self.qos_driver.consume_api(self.agent_api)
         self.qos_driver.initialize()
         self.qos_driver.br_int = mock.Mock()
         self.qos_driver.br_int.get_egress_bw_limit_for_port = mock.Mock(
             return_value=(1000, 10))
+        self.qos_driver.br_int.dump_flows_for = mock.Mock(return_value=None)
         self.get = self.qos_driver.br_int.get_egress_bw_limit_for_port
         self.qos_driver.br_int.del_egress_bw_limit_for_port = mock.Mock()
         self.delete = self.qos_driver.br_int.delete_egress_bw_limit_for_port
         self.qos_driver.br_int.create_egress_bw_limit_for_port = mock.Mock()
         self.create = self.qos_driver.br_int.create_egress_bw_limit_for_port
-        self.rule = self._create_bw_limit_rule_obj()
-        self.qos_policy = self._create_qos_policy_obj([self.rule])
+        self.rules = [self._create_bw_limit_rule_obj(),
+                     self._create_dscp_marking_rule_obj()]
+        self.qos_policy = self._create_qos_policy_obj(self.rules)
         self.port = self._create_fake_port(self.qos_policy.id)
 
     def _create_bw_limit_rule_obj(self):
@@ -51,9 +65,16 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
         rule_obj.obj_reset_changes()
         return rule_obj
 
+    def _create_dscp_marking_rule_obj(self):
+        rule_obj = rule.QosDscpMarkingRule()
+        rule_obj.id = uuidutils.generate_uuid()
+        rule_obj.dscp_mark = 32
+        rule_obj.obj_reset_changes()
+        return rule_obj
+
     def _create_qos_policy_obj(self, rules):
         policy_dict = {'id': uuidutils.generate_uuid(),
-                'tenant_id': uuidutils.generate_uuid(),
+                'project_id': uuidutils.generate_uuid(),
                 'name': 'test',
                 'description': 'test',
                 'shared': False,
@@ -74,6 +95,7 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
         return {'vif_port': FakeVifPort(),
                 'qos_policy_id': policy_id,
                 'network_qos_policy_id': None,
+                'port_id': uuidutils.generate_uuid(),
                 'device_owner': uuidutils.generate_uuid()}
 
     def test_create_new_rule(self):
@@ -83,19 +105,22 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
         # Assert create is the last call
         self.assertEqual(
             'create_egress_bw_limit_for_port',
-            self.qos_driver.br_int.method_calls[-1][0])
+            self.qos_driver.br_int.method_calls[0][0])
         self.assertEqual(0, self.delete.call_count)
         self.create.assert_called_once_with(
-            self.port_name, self.rule.max_kbps,
-            self.rule.max_burst_kbps)
+            self.port_name, self.rules[0].max_kbps,
+            self.rules[0].max_burst_kbps)
+        self._assert_dscp_rule_create_updated()
 
     def test_create_existing_rules(self):
         self.qos_driver.create(self.port, self.qos_policy)
-        self._assert_rule_create_updated()
+        self._assert_bw_rule_create_updated()
+        self._assert_dscp_rule_create_updated()
 
     def test_update_rules(self):
         self.qos_driver.update(self.port, self.qos_policy)
-        self._assert_rule_create_updated()
+        self._assert_bw_rule_create_updated()
+        self._assert_dscp_rule_create_updated()
 
     def test_update_rules_no_vif_port(self):
         port = copy.copy(self.port)
@@ -113,12 +138,22 @@ class QosOVSAgentDriverTestCase(ovs_test_base.OVSAgentConfigTestBase):
         self.qos_driver.delete(port, self.qos_policy)
         self.delete.assert_not_called()
 
-    def _assert_rule_create_updated(self):
+    def _assert_bw_rule_create_updated(self):
         # Assert create is the last call
         self.assertEqual(
             'create_egress_bw_limit_for_port',
-            self.qos_driver.br_int.method_calls[-1][0])
+            self.qos_driver.br_int.method_calls[0][0])
 
         self.create.assert_called_once_with(
-            self.port_name, self.rule.max_kbps,
-            self.rule.max_burst_kbps)
+            self.port_name, self.rules[0].max_kbps,
+            self.rules[0].max_burst_kbps)
+
+    def _assert_dscp_rule_create_updated(self):
+        # Assert add_flow is the last call
+        self.assertEqual(
+            'add_flow',
+            self.qos_driver.br_int.method_calls[-1][0])
+
+        self.qos_driver.br_int.add_flow.assert_called_once_with(
+            actions='mod_nw_tos:128,load:55->NXM_NX_REG2[0..5],resubmit(,0)',
+            in_port=mock.ANY, priority=65535, reg2=0, table=0)
