@@ -16,27 +16,34 @@
 import warnings
 
 import mock
+import netaddr
 from neutron_lib import constants
 from oslo_utils import uuidutils
+from sqlalchemy.orm import exc
 from sqlalchemy.orm import query
 
 from neutron import context
 from neutron.db import db_base_plugin_v2
-from neutron.db import l3_db
+from neutron.db.models import l3 as l3_models
 from neutron.db import models_v2
 from neutron.db import segments_db
 from neutron.extensions import portbindings
+from neutron.objects import ports as port_obj
 from neutron.plugins.ml2 import db as ml2_db
 from neutron.plugins.ml2 import driver_api as api
 from neutron.plugins.ml2 import models
 from neutron.tests.unit import testlib_api
 
 
+PLUGIN_NAME = 'ml2'
+
+
 class Ml2DBTestCase(testlib_api.SqlTestCase):
 
     def setUp(self):
         super(Ml2DBTestCase, self).setUp()
         self.ctx = context.get_admin_context()
+        self.setup_coreplugin(PLUGIN_NAME)
 
     def _setup_neutron_network(self, network_id):
         with self.ctx.session.begin(subtransactions=True):
@@ -44,15 +51,15 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
 
     def _setup_neutron_port(self, network_id, port_id):
         mac_address = db_base_plugin_v2.NeutronDbPluginV2._generate_mac()
-        with self.ctx.session.begin(subtransactions=True):
-            port = models_v2.Port(id=port_id,
-                                  network_id=network_id,
-                                  mac_address=mac_address,
-                                  admin_state_up=True,
-                                  status='DOWN',
-                                  device_id='',
-                                  device_owner='')
-            self.ctx.session.add(port)
+        port = port_obj.Port(self.ctx,
+                             id=port_id,
+                             network_id=network_id,
+                             mac_address=netaddr.EUI(mac_address),
+                             admin_state_up=True,
+                             status='DOWN',
+                             device_id='',
+                             device_owner='')
+        port.create()
         return port
 
     def _setup_neutron_portbinding(self, port_id, vif_type, host):
@@ -74,7 +81,7 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
                 is_dynamic=is_seg_dynamic)
 
         net_segments = segments_db.get_network_segments(
-                           self.ctx.session, network_id,
+                           self.ctx, network_id,
                            filter_dynamic=is_seg_dynamic)
         net_segments = self._sort_segments(net_segments)
 
@@ -120,7 +127,7 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
         net1segs = self._create_segments(segments1, network_id='net1')
         net2segs = self._create_segments(segments2, network_id='net2')
         segs = segments_db.get_networks_segments(
-            self.ctx.session, ['net1', 'net2'])
+            self.ctx, ['net1', 'net2'])
         self.assertEqual(net1segs, self._sort_segments(segs['net1']))
         self.assertEqual(net2segs, self._sort_segments(segs['net2']))
 
@@ -128,7 +135,7 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
         self._create_segments([], network_id='net1')
         self._create_segments([], network_id='net2')
         segs = segments_db.get_networks_segments(
-            self.ctx.session, ['net1', 'net2'])
+            self.ctx, ['net1', 'net2'])
         self.assertEqual([], segs['net1'])
         self.assertEqual([], segs['net2'])
 
@@ -140,13 +147,13 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
         net_segment = self._create_segments([segment])[0]
         segment_uuid = net_segment[api.ID]
 
-        net_segment = segments_db.get_segment_by_id(self.ctx.session,
+        net_segment = segments_db.get_segment_by_id(self.ctx,
                                                     segment_uuid)
         self.assertEqual(segment, net_segment)
 
     def test_get_segment_by_id_result_not_found(self):
         segment_uuid = uuidutils.generate_uuid()
-        net_segment = segments_db.get_segment_by_id(self.ctx.session,
+        net_segment = segments_db.get_segment_by_id(self.ctx,
                                                     segment_uuid)
         self.assertIsNone(net_segment)
 
@@ -158,66 +165,43 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
         net_segment = self._create_segments([segment])[0]
         segment_uuid = net_segment[api.ID]
 
-        segments_db.delete_network_segment(self.ctx.session, segment_uuid)
+        segments_db.delete_network_segment(self.ctx, segment_uuid)
         # Get segment and verify its empty
-        net_segment = segments_db.get_segment_by_id(self.ctx.session,
+        net_segment = segments_db.get_segment_by_id(self.ctx,
                                                     segment_uuid)
         self.assertIsNone(net_segment)
 
-    def test_get_dynamic_segment(self):
-        net_id = uuidutils.generate_uuid()
-        segment1 = {api.NETWORK_TYPE: 'vlan',
-                    api.PHYSICAL_NETWORK: 'physnet1',
-                    api.SEGMENTATION_ID: 1}
-
-        self._create_segments(
-            [segment1], is_seg_dynamic=True, network_id=net_id)
-
-        segs1 = segments_db.get_dynamic_segment(
-            self.ctx.session, net_id)
-        self.assertEqual('vlan', segs1[api.NETWORK_TYPE])
-        self.assertEqual('physnet1', segs1[api.PHYSICAL_NETWORK])
-        self.assertEqual(1, segs1[api.SEGMENTATION_ID])
-
-        segs2 = segments_db.get_dynamic_segment(
-            self.ctx.session, net_id, physical_network='physnet1')
-        self.assertEqual('vlan', segs2[api.NETWORK_TYPE])
-        self.assertEqual('physnet1', segs2[api.PHYSICAL_NETWORK])
-        self.assertEqual(1, segs2[api.SEGMENTATION_ID])
-
-        segs3 = segments_db.get_dynamic_segment(
-            self.ctx.session, net_id, segmentation_id=1)
-        self.assertEqual('vlan', segs3[api.NETWORK_TYPE])
-        self.assertEqual('physnet1', segs3[api.PHYSICAL_NETWORK])
-        self.assertEqual(1, segs3[api.SEGMENTATION_ID])
-
     def test_add_port_binding(self):
-        network_id = 'foo-network-id'
-        port_id = 'foo-port-id'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
         self._setup_neutron_network(network_id)
         self._setup_neutron_port(network_id, port_id)
 
-        port = ml2_db.add_port_binding(self.ctx.session, port_id)
+        port = ml2_db.add_port_binding(self.ctx, port_id)
         self.assertEqual(port_id, port.port_id)
         self.assertEqual(portbindings.VIF_TYPE_UNBOUND, port.vif_type)
 
     def test_get_port_binding_host(self):
-        network_id = 'foo-network-id'
-        port_id = 'foo-port-id'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
         host = 'fake_host'
         vif_type = portbindings.VIF_TYPE_UNBOUND
         self._setup_neutron_network(network_id)
         self._setup_neutron_port(network_id, port_id)
         self._setup_neutron_portbinding(port_id, vif_type, host)
 
-        port_host = ml2_db.get_port_binding_host(self.ctx.session, port_id)
+        port_host = ml2_db.get_port_binding_host(self.ctx, port_id)
         self.assertEqual(host, port_host)
 
     def test_get_port_binding_host_multiple_results_found(self):
-        network_id = 'foo-network-id'
-        port_id = 'foo-port-id'
-        port_id_one = 'foo-port-id-one'
-        port_id_two = 'foo-port-id-two'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
+        port_id_one = uuidutils.generate_uuid()
+        port_id_two = uuidutils.generate_uuid()
+        # NOTE(manjeets) to check startswith testcase we
+        # need port ids with same prefix
+        port_id_one = port_id[:8] + port_id_one[8:]
+        port_id_two = port_id[:8] + port_id_two[8:]
         host = 'fake_host'
         vif_type = portbindings.VIF_TYPE_UNBOUND
         self._setup_neutron_network(network_id)
@@ -226,44 +210,39 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
         self._setup_neutron_port(network_id, port_id_two)
         self._setup_neutron_portbinding(port_id_two, vif_type, host)
 
-        port_host = ml2_db.get_port_binding_host(self.ctx.session, port_id)
+        port_host = ml2_db.get_port_binding_host(self.ctx, port_id[:8])
         self.assertIsNone(port_host)
 
     def test_get_port_binding_host_result_not_found(self):
         port_id = uuidutils.generate_uuid()
 
-        port_host = ml2_db.get_port_binding_host(self.ctx.session, port_id)
+        port_host = ml2_db.get_port_binding_host(self.ctx, port_id)
         self.assertIsNone(port_host)
 
     def test_get_port(self):
-        network_id = 'foo-network-id'
-        port_id = 'foo-port-id'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
         self._setup_neutron_network(network_id)
         self._setup_neutron_port(network_id, port_id)
 
-        port = ml2_db.get_port(self.ctx.session, port_id)
+        port = ml2_db.get_port(self.ctx, port_id)
         self.assertEqual(port_id, port.id)
 
     def test_get_port_multiple_results_found(self):
-        network_id = 'foo-network-id'
-        port_id = 'foo-port-id'
-        port_id_one = 'foo-port-id-one'
-        port_id_two = 'foo-port-id-two'
-        self._setup_neutron_network(network_id)
-        self._setup_neutron_port(network_id, port_id_one)
-        self._setup_neutron_port(network_id, port_id_two)
-
-        port = ml2_db.get_port(self.ctx.session, port_id)
+        with mock.patch(
+                'sqlalchemy.orm.query.Query.one',
+                side_effect=exc.MultipleResultsFound):
+            port = ml2_db.get_port(self.ctx, 'unused')
         self.assertIsNone(port)
 
     def test_get_port_result_not_found(self):
         port_id = uuidutils.generate_uuid()
-        port = ml2_db.get_port(self.ctx.session, port_id)
+        port = ml2_db.get_port(self.ctx, port_id)
         self.assertIsNone(port)
 
     def test_get_port_from_device_mac(self):
-        network_id = 'foo-network-id'
-        port_id = 'foo-port-id'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
         self._setup_neutron_network(network_id)
         port = self._setup_neutron_port(network_id, port_id)
 
@@ -272,15 +251,15 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
         self.assertEqual(port_id, observed_port.id)
 
     def test_get_locked_port_and_binding(self):
-        network_id = 'foo-network-id'
-        port_id = 'foo-port-id'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
         host = 'fake_host'
         vif_type = portbindings.VIF_TYPE_UNBOUND
         self._setup_neutron_network(network_id)
         self._setup_neutron_port(network_id, port_id)
         self._setup_neutron_portbinding(port_id, vif_type, host)
 
-        port, binding = ml2_db.get_locked_port_and_binding(self.ctx.session,
+        port, binding = ml2_db.get_locked_port_and_binding(self.ctx,
                                                            port_id)
         self.assertEqual(port_id, port.id)
         self.assertEqual(port_id, binding.port_id)
@@ -288,7 +267,7 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
     def test_get_locked_port_and_binding_result_not_found(self):
         port_id = uuidutils.generate_uuid()
 
-        port, binding = ml2_db.get_locked_port_and_binding(self.ctx.session,
+        port, binding = ml2_db.get_locked_port_and_binding(self.ctx,
                                                            port_id)
         self.assertIsNone(port)
         self.assertIsNone(binding)
@@ -299,6 +278,7 @@ class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
     def setUp(self):
         super(Ml2DvrDBTestCase, self).setUp()
         self.ctx = context.get_admin_context()
+        self.setup_coreplugin(PLUGIN_NAME)
 
     def _setup_neutron_network(self, network_id, port_ids):
         with self.ctx.session.begin(subtransactions=True):
@@ -307,20 +287,21 @@ class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
             for port_id in port_ids:
                 mac_address = (db_base_plugin_v2.NeutronDbPluginV2.
                                _generate_mac())
-                port = models_v2.Port(id=port_id,
-                                      network_id=network_id,
-                                      mac_address=mac_address,
-                                      admin_state_up=True,
-                                      status='ACTIVE',
-                                      device_id='',
-                                      device_owner='')
-                self.ctx.session.add(port)
+                port = port_obj.Port(self.ctx,
+                                     id=port_id,
+                                     network_id=network_id,
+                                     mac_address=netaddr.EUI(mac_address),
+                                     admin_state_up=True,
+                                     status='ACTIVE',
+                                     device_id='',
+                                     device_owner='')
+                port.create()
                 ports.append(port)
             return ports
 
     def _setup_neutron_router(self):
         with self.ctx.session.begin(subtransactions=True):
-            router = l3_db.Router()
+            router = l3_models.Router()
             self.ctx.session.add(router)
             return router
 
@@ -338,8 +319,8 @@ class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
             return record
 
     def test_ensure_distributed_port_binding_deals_with_db_duplicate(self):
-        network_id = 'foo_network_id'
-        port_id = 'foo_port_id'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
         router_id = 'foo_router_id'
         host_id = 'foo_host_id'
         self._setup_neutron_network(network_id, [port_id])
@@ -349,43 +330,43 @@ class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
             query_first.return_value = []
             with mock.patch.object(ml2_db.LOG, 'debug') as log_trace:
                 binding = ml2_db.ensure_distributed_port_binding(
-                    self.ctx.session, port_id, host_id, router_id)
+                    self.ctx, port_id, host_id, router_id)
         self.assertTrue(query_first.called)
         self.assertTrue(log_trace.called)
         self.assertEqual(port_id, binding.port_id)
 
     def test_ensure_distributed_port_binding(self):
-        network_id = 'foo_network_id'
-        port_id = 'foo_port_id'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
         self._setup_neutron_network(network_id, [port_id])
         router = self._setup_neutron_router()
         ml2_db.ensure_distributed_port_binding(
-            self.ctx.session, port_id, 'foo_host', router.id)
+            self.ctx, port_id, 'foo_host', router.id)
         expected = (self.ctx.session.query(models.DistributedPortBinding).
                     filter_by(port_id=port_id).one())
         self.assertEqual(port_id, expected.port_id)
 
     def test_ensure_distributed_port_binding_multiple_bindings(self):
-        network_id = 'foo_network_id'
-        port_id = 'foo_port_id'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
         self._setup_neutron_network(network_id, [port_id])
         router = self._setup_neutron_router()
         ml2_db.ensure_distributed_port_binding(
-            self.ctx.session, port_id, 'foo_host_1', router.id)
+            self.ctx, port_id, 'foo_host_1', router.id)
         ml2_db.ensure_distributed_port_binding(
-            self.ctx.session, port_id, 'foo_host_2', router.id)
+            self.ctx, port_id, 'foo_host_2', router.id)
         bindings = (self.ctx.session.query(models.DistributedPortBinding).
                     filter_by(port_id=port_id).all())
         self.assertEqual(2, len(bindings))
 
     def test_delete_distributed_port_binding_if_stale(self):
-        network_id = 'foo_network_id'
-        port_id = 'foo_port_id'
+        network_id = uuidutils.generate_uuid()
+        port_id = uuidutils.generate_uuid()
         self._setup_neutron_network(network_id, [port_id])
         binding = self._setup_distributed_binding(
             network_id, port_id, None, 'foo_host_id')
 
-        ml2_db.delete_distributed_port_binding_if_stale(self.ctx.session,
+        ml2_db.delete_distributed_port_binding_if_stale(self.ctx,
                                                         binding)
         count = (self.ctx.session.query(models.DistributedPortBinding).
             filter_by(port_id=binding.port_id).count())
@@ -393,26 +374,26 @@ class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
 
     def test_get_distributed_port_binding_by_host_not_found(self):
         port = ml2_db.get_distributed_port_binding_by_host(
-            self.ctx.session, 'foo_port_id', 'foo_host_id')
+            self.ctx, 'foo_port_id', 'foo_host_id')
         self.assertIsNone(port)
 
     def test_get_distributed_port_bindings_not_found(self):
-        port = ml2_db.get_distributed_port_bindings(self.ctx.session,
+        port = ml2_db.get_distributed_port_bindings(self.ctx,
                                                     'foo_port_id')
         self.assertFalse(len(port))
 
     def test_get_distributed_port_bindings(self):
-        network_id = 'foo_network_id'
-        port_id_1 = 'foo_port_id_1'
-        port_id_2 = 'foo_port_id_2'
+        network_id = uuidutils.generate_uuid()
+        port_id_1 = uuidutils.generate_uuid()
+        port_id_2 = uuidutils.generate_uuid()
         self._setup_neutron_network(network_id, [port_id_1, port_id_2])
         router = self._setup_neutron_router()
         self._setup_distributed_binding(
             network_id, port_id_1, router.id, 'foo_host_id_1')
         self._setup_distributed_binding(
             network_id, port_id_1, router.id, 'foo_host_id_2')
-        ports = ml2_db.get_distributed_port_bindings(self.ctx.session,
-                                                     'foo_port_id')
+        ports = ml2_db.get_distributed_port_bindings(self.ctx,
+                                                     port_id_1)
         self.assertEqual(2, len(ports))
 
     def test_distributed_port_binding_deleted_by_port_deletion(self):
@@ -445,6 +426,6 @@ class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
             with self.ctx.session.begin(subtransactions=True):
                 self.ctx.session.delete(port)
             self.assertEqual([], warning_list)
-        ports = ml2_db.get_distributed_port_bindings(self.ctx.session,
+        ports = ml2_db.get_distributed_port_bindings(self.ctx,
                                                      'port_id')
         self.assertEqual(0, len(ports))
